@@ -1,12 +1,12 @@
 /*
-   Time-stamp: <96/08/04 19:27:32 yusuf>
+   Time-stamp: <98/03/05 09:23:29 yusuf>
 
-   $Id: backup.c,v 1.30 1996/08/05 19:01:59 yusuf Exp $	
+   $Id: backup.c,v 1.63 1998/03/05 01:29:38 yusuf Exp $	
 
 */
 
 #ifndef lint
-static char vcid[] = "$Id: backup.c,v 1.30 1996/08/05 19:01:59 yusuf Exp $";
+static char vcid[] = "$Id: backup.c,v 1.63 1998/03/05 01:29:38 yusuf Exp $";
 #endif /* lint */
 
 
@@ -14,7 +14,7 @@ static char vcid[] = "$Id: backup.c,v 1.30 1996/08/05 19:01:59 yusuf Exp $";
 
 /* Tape structure
  * 
- * Taper header (at beginning of each tape)
+ * Taper header (at beginning of each tape)   
  * ----------------------------------------
  * long taper_magic number  (identifies taper backups  )
  * long archive ID (unqiue # that IDs archives         ) 
@@ -35,7 +35,7 @@ static char vcid[] = "$Id: backup.c,v 1.30 1996/08/05 19:01:59 yusuf Exp $";
  *               :    S_ISLNK   the name of the symbolic link (# chars first)
  * 
  * 
- * File info block:
+ * File info block:   52 bytes
  * 
  *   long   actual size  with uncompressed files, = size
  *   volume           
@@ -60,7 +60,6 @@ static char vcid[] = "$Id: backup.c,v 1.30 1996/08/05 19:01:59 yusuf Exp $";
  * long magic
  * long archive_id                        )
  * int number_tapes                       )   in structure
- * long info_file_size                    )
  * int number_volumes                     )
  * long size_of_volume_headers
  * long no_in_archive                     ) 
@@ -102,22 +101,16 @@ void backup_stop_child(void)
 {
 /* Stops child from compressing and deletes whatever files
  * are left in temporary buffers */
-    char s[MAX_FNAME];
+    char *fn, *cf;
     
     if (backup_child) {
 	kill(backup_child, SIGTERM);		 /* tell child to stop compressing */
 	waitpid(backup_child, NULL, 0);
 	backup_child = 0;
     }
-    while (!feof(fdfifo)) {
-	if (fgets(s, sizeof(s), fdfifo) == NULL) /* original filename */
-	  break;
-	if (fgets(s, sizeof(s), fdfifo) == NULL) /* compressed name */
-	  break;
-	if (s[strlen(s)-1] == '\n')		 /* remove trailing \n */
-	  s[strlen(s)-1] = 0;
-	unlink(s);				 /* delete it */
-    }
+    while (fifo_receive_file (fdfifo, &fn, &cf) && fn != NULL)
+	if (fn != fifo_disk_full && cf != NULL)
+		unlink(cf);			 /* delete compressed file */
     fclose(fdfifo);				 /* close FIFO */
     fdfifo = NULL;
     unlink(fifo_name);				 /* remove file list & FIFO */
@@ -135,12 +128,10 @@ _errstat backup_dpd(char *full_path, struct stat *b)
     if (dev == 0) return 0;			 /* error - ignore */
     if (S_ISDIR(fi.mode)) 			 /* directories have trailing '/' */
 	if (full_path[strlen(full_path)-1] != '/') 
-          strcat(full_path, "/");
-    if ( (proc_dev == 0) ||			 /* don't backup if on  */
-	 ((proc_dev == 1) && (proc_dev != dev)) ) {/* /proc device */ 
-	if (add_one_file_engine(NULL, &fi, full_path) != -1)
-	  total_selected += sizeof(struct file_info) + fi.name_len;
-    }
+	    strcat(full_path, "/");
+    if (for_backup(full_path))
+	if (add_to_info(&fi, full_path, FALSE) != -1)
+	    total_selected += sizeof(struct file_info) + fi.name_len;
     return 0;
 }
 
@@ -148,29 +139,20 @@ _errstat backup_dpd(char *full_path, struct stat *b)
 _errstat add_actual_files(WINDOW *mes_box)
 {
     _s32 count;
-    struct oa_file_entry *se;
-    
+    struct selected_entry *se;
+
+    se = sel_files;
     for (count=0; count<no_sel; count++) {
-	se = find_entry(sel_files, count);
 	if ( (se->selected == 1) || /* only if directly selected */
 	   (se->selected == 4) ) {
-	    process_dir(NULL, 0, name(se), se->incremental, backup_dpd, TRUE);
+	    process_dir(NULL, 0, se->i.name, se->incremental, backup_dpd, TRUE);
 	}
+	se++;
     }
     return 0;
 }
 
 
-_errstat backup_add_missing_dirs(WINDOW *mes)
-{
-/* This routines adds any directories that are required. */
-    
-    cp = info + sizeof(struct info_file_header);
-    if (make_cf(cp, ifd.no_in_archive) == -1) return -1; /* find common path */
-    return add_missing_dirs(mes, TRUE);
-}
-
-
 _errstat write_nameinfo(struct file_info *fi, char *cp)
 {
 /* An error writing to the backup device is a fatal error
@@ -192,7 +174,7 @@ _errstat write_nameinfo(struct file_info *fi, char *cp)
 }
 
 
-_errstat backup_file(WINDOW *mes, _s32 pos_in_archive)
+_errstat backup_file(WINDOW *mes, struct info_file_data *i_data)
 {
 /* Backup the file pointed to by cp and update cp 
  * An error writing to the backup device is a fatal error
@@ -201,18 +183,18 @@ _errstat backup_file(WINDOW *mes, _s32 pos_in_archive)
    Returns -1 if error
 */
     
-    struct file_info *fi;
-    char   tmpf[MAX_FNAME];
+    char   tmpf[MAX_FNAME], *cf, *fn_child;
 
     char   *fn, l[MAX_FNAME+50];
     int    fd;
     _s32   x, totalread, totalwrite;
     struct stat sbuf, csbuf;
     WINDOW *m=NULL;
+    struct file_info *fi;
     
-    fi = (struct file_info *) cp;		 /* fi points to file info */
-    fn = cp + sizeof(struct file_info);
-    fi->pos_in_archive = pos_in_archive;	 /* set pos in volume */
+    i_data->f.volume = (i_data->f.volume < 0) ? -ifd.number_volumes : ifd.number_volumes; /* force to belong to this volume */
+    fi = &i_data->f;				 /* fi points to file info */
+    fn = i_data->name;
 
     if (S_ISLNK(fi->mode)) {			 /* a soft link */
 	memset(tmpf, 0, sizeof(tmpf));		 /* name of original file */
@@ -241,71 +223,64 @@ _errstat backup_file(WINDOW *mes, _s32 pos_in_archive)
 	return write_nameinfo( fi, fn);		 /* file, only write out name */
     }
 
-    *tmpf = 0;					 /* must be a regular file */
+    cf = NULL;					 /* must be a regular file */
     fi->compressed = 0;				 /* not compressed */
     if ((compression) && (!exclude_compression(fn)) && (fdfifo != NULL)) {/* compress if want to and not excluded */
-	sprintf(l, "Reading filename from FIFO for %s", fn);
+	sprintf(l, "Reading filenames from FIFO for %s", fn);
 	if (log_level > 2) write_log(l);
-	f:;
-	if (fgets(tmpf, sizeof(tmpf), fdfifo) == NULL) {   /* gets the original filename */
-	    if (tmpf[strlen(tmpf)-1] == '\n')
-	      tmpf[strlen(tmpf)-1] = 0;
-	    sprintf(l, "Couldn't read filename from FIFO for %s - saving uncompressed", fn);
-	    if (log_level > 2) write_log(l);
-	    *tmpf = 0;
-	    goto non_compress;
-	}
-	if (m) {close_statusbox(m); touchwin(mes); wrefresh(mes);}
-	if (tmpf[strlen(tmpf)-1] == '\n')
-	  tmpf[strlen(tmpf)-1] = 0;
-	if (!strcmp(tmpf, "DISK FULL")) {	 /* child is waiting for disk space */
-	    if (mes) {
-		sprintf(l, "Child has no disk space - waiting");
-		if (log_level > 2) write_log(l);
-		m = status_box(mes, "Child waiting for disk space", 1, TRUE, 1);
-		goto f;				 /* do it again, sam */
+	while (1) {
+	    if (fifo_receive_file (fdfifo, &fn_child, &cf)) {
+		if (m) {close_statusbox(m); other_mes = 0; touchwin(mes); wrefresh(mes);} 
+		sprintf(l, "Fatal error while reading filenames from FIFO for %s - saving uncompressed", fn);
+		write_log(l);
+		cf = NULL;
+		goto non_compress;
 	    }
+	    if (m) {close_statusbox(m); other_mes = 0; touchwin(mes); wrefresh(mes);} 
+	    if (fn_child == NULL) {
+		sprintf(l, "EOF while reading filenames from FIFO for %s - saving uncompressed", fn);
+		if (log_level > 2) write_log(l);
+		cf = NULL;
+		goto non_compress;
+	    }
+	    if (fn_child == fifo_disk_full) {		 /* child is waiting for disk space */
+		if (mes) {
+		    sprintf(l, "Child has no disk space - waiting");
+		    if (log_level > 2) write_log(l);
+		    m = status_box(mes, "Child waiting for disk space", 1, TRUE, 1);
+		    other_mes=1;
+		    continue;
+		}
+	    }
+	    if (strcmp(fn_child, fn)) {			 /* checks that FIFO & me agree */
+		sprintf(l, "Mismatch reading in FIFO output for %s [got %s] - saving uncompressed", fn, fn_child);
+		if (log_level > 2) write_log(l); 
+		cf = NULL;
+		goto non_compress;
+	    }
+	    if (cf == NULL) {
+		sprintf(l, "FIFO didn't compress %s - saving uncompressed", fn);
+		if (log_level > 2) write_log(l);
+		goto non_compress;			 /* assume no compression */
+	    }
+	    break;
 	}
-	if (strcmp(tmpf, fn)) {			 /* checks that FIFO & me agree */
-	    sprintf(l, "Mismatch reading in FIFO output for %s [got %s] - saving uncompressed", fn, tmpf);
-	    if (log_level > 2) write_log(l); 
-	    *tmpf=0;
-	    goto non_compress;
-	}
-	sprintf(l, "Reading compressed filename from FIFO for %s", fn);   /* read name of compressed file */
+	sprintf(l, "Opening compressed file %s", cf);
 	if (log_level > 2) write_log(l);
-	if (fgets(tmpf, sizeof(tmpf), fdfifo) == NULL) {
-	    if (tmpf[strlen(tmpf)-1] == '\n')
-	      tmpf[strlen(tmpf)-1] = 0;
-	    sprintf(l, "Couldn't read compressed filename from FIFO for %s - saving uncompressed", fn);
-	    if (log_level > 2) write_log(l);
-	    *tmpf=0;
-	    goto non_compress;
-	}
-	if (tmpf[strlen(tmpf)-1] == '\n')
-	  tmpf[strlen(tmpf)-1] = 0;
-	if (!strcmp(tmpf, FIFO_ERR)) {
-	    sprintf(l, "FIFO didn't compress %s - saving uncompressed", fn);
-	    if (log_level > 2) write_log(l);
-	    *tmpf=0;
-	    goto non_compress;			 /* assume no compression */
-	}
-	sprintf(l, "Opening compressed file %s", tmpf);
-	if (log_level > 2) write_log(l);
-	fd = open(tmpf, O_RDWR);		 /* open the temp file */
+	fd = open(cf, O_RDWR);			 /* open the temp file */
 	if (fd == -1) {
 	    write_error_log(l);
-	    unlink(tmpf);			 /* compressed file, delete it */
+	    unlink(cf);				 /* compressed file, delete it */
 	    fi->checksum = -1;			 /* mark file as invalid */
 	    return write_nameinfo( fi, fn);
 	}
-	sprintf(l, "Getting compressed file status info for %s", tmpf);
+	sprintf(l, "Getting compressed file status info for %s", cf);
 	if (log_level > 2) write_log(l);
 	if (fstat(fd, &csbuf) == -1) {		 /* get compressed file size */
 	    write_error_log(l);
 	    fi->checksum = -1;			 /* mark file as invalid */
 	    close(fd);
-	    unlink(tmpf);
+	    unlink(cf);
 	    return write_nameinfo( fi, fn);
 	}
 	
@@ -341,10 +316,12 @@ _errstat backup_file(WINDOW *mes, _s32 pos_in_archive)
     if (write_nameinfo(fi, fn) == -1) {		 /* couldn't */
 	write_fatal_log(l);
 	close(fd);
+	if (*tmpf) unlink(tmpf);
 	return -1;				 /* write info/name - fatal */
     }
     if (fi->checksum == -1) {			 /* couldn't get checksum - not fatal */
 	close(fd);
+	if (*tmpf) unlink(tmpf);
 	return 0;
     }
     totalread = 0;
@@ -377,8 +354,8 @@ _errstat backup_file(WINDOW *mes, _s32 pos_in_archive)
 	write_error_log(l);
     }
     
-    if (*tmpf)					 /* remove temporary files */
-      unlink(tmpf);
+    if (cf != NULL)					 /* remove temporary files */
+      unlink(cf);
     return 0;
 }
 
@@ -390,24 +367,31 @@ _errstat write_volume_header(_s32 no_files)
  *  Returns 0 if OK, -1 if error
 */
     _s32 c;
-    struct oa_file_entry *fe;
+    struct selected_entry *se;
     int  c1, sz;
     char *cur_pos;
     struct volume_header *vh1, vh;
     char null_string=0;
+    char sname[MAX_FNAME];
 
     if (log_level > 2) write_log("Writing volume header");
     vh.volume_magic = VOLUME_MAGIC;
     vh.backup_time = time(NULL);		 /* time backed up */
     strcpy(vh.volume_title, volume_title);
-    vh.no_sels = no_sel;
+    vh.no_sels = no_sel+no_exclude;
     vh.no_in_volume = no_files;
     sz = sizeof(struct volume_header);
-    fe=sel_files;				 /* work out how much space */
+    se=sel_files;				 /* work out how much space */
     for (c=0; c<no_sel; c++) {			 /* needed for volume header */
-	sz += sizeof(_s32) + strlen(name(fe))+ 1 +
+	sz += sizeof(_s32) + strlen(se->i.name)+ 1 +
 	  sizeof(_s32)+ 0 + 1;			 /* room for filter */
-	advance_ofe(&fe);
+	se++;
+    }
+    se=excluded_files;				 /* work out how much space */
+    for (c=0; c<no_exclude; c++) {		 /* needed for volume header */
+	sz += sizeof(_s32) + strlen(se->i.name)+ 2 + /* 2 bytes (one for space) */
+	  sizeof(_s32)+ 0 + 1;			 /* room for filter */
+	se++;
     }
     vh.size_header = sz;
     ifd.size_volume_headers += sz;		 /* update size */
@@ -428,14 +412,14 @@ _errstat write_volume_header(_s32 no_files)
     }
     *((struct volume_header *)  cur_pos) = vh;
     cur_pos += sizeof(struct volume_header);
-    fe=sel_files;				 /* write file selections */
+    se=sel_files;				 /* write file selections */
     for (c=0; c<no_sel; c++) {
-	c1 = strlen(name(fe)) + 1;		 /* file name */
-	if (tape_write_namelen(name(fe)) == -1)
+	c1 = strlen(se->i.name) + 1;		 /* file name */
+	if (tape_write_namelen(se->i.name) == -1)
 	  return -1;
 	memcpy(cur_pos, &c1, sizeof(_s32));	 /* filename length */
 	cur_pos += sizeof(_s32);		 /* advance to filename pos */
-	strcpy(cur_pos, name(fe));
+	strcpy(cur_pos, se->i.name);
 	cur_pos += c1;				 /* past filename */
 	c1 = 0+1;				 /* filter */
 	if (tape_write_namelen(&null_string) == -1)
@@ -444,7 +428,27 @@ _errstat write_volume_header(_s32 no_files)
 	cur_pos += sizeof(_s32);
 	strcpy(cur_pos, "");			 /* filter */
 	cur_pos += c1;
-	advance_ofe(&fe);
+	se++;
+    }
+    se=excluded_files;				 /* write file exclusions */
+    for (c=0; c<no_exclude; c++) {
+	sname[0] = '!';
+	strcpy(&sname[1], se->i.name);
+	c1 = strlen(sname) + 1;			 /* file name */
+	if (tape_write_namelen(sname) == -1)
+	  return -1;
+	memcpy(cur_pos, &c1, sizeof(_s32));	 /* filename length */
+	cur_pos += sizeof(_s32);		 /* advance to filename pos */
+	strcpy(cur_pos, sname);
+	cur_pos += c1;				 /* past filename */
+	c1 = 0+1;				 /* filter */
+	if (tape_write_namelen(&null_string) == -1)
+	  return -1;
+	memcpy(cur_pos, &c1, sizeof(_s32));
+	cur_pos += sizeof(_s32);
+	strcpy(cur_pos, "");			 /* filter */
+	cur_pos += c1;
+	se++;
     }
     return 0;					 
 }
@@ -454,29 +458,91 @@ _errstat make_comp_file(char *s)
 {
 /* Creates a file and gives it a name then writes the names of the files
  * to be compressed, filename after filename.
- * 
+ *
+   Assumes the info file is open
+   
  * Returns -1 if error, 0 otherwise
  */
-    FILE *cf;
+    FILE *c1f;
     int c;
-    cp = info + sizeof(struct info_file_header);
+    struct info_file_data i_data;
+    
     taper_tmpnam(s);
-    cf = fopen(s, "w+");			 /* create & open file */
-    if (cf == NULL) return -1;
+    c1f = fopen(s, "w+");			 /* create & open file */
+    if (c1f == NULL) return -1;
     
     for (c=0; c<ifd.no_in_archive;c++) {
-	if (((struct file_info *) cp)->pos_in_archive == 0)   /* only if not already in archive */
-	  if (S_ISREG(((struct file_info *) cp)->mode)) {   /* only regular files */
-	      fputs(cp+sizeof(struct file_info), cf);
-	      fputs("\n", cf);
+	if (read_info_rec(c, &i_data) == -1)
+	    return do_exit(ERROR_READING_INFO);
+	if (i_data.f.pos_in_archive == 0)   /* only if not already in archive */
+	  if (S_ISREG(i_data.f.mode)) {   /* only regular files */
+	      fputs(i_data.name, c1f);
+	      fputs("\n", c1f);
 	  }
-	cp+=sizeof(struct file_info);		 /* advance to next entry */
-	PAST_STRING_FI(cp);
     }
-    fclose(cf);
+    fclose(c1f);
     return 0;
 }
 
+
+PRIVATE WINDOW *s_mes=NULL;
+PRIVATE _s32 s_count, s_noarchive;
+PRIVATE struct info_file_data s_idata;
+PRIVATE time_t s_tstart, s_tcurrent;
+PRIVATE _u32   bytes_processed, bytes_written;
+
+PRIVATE void backup_alarm(int sig)
+{
+    /* Updates the status box on a SIGALARM */
+
+    char s[200], s1[200], s2[200], s3[200];
+    time_t t_elapsed;
+    
+    signal(SIGALRM, SIG_IGN);	 /* reset alarm */
+    if (!other_mes) {
+	t_elapsed=s_tcurrent-s_tstart;
+	status_box(s_mes, s_idata.name, 0, FALSE, 1);
+	sprintf(s, "File %d of %d",  s_count, s_noarchive);
+	status_box(s_mes, s, 2, FALSE, 1);
+	sprintf(s, "Processed %s of %s", print_kb(s2, bytes_processed), 
+		print_kb(s3, total_selected));
+	status_box(s_mes, s, 3, FALSE, 1);
+	if (bytes_written)
+	    sprintf(s, "Written %s : Ratio %.2f",
+		    print_kb(s2, bytes_written), 
+		    (float) bytes_processed/(float) bytes_written);
+	else
+	    sprintf(s, "Written %s : Ratio 1.0",
+		    print_kb(s2, bytes_written));
+	status_box(s_mes, s, 4, FALSE, 1);
+	sprintf(s, "Total on archive %s", print_kb(s2, bytes_written+total_compressed));
+	status_box(s_mes, s, 5, FALSE, 1);
+	if (total_selected) 
+	    sprintf(s, "%.0f%% done, Elapsed %s, Remaining %s",  (float) bytes_processed/(float) total_selected * 100,
+		    convtime(s2, s_tstart, s_tcurrent),
+		    (bytes_processed == 0) ? "" :
+		    convtime(s3, t_elapsed, t_elapsed / ((float) bytes_processed/(float) total_selected)));
+	else				 /* if bytes=0, base on # files */
+	    sprintf(s, "%d%% done, Elapsed %s, Remaining %s",  100*s_count/s_noarchive,
+		    convtime(s2, s_tstart, s_tcurrent),
+		    ((s_count/s_noarchive) == 0) ? "" :
+		    convtime(s3, t_elapsed,  t_elapsed / (s_count/s_noarchive)));
+	status_box(s_mes, s, 7, FALSE, 1);
+	if (t_elapsed) {
+	    sprintf(s, "Backup rate %s/min [%s/min]",
+		print_mb(s1, bytes_written/t_elapsed*60),
+		    print_mb(s2, bytes_processed/t_elapsed*60));
+	    status_box(s_mes, s, 8, FALSE, 1);
+	    
+	}
+	if (!no_windows) {
+	    touchwin(s_mes); wrefresh(s_mes);	 /* in case new tape */
+	}
+    }
+    signal(SIGALRM, backup_alarm);	 /* reset alarm */
+}
+
+
 _errstat  do_backup(void) {				 
 /* For aborting a backup, 
  * 
@@ -484,41 +550,34 @@ _errstat  do_backup(void) {
  *   was aborted. The info file is changed accordingly
 */
     _s32   count=0, no_old_files, no_written, org_files;
-    _u32   bytes_processed, bytes_written;
-    WINDOW *mes=NULL;
     char   s[100], s2[30], s3[30], s4[30], tmpbuf[MAX_FNAME], cfile[MAX_FNAME];
-    time_t t_start, t_current;
     struct volume_tape_info vti;
     int    x;
     _s8    quitting;
     struct file_info *fi;
     char   sa[9][150];
     struct volume_header v;
+    struct info_file_data i_data;
+    time_t t_current;
 	
     if (!no_sel)				 /* none selected */
       return 0;
     if (!no_windows)
-      mes = status_box(mes, "          Opening backup device...          ", 3, TRUE, 6);
+      s_mes = status_box(s_mes, "          Opening backup device...          ", 3, TRUE, 6);
 
-    status_box(mes, "Adding to archive directory...", 3, FALSE, 1);
+    status_box(s_mes, "Adding to archive directory...", 3, FALSE, 1);
     if (!append) {
 	tdh.magic = TAPER_MAGIC_NUMBER;		 /* identify our directory volume */
 	tdh.tape_number = 1;			 /* first volume */
-	ifd.archive_id = tdh.archive_id;	 /* setup info file header */
-	ifd.info_file_size = sizeof(struct info_file_header);
-	ifd.magic = INFO_MAGIC;
-	ifd.number_tapes = 0;
-	ifd.number_tsi = 0;
-	ifd.number_volumes = 0;
-	ifd.size_volume_headers = 0;
-	ifd.no_in_archive = 0;
+	clear_ifd();
+	ifd.archive_id = tdh.archive_id;
 	strcpy(ifd.archive_title, archive_title);
 	strcpy(tdh.archive_title, archive_title);
 	no_old_files = 0;
 	vt_info = my_realloc(vt_info, sizeof(struct volume_tape_info));
+	if (open_info_file(FALSE, ifd.archive_id, TRUE) == -1) return -1;
     }
     else {
-	ifd = *((struct info_file_header *) info);/* get original ifd */
 	ifd.magic = INFO_MAGIC;			 /* uncompressed originally */
 	no_old_files = ifd.no_in_archive;
 	vt_info = my_realloc(vt_info, sizeof(struct volume_tape_info)*(ifd.number_volumes+1));
@@ -527,40 +586,34 @@ _errstat  do_backup(void) {
     org_files = no_old_files;
     if (!append) {
 	if (tape_open(O_RDWR) == -1) return -1;
-	tape_set_blk_size();
 	if (write_tape_header(&tdh) == -1)
 	  return -1;
     }
     else {
-	if (goto_end_vol(mes, 3, ifd.number_volumes, 1, TRUE, TRUE) == -1)/* goto end of */
-	  return -1;				 /* previous volume */
-	tape_set_blk_size();
+	if (use_eom) {
+	    if (tape_eom() == -1) return -1;
+	}
+	else {
+	    if (goto_end_vol(s_mes, 3, ifd.number_volumes, 1, TRUE, O_RDWR) == -1)/* goto end of */
+		return -1;				 /* previous volume */
+	}
     }
 
     ifd.number_volumes++;
     vti.volume = ifd.number_volumes;		 /* update volume/tape info */
     vti.start_tape = ifd.number_tapes;
-    cp = info + sizeof(struct info_file_header);
-    if (append) 				 /* move past existing entries */
-      for (count=0; count < no_old_files; count++) {
-	  cp += sizeof(struct file_info);
-	  PAST_STRING_FI(cp);
-      }
 
-    if (add_actual_files(mes) == -1)
-      return -1;				 /* add filenames to directory */
+    if (add_actual_files(s_mes) == -1)		 /* add filenames to info file */
+      return -1;				 
     
-    memcpy(info, &ifd, sizeof(struct info_file_header));
-    status_box(mes, "", 3, FALSE, 1);
-    if (backup_add_missing_dirs(mes) == -1)	 /* add directories that are required */
+    status_box(s_mes, "", 3, FALSE, 1);
+    if (process_info(s_mes, 3, FALSE) == -1)	 /* add directories that are required */
       return -1;
-    status_box(mes, "", 1, FALSE, 1);
+    status_box(s_mes, "", 1, FALSE, 1);
 
     if (write_volume_header(ifd.no_in_archive-no_old_files) == -1) return -1;/* write volume header */
 /* free superflous memory */    
     sel_files = my_realloc(sel_files, 1);
-    archive_files = my_realloc(archive_files, 1);
-    len_archive_files = 0; cur_af_size = 0;
 
     bytes_written=0;
     bytes_processed=0;
@@ -598,7 +651,7 @@ _errstat  do_backup(void) {
 	      sprintf(s4, "%d", min_free);
 	      if (execlp("bg_backup", "bg_backup", log_file, s3, cfile, fifo_name, s, s2, exclude_compress, temp_dir, s4, NULL) == -1) {		  write_error_log("starting background backup");
 		  fdfifo = fopen(fifo_name, "w");
-		  fprintf(fdfifo, "NO CHILD\n"); /* no child! */
+		  fifo_send_no_child (fdfifo);	 /* no child! */
 		  fclose(fdfifo);
 		  exit(-1);			 /* should never get here */
 	      }
@@ -618,34 +671,38 @@ _errstat  do_backup(void) {
 	      }
 	      return do_exit(ERROR_OPENING_FIFO);
 	  }
-	  fgets(tmpbuf, sizeof(tmpbuf), fdfifo);
-	  tmpbuf[strlen(tmpbuf)-1] = 0;		 /* remove trailing '\n' */
-	  if(!strcmp(tmpbuf, "NO CHILD")) {
+	  if (fifo_receive_child_status (fdfifo)) {
+	      /* no child, or fatal error */
 	      waitpid(backup_child, NULL, WNOHANG);
 	      return do_exit(ERROR_NO_BACKUP_CHILD);
 	  }
-	  if (comp_head_start) {			 /* let the compression program */
-	      status_box(mes, "Giving compression a head start", 3, FALSE, 1);
-	      sleep(60*comp_head_start);		 /* have a head start */
-	      t_start = time(NULL);		 /* restart timer */
+	  if (comp_head_start) {		 /* let the compression program */
+	      status_box(s_mes, "Giving compression a head start", 3, FALSE, 1);
+	      sleep(60*comp_head_start);	 /* have a head start */
+	      s_tstart = time(NULL);		 /* restart timer */
 	  }
       }
-    cp = info + sizeof(struct info_file_header);
-    if (mes) nodelay(mes, TRUE);
+    if (s_mes) nodelay(s_mes, TRUE);
     quitting=0;
     no_written=0;
-    t_start = time(NULL);
+    s_tstart = time(NULL);
+    set_1s_timer();				 /* set a 1s timer */
+    other_mes = 0;
+    signal(SIGALRM, backup_alarm);
     for (count=0; count< ifd.no_in_archive; count++) {
-	*s = (mes == NULL) ? 0 : wgetch(mes);
+	if (read_info_rec(count, &i_data) == -1) return do_exit(ERROR_READING_INFO);
+	fi = &i_data.f;
+	*s = (s_mes == NULL) ? 0 : wgetch(s_mes);
 	if (((*s == 'q') || (*s == 'Q')) && (!quitting)) {
 	    if (message_box("Confirm abort backup", MB_YESNO)) {
-		touchwin(mes); wrefresh(mes);
+		touchwin(s_mes); wrefresh(s_mes);
 		backup_stop_child();		 /* stop child from compressing */
-		fi = (struct file_info *) cp;
 		fi->act_size = -1;
 		fi->name_len = 0;
 		fi->checksum = -2;
-		write_nameinfo(fi, "");
+		if (write_info_rec(count, &i_data) == -1)
+		    return do_exit(ERROR_WRITING_INFO); /* don't need to reindex */
+		write_nameinfo(fi, "");		 /* because haven't changed index field */
 		ifd.no_in_archive = no_old_files;/* correct info file */
 		memcpy(&v, get_vh(vol_headers, ifd.number_volumes), sizeof(struct volume_header));   /* this needed to */
 		v.no_in_volume = no_written;	 /* ensure aligned access */
@@ -655,88 +712,58 @@ _errstat  do_backup(void) {
 		break;
 	    }
 	}
-	if (((struct file_info *) cp)->pos_in_archive == 0) {  /* only need new ones */
+	if (fi->pos_in_archive == 0) {		 /* only need new ones */
 	    no_written++;
-	    status_box(mes, cp+sizeof(struct file_info), 0, FALSE, 1);
-	    sprintf(s, "File %d of %d", count, ifd.no_in_archive);
-	    status_box(mes, s, 2, FALSE, 1);
-	    sprintf(s, "Processed %s of %s", print_kb(s2, bytes_processed), 
-		    print_kb(s3, total_selected));
-	    status_box(mes, s, 3, FALSE, 1);
-	    if (bytes_written)
-	      sprintf(s, "Written %s : Ratio %.2f",
-		      print_kb(s2, bytes_written), 
-		      (float) bytes_processed/(float) bytes_written);
-		else
-	      sprintf(s, "Written %s : Ratio 1.0",
-		      print_kb(s2, bytes_written));
-	    status_box(mes, s, 4, FALSE, 1);
-	    sprintf(s, "Total on archive %s", print_kb(s2, bytes_written+total_compressed));
-	    status_box(mes, s, 5, FALSE, 1);
-	    t_current = time(NULL);
-	    x = t_current-t_start;
-	    if (total_selected) 
-	      sprintf(s, "%.0f%% done, Elapsed %s, Remaining %s",  (float) bytes_processed/(float) total_selected * 100,
-		      convtime(s2, t_start, t_current),
-		      (bytes_processed == 0) ? "" :
-		      convtime(s3, x, x / ((float) bytes_processed/(float) total_selected)));
-	    else				 /* if bytes=0, base on # files */
-	      sprintf(s, "%d%% done, Elapsed %s, Remaining %s",  100*count/ifd.no_in_archive,
-		      convtime(s2, t_start, t_current),
-		      (count == 0) ? "" :
-		      convtime(s3, x,  x / (count/ifd.no_in_archive)));
-	    status_box(mes, s, 7, FALSE, 1);
-	    if (x) {
-		sprintf(s, "Backup rate %s/min [%s/min]",
-			print_mb(tmpbuf, bytes_written/x*60),
-			  print_mb(s2, bytes_processed/x*60));
-		status_box(mes, s, 8, FALSE, 1);
-	    }
-	    if (!no_windows) {
-		touchwin(mes); wrefresh(mes);	 /* in case new tape */
-	    }
-	    if (backup_file(mes, ++no_old_files) == -1) {
+	    s_count = count; s_noarchive = ifd.no_in_archive; /* set up */
+	    s_idata = i_data;			 /* data for status box */
+	    s_tcurrent = time(NULL);
+	    no_old_files++;
+	    i_data.f.pos_in_archive = no_old_files;
+	    if (backup_file(s_mes, &i_data) == -1) {
 		tape_close();			 /* rewinding device so will auto update headers */
 		backup_stop_child();
 		if (*cfile) unlink(cfile);
 		waitpid(backup_child, NULL, WNOHANG);
 		return -1;			 /* error message handled by backup file */
 	    }
-	    bytes_processed += sizeof(struct file_info) + ((struct file_info *) cp)->name_len;
-	    bytes_written += sizeof(struct file_info) + ((struct file_info *) cp)->name_len;
-	    if ((S_ISREG(((struct file_info *) cp)->mode)) ||
-		(S_ISLNK(((struct file_info *) cp)->mode)) ) {   /* only add this if regular file */
-		bytes_processed += ((struct file_info *) cp)->size;
-		bytes_written += ((struct file_info *) cp)->act_size;
+	    if (write_info_rec(count, &i_data) == -1) return do_exit(ERROR_WRITING_INFO);
+	    if (add_to_info_index(&i_data) == -1) return do_exit(ERROR_WRITING_INFO); /* adds to indicies */
+	    bytes_processed += sizeof(struct file_info) + fi->name_len;
+	    bytes_written += sizeof(struct file_info) + fi->name_len;
+	    if ((S_ISREG(fi->mode)) ||
+		(S_ISLNK(fi->mode)) ) {   /* only add this if regular file */
+		bytes_processed += fi->size;
+		bytes_written += fi->act_size;
 	    }
 	}
-	
-	cp += sizeof(struct file_info);
-	PAST_STRING_FI(cp);
     }
+    reset_timer();
     t_current = time(NULL);
-    x = t_current - t_start;
+    x = t_current - s_tstart;
     if (x == 0) x=1;
-    status_box(mes, "", 0, FALSE, 1);
-    status_box(mes, "", 2, FALSE, 1);
-    status_box(mes, "", 4, FALSE, 1);
-    status_box(mes, "", 6, FALSE, 1);
-    status_box(mes, "Updating header segments", 3, FALSE, 1);
+    status_box(s_mes, "", 0, FALSE, 1);
+    status_box(s_mes, "", 2, FALSE, 1);
+    status_box(s_mes, "", 4, FALSE, 1);
+    status_box(s_mes, "", 5, FALSE, 1);
+    status_box(s_mes, "", 7, FALSE, 1);
+    status_box(s_mes, "", 8, FALSE, 1);
+    status_box(s_mes, "Updating header segments", 3, FALSE, 1);
     tape_close();				 /* rewinding device so will auto update headers */
     vti.blocks_on_last_tape = blocks_passed;
     vti.end_tape = ifd.number_tapes;
     memcpy(vt_info+ifd.number_volumes-1, &vti, sizeof(vti));   
 /* Code for info file at end here */
-    if (write_out_info_file(info, ifd.archive_id) != -1)
-      if (compress_info) compress_info_file(ifd.archive_id);
+    status_box(s_mes, "Closing/Compressing info File", 3, FALSE, 1);
+    if (write_info_file()) return -1;		 /* write out info file */
     if (fdfifo) {
       fclose(fdfifo);				 /* close FIFO */
-      unlink(fifo_name);				 /* remove file list & FIFO */
+      unlink(fifo_name);			 /* remove file list & FIFO */
     }
     if (*cfile) unlink(cfile);			 /* in case child didn't */
     waitpid(backup_child, NULL, 0);
     backup_child = 0;
-    close_statusbox(mes);
+    close_info_file();
+    close_statusbox(s_mes);
     if (win_main) {
 	touchwin(win_main); 
 	wrefresh(win_main);
@@ -746,7 +773,7 @@ _errstat  do_backup(void) {
     sprintf(sa[2], "Backed up: %d files,  %s  [%s]", ifd.no_in_archive - org_files,
 	    print_mb(s2, bytes_written),
 	    print_mb(s3, bytes_processed));
-    if (log_level > 1) write_log(sa[2]);
+    if (log_level) write_log(sa[2]);
     if (mf) {write(mf, sa[2], strlen(sa[2])); write(mf, "\n", 1);}
         sprintf(sa[3], "Total on archive %s [%s]. Ratio %.2f", 
 	    print_mb(s2, bytes_written+total_compressed),
@@ -754,35 +781,40 @@ _errstat  do_backup(void) {
 	    (total_compressed+bytes_written == 0) ? 1 : 
 	       (float) (total_uncompressed+bytes_processed)/
 	       (float) (total_compressed+bytes_written));
-    if (log_level > 1) write_log(sa[3]);
+    if (log_level) write_log(sa[3]);
     if (mf) {write(mf, sa[3], strlen(sa[3])); write(mf, "\n", 1);}
     strcpy(sa[4], "");
-    sprintf(sa[5], "Time elapsed %s.", convtime(s2, t_start, t_current));
-    if (log_level > 1) write_log(sa[5]);
+    sprintf(sa[5], "Time elapsed %s.", convtime(s2, s_tstart, t_current));
+    if (log_level) write_log(sa[5]);
     if (mf) {write(mf, sa[5], strlen(sa[5])); write(mf, "\n", 1);}
     sprintf(sa[6], "Backup rate %s/min [%s/min]", 
 	    print_mb(tmpbuf, bytes_written/x*60),
 	    print_mb(s3, bytes_processed/x*60));
-    if (log_level > 1) write_log(sa[6]);
+    if (log_level) write_log(sa[6]);
     if (mf) {write(mf, sa[6], strlen(sa[6])); write(mf, "\n", 1);}
     strcpy(sa[7], "");
     sprintf(sa[8], "%d warnings, %d errors", log_warnings, log_errors);
-    if (log_level > 1) write_log(sa[8]);
+    if (log_level) write_log(sa[8]);
     if (mf) {write(mf, sa[8], strlen(sa[8])); write(mf, "\n", 1);}
-    if (!no_windows) multi_message_box(sa, 9, MB_OK);
+    if (!no_windows) multi_message_box(sa, 9, MB_OK, TRUE);
     return 0;
 }
 
 
 _errstat read_in_fileset(int argc, char *argv[])
 {
-/* Reads in the files */
+/* Reads in the files
+
+   Returns 1 if all OK
+           -1 otherwise
+
+ */
     FILE *f;
     char ln[MAX_FNAME], s[MAX_FNAME];
-    struct oa_file_entry *ce;
-    _s32 c, c1;
+    _s32 c;
     struct direntry entry;
     time_t t;
+    _s8  inc_or_exc;
     
     c = 0;
     while (c < argc) {
@@ -807,50 +839,56 @@ _errstat read_in_fileset(int argc, char *argv[])
 		      sprintf(ln, "Using file set %s\n", &argv[c][1]);
 		      write(mf, ln, strlen(ln));
 		      while (!feof(f)) {
-			  fgets(ln, sizeof(ln), f);
-			  if (ln[strlen(ln)-1] == '\n')
-			    ln[strlen(ln)-1] = 0;
-			  if (*ln) {
-			      ce=sel_files;
-			      for (c1=0; c1<no_sel; c1++) {		 /* make sure not already in */
-				  if (!strcmp(name(ce), ln)) 
-				    break;
-				  advance_ofe(&ce);
+			  if (get_line(ln, f) == NULL) break;	 /* EOF */
+			  if ((*ln != 'E') && (*ln != 'I')) {
+			      strcpy(ln, "Illegal line in backup set - using old format?\n");
+			      write(mf, ln, strlen(ln));
+			      strcpy(ln, "BACKUP ABORTED\n");
+			      write(mf, ln, strlen(ln));
+			      fclose(f);
+			      return -1;
+			  }
+			  inc_or_exc = (*ln == 'I') ? ENTRY_INCLUDE : ENTRY_EXCLUDE;
+			  if (get_line(ln, f) == NULL) break;
+			  if (!chk_sel_excl(ln)) {
+			      if (get_statinfo(ln, &entry.info) != -1) {/* ignore if error */
+				  if (inc_or_exc == ENTRY_INCLUDE) {
+				      select_entry_1(&entry, ln);
+				      write(mf, "  Selected ", strlen("  Selected "));
+				  }
+				  else {
+				      exclude_entry_1(&entry, ln);
+				      write(mf, "  Excluded ", strlen("  Excluded "));
+				  }
+				  write(mf, ln, strlen(ln));
+				  write(mf, " from fileset\n", strlen(" from fileset\n"));
 			      }
-			      if (c1==no_sel)
-				if (get_statinfo(ln, &entry.info) != -1) {/* ignore if error */
-				    select_entry_1(&entry, ln);
-				    write(mf, "  Selected ", strlen("  Selected "));
-				    write(mf, ln, strlen(ln));
-				    strcpy(ln, " [from fileset]\n");
-				    write(mf, ln, strlen(ln));
-				}
 			      else {
 				  sprintf(s, "WARNING: %s is not found in fileset %s\n", ln, &argv[c][1]);
 				  write(mf, s, strlen(s));
 			      }
 			  }
-			  fgets(ln, sizeof(ln), f);		 /* ignore filter */
 		      }
 		      fclose(f);
 		  }
-		  else {
-		      if (get_statinfo(argv[c], &entry.info) != -1) {/* ignore if error */
-			  select_entry_1(&entry, argv[c]);
-			  sprintf(ln, "Selected %s\n", argv[c]);
-			  write(mf, ln, strlen(ln));
-		      }
-		      else {
-			  sprintf(ln, "WARNING: %s is not found\n", &argv[c][1]);
-			  write(mf, ln, strlen(ln));
-		      }
-		  }
-	      }
+	}
 	c++;
     }
     time(&t);
     sprintf(ln, "\nBackup commenced at %s\n", ctime(&t));
     write(mf, ln, strlen(ln));
+
+    if ((unattended_id != -1)) {		 /* unattended backup and user */
+	if (unattended_id != tdh.archive_id) {	 /* wants to append to a particular one */
+	    sprintf(s, "Archive %d is in tape drive. I want archive %d\n",
+		    tdh.archive_id, unattended_id);
+	    if (mf) write(mf, s, strlen(s));
+	    write_log(s);
+	    if (mf) write(mf, "Backup NOT made\n", strlen("Backup NOT made\n"));
+	    write_log("Backup NOT made");
+	}
+	return -1;
+    }
     return 1;
 }
 
@@ -859,13 +897,13 @@ _errstat utils_write_nullheader(WINDOW **mes);
 _errstat check_backup(void) {
 /* Make sure backup is a valid device and confirm with use about
    an erase if it has been requested.         */
-    int x;
+    int x, fd;
     WINDOW *mes=NULL;
     char info_file[MAX_FNAME];
-    struct stat buf;
-    _s32 oldarch;
+    _s32 oldarch=0;
 
-    if (stat(tape, &buf) == -1) {		 /* touch file if needed */
+    if ((fd=rmtopen(tape, O_RDONLY, S_IRWXU)) == -1) {
+	/*    if (stat(tape, &buf) == -1) {*/		 /* touch file if needed */
 	if (tape_type == TAPE_TYPE_FILE) {
 	    if (log_level > 2) write_log("Creating backup file");
 	    close(creat(tape, S_IREAD|S_IWRITE));
@@ -874,8 +912,10 @@ _errstat check_backup(void) {
 	else
 	  return do_exit(ERROR_OPENING_BACKUP);
     }
-    else
-      x= do_read_vol_dir(-1, tape, O_RDWR, append, FALSE);
+    else {
+	close(fd);
+	x= do_read_vol_dir(-1, tape, O_RDWR, append, FALSE, TRUE);
+    }
     if (x==-1) return -1;
     if (x == BAD_MAGIC) {
 	if (!no_windows) {			 
@@ -901,7 +941,7 @@ _errstat check_backup(void) {
 	      return 0;
 	}
     }
-	
+
     if (x == TAPE_EXIST_EMPTY) {		 /* empty taper tape */
 	append = 0;
 	return 1;
@@ -917,16 +957,24 @@ _errstat check_backup(void) {
 	}
 	else {
 	  oldarch = tdh.archive_id;
-	    utils_write_nullheader(&mes);
-	    tape_close();
+	  utils_write_nullheader(&mes);
+	  tape_close();
 	}
 	strcpy(info_file, taper_info_files);	 /* look for information file */
 	if (info_file[strlen(info_file)-1] != '/')	 /* associated with this archive */
 	  strcat(info_file, "/");
 	sprintf(info_file, "%staper_info.%u", info_file, oldarch);
 	unlink(info_file);			 /* attempt to delete info file */
-	if (mes) 
-	  close_statusbox(mes);
+	info_fd = 0;
+	for (x=0; x<=INFO_INDICIES; x++) {
+	    strcpy(info_file, taper_info_files);	 /* look for information file */
+	    if (info_file[strlen(info_file)-1] != '/')	 /* associated with this archive */
+		strcat(info_file, "/");
+	    sprintf(info_file, "%staper_info.%u.%d", info_file, oldarch, x);
+	    unlink(info_file);
+	    info_index[x] = 0;
+	}
+	if (mes)  close_statusbox(mes);
 	return 1;
     }
 	
@@ -945,6 +993,7 @@ void taper_backup(int argc, char *argv[])
 {
     int      x;
     int      old_append;
+    WINDOW   *mes_box=NULL;
 
     old_append = append;			 /* save value of append for future use */
     if (open_logfile("Backup") == -1)
@@ -974,10 +1023,6 @@ void taper_backup(int argc, char *argv[])
 	    strcpy(tdh.archive_title, archive_title);
 	    strcpy(ifd.archive_title, archive_title);
 	}
-#ifdef MEMORY_TIGHT
-	archive_files = my_realloc(archive_files, 1);   /* free this memory */
-	len_archive_files = 0; 
-#endif	
 	print_title_line();
 	chdir(cur_dir);
 	if (!no_windows) {
@@ -994,6 +1039,7 @@ void taper_backup(int argc, char *argv[])
 	print_my_name();
 	if (x > 0)  do_backup();
 	if (dv > 0) tape_close();
+	if (!no_windows) close_statusbox(mes_box);
 	
     }
     else {
