@@ -1,7 +1,7 @@
 /*
    Time-stamp: <96/07/19 20:15:26 yusuf>
 
-   $Id: lzrw3.h,v 1.2 1996/07/27 20:42:31 yusuf Exp $	
+   $Id: lzrw3.h,v 1.3 1997/06/29 01:31:29 yusuf Exp $	
 
 */
 
@@ -9,6 +9,8 @@
 
 #ifndef _COMPRESS_H
 #define _COMPRESS_H
+
+#define NULL ((void *)0)
 /*
  /home/cvs/zftape/compress/compress.h,v
  claus
@@ -249,7 +251,7 @@ ULONG *p_dst_len       /* Pointer to a longword where routine will write:     */
 /******************************************************************************/
 
 #if defined(linux) || defined(LINUX)
-#include <linux/types.h>
+/* #include <linux/types.h> */
 /*#include <linux/string.h>*/
 #define fast_copy(src,dst,len) memcpy(dst,src,len)
 
