@@ -1,7 +1,7 @@
 /*
-   Time-stamp: <96/08/05 19:30:55 yusuf>
+   Time-stamp: <97/08/09 21:42:50 yusuf>
 
-   $Id: defaults.h,v 1.11 1996/08/05 19:02:00 yusuf Exp $
+   $Id: defaults.h,v 1.21 1997/08/09 14:02:04 yusuf Exp $
 */
 
 
@@ -49,7 +49,6 @@
 							accepts a filename to compress and then writes
 							the output to stdout */
 
-#define MEM_BLOCK_SIZE 100000			 /* no of bytes to allocate in each memory alloc procedure */
 #define DEFAULT_MIN_SEEK 10			 /* default # blocks to have to move before seeking */
 
 #define ZFTAPE_DEFAULT_BLOCK_SIZE (28*1024)	 /* zftape expect max 28K blocks */
@@ -70,8 +69,6 @@
 #define PRIVATE static
 #define PUBLIC 
 
-#define FIFO_ERR "&*^Error&^*"			 /* String returned by FIFO if couldn't compress */
-
 #define TAPE_TYPE_FTAPE 0
 #define TAPE_TYPE_ZFTAPE 1
 #define TAPE_TYPE_SCSI 2
@@ -82,17 +79,22 @@
 #define MAX_ARCHIVETITLE 50			 /* max size of archive title */
 #define MAX_FILTER 15				 /* max size of a filter */
 #define MAX_FNAME NAME_MAX			 /* max size of filename */
+#define MAX_REGEXP 50				 /* max size of  reg exp */
 
+/* Tape magic numbers */
 #define TAPER_MAGIC_NUMBER 0x73912376		 /* new from 6.5 onwards */
 #define TAPER_64_MAGIC 0x739123		
 #define TAPER_4_MAGIC_NUMBER 0x729123		 /* for versions 4.x */
+
+/* Volume magic numbers */
 #define VOLUME_MAGIC 0x13921323			 /* for versions 6.6 onwards */
 #define VOLUME_65_MAGIC 0x13921322		 /* from 6.5 onwards */
 #define VOLUME_64_MAGIC 0x139213
 #define VOLUME_MAGIC_INFO 0x43917312		 /* magic if this volume is an info file */
 
-#define INFO_MAGIC 0x67128912			 /* magic number of uncompressed info file */
-#define INFO_MAGIC_COMPRESSED 0x22191355	 /* magic number of compressed info file */
+/* Info file magic numbers */
+#define INFO_MAGIC 0x67128912			 /* magic number of info file */
+#define COMPRESSED_MAGIC 0x22191355		 /* magic number of compressed info file */
 #define END_ENTRY 0x04
 
 #define TAPE_EXIST 0				 /* return values for */
@@ -102,6 +104,17 @@
 
 #define COLOR_ONVOL 7
 #define COLOR_SELECTED 8
-#define COLOR_BOTTOM 9
-#define COLOR_TITLE 10
+#define COLOR_EXCLUDED 9
+#define COLOR_BOTTOM 10
+#define COLOR_TITLE 11
+
+#define INFO_INDICIES 1				 /* number of indexes for info file */
+#define INFO_NAME 0
+
+/* For mtree.c */
+#define MAXSTACK 3				 /* # of stacks (ie. # separate instances) */
+#define MAXSTACK_ITEMS 20			 /* maximum size of stack */
+#define TREEORD 21				 /* Order of the B-Tree */
+#define MB2 (TREEORD/2 + (TREEORD%2!=0))	/* m/2 rounded up - see notes */
+
 
