@@ -1,34 +1,27 @@
 /*
-   Time-stamp: <96/07/19 20:17:55 yusuf>
+   Time-stamp: <98/02/05 17:54:01 yusuf>
 
-   $Id: mkinfo.c,v 1.18 1996/07/27 20:42:10 yusuf Exp $	
+   $Id: mkinfo.c,v 1.32 1998/03/03 04:35:11 yusuf Exp $	
 
 */
 
 #ifndef lint
-static char vcid[] = "$Id: mkinfo.c,v 1.18 1996/07/27 20:42:10 yusuf Exp $";
+static char vcid[] = "$Id: mkinfo.c,v 1.32 1998/03/03 04:35:11 yusuf Exp $";
 #endif /* lint */
 
 
 
 #include "taper.h"
 
-PRIVATE int info_fd;
-PRIVATE _errstat write_info(_vptr buf, _s32 szbuf, char *lmes)
-{
-    if (write(info_fd, buf, szbuf) != szbuf) {
-	write_error_log(lmes);
-	return do_exit(ERROR_WRITING_INFO);
-    }
-    return 0;
-}
 
 PUBLIC _u32 mktr, mktrc;
-PRIVATE _errstat mkinfo_action(struct file_info *fi, char *fn, struct oa_file_entry *dummy)
+PRIVATE _errstat mkinfo_action(struct file_info *fi, char *fn, struct info_file_data *dummy)
 {
     char l[MAX_FNAME];
     _s32  x, p=0;
-    
+    struct info_file_data i_data;
+
+    fi->volume = (fi->volume < 0) ? -ifd.number_volumes : ifd.number_volumes;
     strcpy(l, "Writing filename & file info");
     if (log_level > 2) write_log(l);
     if (S_ISLNK(fi->mode)) 
@@ -42,17 +35,16 @@ PRIVATE _errstat mkinfo_action(struct file_info *fi, char *fn, struct oa_file_en
 	  fi->size = strlen(l)+1+sizeof(_s32);
 	  p = 1;
       }
-    if (write_info(fi, sizeof(struct file_info), l) == -1) return -1;
-    if (write_info(fn, fi->name_len, l) == -1) return -1;
+    i_data.f = *fi;
+    strcpy(i_data.name, fn);
+    if (add_to_info(fi, fn, TRUE) == -1) return -1;
     x = sizeof(*fi)+fi->name_len;
-    ifd.info_file_size += x;
     mktr += x; mktrc += x;
     if (S_ISLNK(fi->mode) || S_ISREG(fi->mode)) {
 	mktr += fi->act_size;
 	mktrc += fi->size;
     }
     
-    ifd.no_in_archive++;
     if (fi->checksum == -1) {
 	sprintf(l, "There was a problem when backing up file %s, therefore file is ignored", fn);
 	write_warning_log(l);
@@ -67,38 +59,62 @@ PRIVATE _errstat mkinfo_action(struct file_info *fi, char *fn, struct oa_file_en
 
 PUBLIC _s32 file_no;
 PUBLIC _s8 passing;				 /* for cumulative  */
+PRIVATE WINDOW *s_mes_box;
+PRIVATE _s32 s_cur_in_vol, s_no_in_vol, s_vol, s_file_no;
+PRIVATE _u32 s_file_size;
+PRIVATE char *s_fn;
+PRIVATE time_t s_t_start, s_t_current;
+
 PUBLIC void mkinfo_print_status(WINDOW *mes_box, _s32 cur_in_vol, 
 			 _s32 no_in_vol, _s32 vol,
 			 _u32 file_size, char *fn,
 			 time_t t_start, time_t t_current)
 {
+    if (restore_mode != RESTORE_VERIFY) file_no++;   /* in verify, will be updated in restore */
+    s_file_no = file_no;
+    s_mes_box = mes_box;
+    s_cur_in_vol = cur_in_vol;
+    s_no_in_vol = no_in_vol;
+    s_vol = vol;
+    s_file_size = file_size;
+    s_fn = fn;
+    s_t_start = t_start; s_t_current = t_current;
+    return;
+}
+
+
+PRIVATE void mkinfo_alarm(int sig)
+{
+    /* Updates the display */
+    
     char num1[50], scr[MAX_FNAME], num2[50];
     _s32 x;
     
-    (passing == TRUE) ? sprintf(scr, "Passing %s", fn) :
-                        sprintf(scr, "Reading %s", fn);
-    status_box(mes_box, scr, 0, FALSE, 1);
-    if (restore_mode != RESTORE_VERIFY) file_no++;   /* in verify, will be updated in restore */
-    sprintf(scr, "Volume %d. File %d of %d", vol, cur_in_vol, no_in_vol);
-    status_box(mes_box, scr, 2, FALSE, 1);
-    sprintf(scr, "Total read: file %d, size %s", file_no, print_kb(num1, mktr));
-    status_box(mes_box, scr, 3, FALSE, 1);
-    sprintf(scr, "Time elapsed %s", convtime(num1, t_start, t_current));
-    status_box(mes_box, scr, 5, FALSE, 1);
-    x=t_current-t_start;
+    signal(SIGALRM, SIG_IGN);
+    (passing == TRUE) ? sprintf(scr, "Passing %s", s_fn) :
+                        sprintf(scr, "Reading %s", s_fn);
+    convert_to_printable (scr);
+    status_box(s_mes_box, scr, 0, FALSE, 1);
+    sprintf(scr, "Volume %d. File %d of %d", s_vol, s_cur_in_vol, s_no_in_vol);
+    status_box(s_mes_box, scr, 2, FALSE, 1);
+    sprintf(scr, "Total read: file %d, size %s", s_file_no, pr_size(num1, mktr));
+    status_box(s_mes_box, scr, 3, FALSE, 1);
+    sprintf(scr, "Time elapsed %s", convtime(num1, s_t_start, s_t_current));
+    status_box(s_mes_box, scr, 5, FALSE, 1);
+    x=s_t_current-s_t_start;
     if (x) {
 	(passing == TRUE) ?
 	sprintf(scr, "Mkinfo rate %s/min [%s/min]",
-		print_mb(num1, mktr/x*60),
-		print_mb(num2, mktrc/x*60)) :
+		pr_size(num1, mktr/x*60),
+		pr_size(num2, mktrc/x*60)) :
 	sprintf(scr, "Restore rate %s/min [%s/min]",
-		print_mb(num1, mktr/x*60),
-		print_mb(num2, mktrc/x*60));
-	status_box(mes_box, scr, 7, FALSE, 1);
+		pr_size(num1, mktr/x*60),
+		pr_size(num2, mktrc/x*60));
+	status_box(s_mes_box, scr, 7, FALSE, 1);
     }
+    signal(SIGALRM, mkinfo_alarm);
 }
 
-
 PUBLIC _errstat mkinfoce(struct file_info *fi, char *fn)
 {
 /* Function called by traverse if checksum error encounterd 
@@ -112,10 +128,9 @@ PUBLIC _errstat mkinfoce(struct file_info *fi, char *fn)
     my_strcpy(s[1], fn, 60);
     strcpy(s[2], "");
     sprintf(s[3], "Assume aborted backup");
-    if (multi_message_box(s, 4, MB_YESNO) == 1) {
+    if (multi_message_box(s, 4, MB_YESNO, TRUE) == 1) {
 	x = sizeof(*fi)+fi->name_len;		 /* remove things placed into info file */
 	lseek(info_fd, -x, SEEK_CUR);		 /* move back pointer */
-	ifd.info_file_size -= x;
 	ifd.no_in_archive--;
 	return -1;
     }
@@ -225,11 +240,13 @@ PUBLIC _errstat mkinfo_loop(file_passed_action action, print_status ps)
 			ifd.number_volumes, vh.no_in_volume, vh.no_sels);
 		write_log(s);
 	    }
+	    set_1s_timer(); signal(SIGALRM, mkinfo_alarm);
 	    if (traverse_volume(action, vh.no_in_volume, t_start, 
 				mes_box, 1, ps, &in_vol, FALSE, 
 				(action == mkinfo_action) ? mkinfoce : NULL) < 0) /* traverse this volume */
 	      goto err;
 	}
+	reset_timer();
 	memcpy(&v, get_vh(vol_headers, cur_vol+1), sizeof(struct volume_header));   /* needed to ensure */
 	v.no_in_volume = in_vol;		 /* aligned access */
 	memcpy(get_vh(vol_headers, cur_vol+1), &v, sizeof(struct volume_header));
@@ -256,6 +273,7 @@ PUBLIC _errstat mkinfo_loop(file_passed_action action, print_status ps)
     return 0;
     
     err:;
+      reset_timer();
       close_statusbox(mes_box);
       return -1;
 }
@@ -264,15 +282,14 @@ PUBLIC _errstat mkinfo_loop(file_passed_action action, print_status ps)
 PRIVATE _errstat recreate(void) 
 {
     WINDOW *mes_box=NULL;
-    char   ifn[MAX_FNAME];
 
     passing = TRUE;
-    info_fd = 0;
     mes_box = status_box(mes_box, "Rewinding tape", 2, TRUE, 4);/* create window */
+    t_start=time(NULL);
     if (tape_rewind() == -1) goto err;
     status_box(mes_box, "Identifying tape", 2, FALSE, 1);
-    if (tape_open(O_RDWR) == -1) goto err;	 /* open rewinding device */
-    if (tape_readheader(&tdh, 0) == -1) goto err;
+    if (tape_open(O_RDONLY) == -1) goto err;	 /* open rewinding device */
+    if (tape_readheader(&tdh, 0, O_RDONLY) == -1) goto err;
     if ((tdh.magic != TAPER_MAGIC_NUMBER) && (tdh.magic != TAPER_64_MAGIC)) {
 	if (tdh.magic == TAPER_4_MAGIC_NUMBER)
 	  message_box("Sorry, taper 4 archives are not compatible", MB_OK);
@@ -280,77 +297,30 @@ PRIVATE _errstat recreate(void)
 	goto err;
     }
 
-    memset(&ifd, 0, sizeof(ifd));		 /* clear all variables */
+    clear_ifd();
     ifd.archive_id = tdh.archive_id;
     strcpy(ifd.archive_title, tdh.archive_title);
-    ifd.number_tapes = 1;
-    ifd.info_file_size = sizeof(ifd);
-    ifd.no_in_archive = 0;
-    ifd.number_tsi = 0;
     update_tsi = 1;
-    ifd.number_volumes = 1;
+    ifd.number_tapes = 1;
     print_title_line();				 /* refresh title line */
-    if ((info_fd = open_info_file(FALSE, tdh.archive_id)) == -1) goto err;
+    if (open_info_file(FALSE, tdh.archive_id, TRUE) == -1) goto err;
     vt_info = my_realloc(vt_info, sizeof(struct volume_tape_info));
     vt_info->volume = 1;			 /* first volume always on tape 1 */
     vt_info->start_tape = 1;
 
-    if (write_info(&ifd, sizeof(ifd), "Writing info file header") == -1) {
-	do_exit(ERROR_WRITING_INFO);
-	goto err;
-    }
-
     if (mkinfo_loop(mkinfo_action, mkinfo_print_status) == -1) {
 	if (log_level > 2) write_log("Mkinfo loop returned an error");
 	goto err;
     }
     touchwin(mes_box);
-    status_box(mes_box, "Closing & Rewinding tape", 2, FALSE, 1);
-    if (log_level > 2) write_log("Writing volume header information");
-    if (write(info_fd, vol_headers, ifd.size_volume_headers) == -1) {
-	do_exit(ERROR_WRITING_INFO);
-	goto err;
-    }
-    if (log_level > 2) write_log("Writing VT information");
-    if (write(info_fd, vt_info, sizeof(struct volume_tape_info)*ifd.number_volumes) == -1) {
-	do_exit(ERROR_WRITING_INFO);
-	goto err;
-    }
-    if (log_level > 2) write_log("Writing tape size information");
-    if (ifd.number_tsi)
-      if (write(info_fd, tsi, sizeof(struct tape_size_info)*ifd.number_tsi) == -1) {
-	  do_exit(ERROR_WRITING_INFO);
-	  goto err;
-      }
-    if (log_level > 2) write_log("Rewinding and writing header information");
-    lseek(info_fd, 0, SEEK_SET);		 /* update info file */
-    if (write_info(&ifd, sizeof(ifd), "Updating info file header") == -1) {
-	do_exit(ERROR_WRITING_INFO);
-	goto err;
-    }
-    if (log_level > 2) write_log("Closing info file");
-    close(info_fd);				 /* close info file */
-#ifdef TAPER_BIG_ENDIAN    
-/* If using big endian, the info file has to be converted
- * to little endian */
-    status_box(mes_box, "Endianizing info file", 2, FALSE, 1);
-    info = my_realloc(info, ifd.info_file_size);
-    if ((info_fd = open_info_file(TRUE, tdh.archive_id)) == -1) goto err;
-    if (read(info_fd, info, ifd.info_file_size) != ifd.info_file_size) goto err;
-    close(info_fd);
-    if (write_out_info_file(info, ifd.archive_id) == -1) goto err;
-#endif    
-    if (compress_info) {
-	compress_info_file(ifd.archive_id);
-	status_box(mes_box, "Compressing info file", 2, FALSE, 1);
-    }
+    status_box(mes_box, "Closing/Compressing info file", 2, FALSE, 1);
+    write_info_file();
+    close_info_file();
     close_statusbox(mes_box);
     return 0;
     
     err:;
-	if (info_fd) close(info_fd);
-	make_info_filename(ifn, tdh.archive_id); /* erase info file */
-	unlink(ifn);
+        del_info_file(ifd.archive_id);
         if (mes_box) close_statusbox(mes_box);
 	return -1;   
 }
@@ -361,8 +331,8 @@ PRIVATE _errstat mkinfo_do_mallocs(void)
     vt_info = my_malloc(1);
     tsi = my_malloc(1);
     vol_headers = my_malloc(sizeof(_s32));
-    info = my_malloc(1);
-    if ((info == NULL) || (vt_info == NULL) || (vol_headers == NULL)) return -1;
+    m_files = my_malloc(1);
+    if ((vt_info == NULL) || (vol_headers == NULL)) return -1;
     return 0;
 }
 
@@ -372,7 +342,7 @@ PRIVATE void mkinfo_free_memory(void)
     my_free(vt_info);
     my_free(tsi);
     my_free(vol_headers);
-    my_free(info);
+    my_free(m_files);
 }
 
 
@@ -397,24 +367,24 @@ void taper_mkinfo(void)
     strcpy(sa[1], "");
     sprintf(sa[2], "Found: %d files in  %d volumes, ", ifd.no_in_archive,
 	    ifd.number_volumes);
-    if (log_level > 1) write_log(sa[2]);
+    if (log_level) write_log(sa[2]);
     sprintf(sa[3], "Total on archive %s [%s]. Ratio %.2f", 
-	    print_mb(s2, mktr),
-	    print_mb(s3, mktrc),
+	    pr_size(s2, mktr),
+	    pr_size(s3, mktrc),
 	    (mktr == 0) ? 1 : (float) mktrc / (float) mktr);
-    if (log_level > 1) write_log(sa[3]);
+    if (log_level) write_log(sa[3]);
     strcpy(sa[4], "");
     sprintf(sa[5], "Time elapsed %s.", convtime(s2, t_start, t_current));
-    if (log_level > 1) write_log(sa[5]);
+    if (log_level) write_log(sa[5]);
 
     sprintf(sa[6], "Mkinfo rate %s/min [%s/min]", 
-	    print_mb(s2, mktr/x*60),
-	    print_mb(s3, mktrc/x*60));
-    if (log_level > 1) write_log(sa[6]);
+	    pr_size(s2, mktr/x*60),
+	    pr_size(s3, mktrc/x*60));
+    if (log_level) write_log(sa[6]);
     strcpy(sa[7], "");
     sprintf(sa[8], "%d warnings, %d errors", log_warnings, log_errors);
-    if (log_level > 1) write_log(sa[8]);
-    multi_message_box(sa, 9, MB_OK);
+    if (log_level) write_log(sa[8]);
+    multi_message_box(sa, 9, MB_OK, TRUE);
     mkinfo_free_memory();
     close_logfile("Mkinfo");
     return;			                 /* succesful return */
