@@ -1,12 +1,13 @@
-/*
-   Time-stamp: <96/08/05 19:57:22 yusuf>
 
-   $Id: restore.c,v 1.25 1996/08/05 19:02:01 yusuf Exp $	
+/*
+   Time-stamp: <97/07/06 12:14:54 yusuf>
+
+   $Id: restore.c,v 1.53 1997/07/06 04:26:25 yusuf Exp $	
 
 */
 
 #ifndef lint
-static char vcid[] = "$Id: restore.c,v 1.25 1996/08/05 19:02:01 yusuf Exp $";
+static char vcid[] = "$Id: restore.c,v 1.53 1997/07/06 04:26:25 yusuf Exp $";
 #endif /* lint */
 
 
@@ -27,27 +28,30 @@ PRIVATE FILE *fdfifo;				 /* for the FIFO */
 PRIVATE char fifo_name[MAX_FNAME];
 
 
-PRIVATE struct oa_file_entry *find_first_file(_s32 *cpos, struct oa_file_entry *fe)
+PRIVATE _s32 find_first_file(_s32 *cpos, struct info_file_data *i_data)
 {
 /* Looks through the currently selected files and works out
  * which is the first to be read in
  * 
  * Assumes archive directory is sorted in pos_in_archive order
  * so can break as soon as it finds a file to be restored
- * 
- * Starts search at fe
- * 
- * Returns a pointer to the file entry
+ *
+   Starts search at cpos
+   
+ * Returns the record # or -1 if error
+   The record is returned in i_data
 */
 
     while (*cpos <  no_in_archive) {
-	if (fe->selected)			 /* only have to go through archive*/
-	  if (S_ISREG(fe->i.mode) || (S_ISLNK(fe->i.mode))) /* to recreate reg files & */
-	    return fe;
-	advance_ofe(&fe);			 /* we have information on everything else */
+	if (check_selected(*cpos)) {
+	    if (read_info_rec(*cpos, i_data) == -1) return do_exit(ERROR_READING_INFO);
+	    if (S_ISREG(i_data->f.mode) || (S_ISLNK(i_data->f.mode))) /* to recreate reg files & */
+		return *cpos;
+	}
 	(*cpos)++;
     }
-    return NULL;
+    *cpos = -1;
+    return -1;
 }
 
 
@@ -63,7 +67,7 @@ PUBLIC _errstat restorece(struct file_info *fi, char *fn)
     my_strcpy(s[1], fn, 60);
     strcpy(s[2], "");
     sprintf(s[3], "Assume aborted backup");
-    if (multi_message_box(s, 4, MB_YESNO) == 1)
+    if (multi_message_box(s, 4, MB_YESNO, TRUE) == 1)
       return -1;
     return 0;
 }
@@ -72,7 +76,7 @@ PUBLIC _errstat restorece(struct file_info *fi, char *fn)
 PRIVATE _u32 rtotal_read;
 PRIVATE _u32 rtotal_processed;
 PRIVATE _u32 rtotal_files;
-PRIVATE _errstat restore_action(struct file_info *fi, char *fn, struct oa_file_entry *fe)
+PUBLIC _errstat restore_action(struct file_info *fi, char *fn, struct info_file_data *i_data)
 /* Acts on the restore loop finding filename fn
  * Action returns 1 if it advanced past file (ie. it read it in)
  * or 0 if it didn't read it in
@@ -90,9 +94,9 @@ PRIVATE _errstat restore_action(struct file_info *fi, char *fn, struct oa_file_e
 	rtotal_processed += fi->name_len+sizeof(struct file_info);   /* make sure we want this file */
 	if (S_ISREG(fi->mode))
 	  rtotal_processed += fi->act_size;
-	if (!fe->selected)			 /* not selected */
-	  return 0;				 /* tell loop that we didn't touch it */
-	fe->selected = 0;			 /* mark as unselected now that it's read */
+	if (!check_selected(i_data->recnum))
+	    return 0;				 /* tell loop that we didn't touch it */
+	(m_files+i_data->recnum)->selected = 0;	 /* unselect file now that we've read it */
 	ret = -3;
     }
     else
@@ -115,10 +119,7 @@ PRIVATE _errstat restore_action(struct file_info *fi, char *fn, struct oa_file_e
     switch(restore_mode) {
      case RESTORE_NORMAL:
 	rtotal_files++;				 /* increment file count */
-	strcpy(fn, stripped_cf);		 /* make filename as user */
-	if (*stripped_cf)			 
-	  strcat(fn, "/");			 /* wants it */
-	strcat(fn, name(fe));
+	strcpy(fn, &i_data->name[strip_cf]);
 	rtotal_read += fi->name_len+sizeof(struct file_info);    
 	break;
      case RESTORE_FULL:
@@ -185,9 +186,7 @@ PRIVATE _errstat restore_action(struct file_info *fi, char *fn, struct oa_file_e
 	  s[strlen(s)-1] = 0;
 	sprintf(l, "Creating directory %s", s);
 	if (log_level > 1) write_log(l);
-	if (mkdir(s, 493) == -1)
-	  if (errno != EEXIST)			 /* directory exists is not an error */
-	    write_error_log(l); 
+	make_path(s);
 	return setowners(s, ret, fi);
     }
 
@@ -204,7 +203,6 @@ PRIVATE _errstat restore_action(struct file_info *fi, char *fn, struct oa_file_e
 /* OK - this is a regular file. Write the details to the FIFO
  * and write the file to a temporary file */
 
-    fprintf(fdfifo, "%s\n", s); 		 /* write filename */
     taper_tmpnam(tmpf);				 /* make temporary name to store data in */
     if (log_level > 2) {
 	sprintf(l, "Writing %s to %s to FIFO", s, tmpf);
@@ -212,18 +210,16 @@ PRIVATE _errstat restore_action(struct file_info *fi, char *fn, struct oa_file_e
     }
     
     x = read_into_temp(fi, tmpf, s);
-    if (x == -1) 
-      fprintf(fdfifo, "%s\n", FIFO_ERR);	 /* couldn't create temp file */
+    if (x == -1)
+	fifo_send_error (fdfifo);	 	 /* couldn't create temp file */
     if (x == -2) {
 	if (restore_mode != RESTORE_NORMAL)
 	  if (restorece(fi, s) == -1) 		 /* user wants to abort this volume */
 	      return -4;
     }
-    if (x != -1) {
-	fprintf(fdfifo, "%s\n", tmpf);		 /* queue it for child to process */
-	fwrite(fi, sizeof(*fi), 1, fdfifo);	 /* write file info */
-    }
-    fflush(fdfifo);
+    if (x != -1)
+	fifo_send_file_with_info (fdfifo, fi, s, tmpf);
+
     if (ret == -4) return ret;
     return (ret == -3) ? -2 : 1;		 /* we passed file */
 }
@@ -278,46 +274,28 @@ PRIVATE _errstat restore_devs_dirs(void)
 */
 {
     _s32 c;
-    struct oa_file_entry *fe;
+    struct info_file_data i_data;
     char fn[MAX_FNAME];
     
-    fe = archive_files;
     for (c=0; c<no_in_archive; c++) {
-	if (fe->selected) 
-	    if (!(S_ISREG(fe->i.mode) || S_ISLNK(fe->i.mode))) {
-		strcpy(fn, cf);			 /* make the full pathname */
-		strcat(fn, name(fe));
-		if (restore_action(&fe->i, fn, fe) == -1) 
+	if (read_info_rec(c, &i_data) == -1) return do_exit(ERROR_READING_INFO);;
+	if (check_selected(i_data.recnum))
+	    if (!(S_ISREG(i_data.f.mode) || S_ISLNK(i_data.f.mode))) {
+		strcpy(fn, i_data.name);
+		if (restore_action(&i_data.f, fn, &i_data) == -1) 
 		  return -1;
 	    }
-	advance_ofe(&fe);
     }
     return 0;
 }
 
 
 PRIVATE time_t t_start, t_current;		 /* for time stats */
-PRIVATE _errstat start_restore(WINDOW *mes_box)
+PUBLIC _errstat restore_start_child(WINDOW *mes_box)
 {
-/* Does the initial things for restore:
- * 
- * reads tape header, checks proper archive,
- * sets up child process etc..
- * 
- * Returns 0 if all OK, -1 otherwise
- * 
-*/
     char l[200];
     char s2[30], s3[30], s4[30], s5[10];
 
-    file_no = 0;
-    status_box(mes_box, "Identifying tape", 2, FALSE, 1);
-    if (tape_rewind() == -1) return -1;		 /* rewind tape */
-    if (tape_open(O_RDONLY) == -1)		 /* open rewinding */
-	return -1;
-    if (tape_readheader(&tdh, 0) == -1) return -1;   /* read tape header */
-    if ((tdh.magic != TAPER_MAGIC_NUMBER) && (tdh.magic != TAPER_64_MAGIC))
-	return do_exit(ERROR_MAGIC);
     taper_tmpnam(fifo_name);			 /* set up FIFO */
     if ((mknod(fifo_name, S_IFIFO|S_IREAD|S_IWRITE, 0)) == -1)  
 	return do_exit(ERROR_CREATING_FIFO);
@@ -334,7 +312,7 @@ PRIVATE _errstat start_restore(WINDOW *mes_box)
 	return do_exit(ERROR_UNABLE_FORK);
     }
     if (restore_child == 0) {			 /* this is the child */
-	close(dv);
+	rmtclose(dv);
 	chdir(original_cur_dir);
 #ifdef TRIPLE_BUFFER
 	sprintf(l, "R:Executing bg_restore %s %d %s %d %d %d \"%s\"",
@@ -377,12 +355,34 @@ PRIVATE _errstat start_restore(WINDOW *mes_box)
 }
 
 
-PRIVATE void end_restore(void)
+PUBLIC _errstat start_restore(WINDOW *mes_box)
+{
+/* Does the initial things for restore:
+ * 
+ * reads tape header, checks proper archive,
+ * sets up child process etc..
+ * 
+ * Returns 0 if all OK, -1 otherwise
+ * 
+*/
+    file_no = 0;
+    status_box(mes_box, "Identifying tape", 2, FALSE, 1);
+    if (tape_rewind() == -1) return -1;		 /* rewind tape */
+    if (tape_open(O_RDONLY) == -1)		 /* open rewinding */
+	return -1;
+    if (tape_readheader(&tdh, 0, O_RDONLY) == -1) return -1;   /* read tape header */
+    if ((tdh.magic != TAPER_MAGIC_NUMBER) && (tdh.magic != TAPER_64_MAGIC))
+	return do_exit(ERROR_MAGIC);
+    return restore_start_child(mes_box);
+}
+
+
+PUBLIC void end_restore(void)
 {
 /* Closes tape, tells child we are finished etc..
 */
-    if (fdfifo != NULL)	{			 /* tell FIFO we are finished */
-	fprintf(fdfifo, "0\ndummy\nEND\n");
+    if (fdfifo != NULL) {			 /* tell FIFO we are finished */
+	fifo_send_end (fdfifo);
 	fclose(fdfifo);
     }
     if (restore_child) waitpid(restore_child, NULL, 0);/* wait for child to finish */
@@ -394,9 +394,9 @@ PRIVATE void end_restore(void)
 								     
 
 
-int calc_file_pos(_s32 *lpos, struct oa_file_entry **cur_fe, struct oa_file_entry *fe, _s32 *tape, _s32 *block, _s32 *offset)
+int calc_file_pos(_s32 *lpos, _s32 *cur_d, struct info_file_data *d, _s32 *tape, _s32 *block, _s32 *offset, _s8 new_vol)
 {
-/* Works out exactly where abouts 'fe' is in an archive:
+/* Works out exactly where abouts  #'d' is in an archive:
  * 
  *  on what tape, on what volume, on what block within that tape 
  *   (block relative to start of volume) and how many bytes in that
@@ -405,56 +405,61 @@ int calc_file_pos(_s32 *lpos, struct oa_file_entry **cur_fe, struct oa_file_entr
  * If we 'position' the tape (ie. by asking for a tape that
 			      is mid volume)
  * then we return 1
+
+    lpos = file position currently calculated to - if -1, then first call to f(x)
+    cur_d = file we want position of
+    d = data of cur_d
+    tape, block, offset = position parameters
+
+    if new_vol, then assumes we are starting a new volume
  * 
- * if cur_fe is not NULL, then assumes that we have already found the
- * position of cur_fe and it is in block, tape etc..
- * 
- * 
- * Of course, assumes that info file is in memory
+ *
+ * Of course, assumes that info file is open
 */
     struct volume_tape_info *vti=vt_info;
-    struct oa_file_entry *fe1;
     struct volume_header *vh;
+    struct info_file_data i_data;
     int  mw;
-    _s32 c;
+    _s32 c, rec;
     
     while (1) {					 /* find on what tape */
-	if (vti->volume == fe->i.volume) break;	 /* this vol begins */
+	if (vti->volume == d->f.volume) break;	 /* this vol begins */
 	vti++;
     }
-    if (*cur_fe == NULL) {			 /* start search at beginning */
+    if ((*lpos == -1) || (new_vol)) {		 /* start search at beginning */
 	*tape = vti->start_tape;
 	*block = 0;
-	vh = (struct volume_header *) get_vh(vol_headers, fe->i.volume);
+	vh = (struct volume_header *) get_vh(vol_headers, d->f.volume);
 	*offset = vh->size_header;
-	if (fe->i.volume == 1)
+	if (d->f.volume == 1)
 	  *offset += sizeof(struct tape_header);
     }
-    fe1 = (*cur_fe == NULL) ? archive_files : *cur_fe;
-    if (*cur_fe == NULL) *lpos = 0;
-    while (*lpos < fe->i.pos_in_archive-1) { /* calc size of files prior to this */
-	if (fe1->i.volume == fe->i.volume) {
-	    *offset += fe1->i.name_len;
+    *lpos = (*lpos == -1) ? 0  : *lpos;
+    rec = *lpos;
+    while (rec < ifd.no_in_archive) {
+	if (read_info_rec(rec, &i_data) == -1) return do_exit(ERROR_READING_INFO);
+	if (i_data.f.pos_in_archive >= d->f.pos_in_archive) break;
+	if (abs(i_data.f.volume) == abs(d->f.volume)) {
+	    *offset += i_data.f.name_len;
 	    *offset += sizeof(struct file_info);
-	    if (!S_ISDIR(fe1->i.mode)) 
-	      if ((fe1->i.checksum != -1)  
-	         && !((fe1->i.checksum == -2) && (!old_archive)) )
-	      *offset += fe1->i.act_size;
+	    if (!S_ISDIR(i_data.f.mode)) 
+		if ((i_data.f.checksum != -1)  
+		    && !((i_data.f.checksum == -2) && (!old_archive)) )
+		    *offset += i_data.f.act_size;
 	    if (*offset/block_size) {
 		*block  += *offset/block_size;
 		*offset = *offset%block_size;
 	    }
 	}
-	advance_ofe(&fe1);
-	(*lpos)++;
+	rec++;
     }
-    *cur_fe = fe1;
+    *lpos = rec;				 /* this is what we got up to */
 /* We now now how many blocks into the volume we need to go */
 /* We now have to work out whether we have to use a different tape */
     c = 0;
     mw = 0;
     while (c < ifd.number_tsi) {
-	if (tsi[c].volume == fe->i.volume) {
+	if (tsi[c].volume == d->f.volume) {
 	    if (*block < tsi[c].blocks-1) return mw;/* this block is on this tape */
 	    if (*block == tsi[c].blocks-1) {	 /* accounts for want something from last block */
 		if (*offset <= block_size - tsi[c].lb_bytes_short) return mw;
@@ -478,12 +483,12 @@ PRIVATE void do_restore(void)
 {
     WINDOW *mes_box=NULL;
     char l[200];
-    struct oa_file_entry *fe, *cur_fe, *fcur_fe;
-    _s32 tape, block, offset, at_vol, lpos, flpos, z=0, lvol;
+    _s32 tape, block, offset, at_vol, lpos, z=0, cur_d;
     char fn[MAX_FNAME];
     struct file_info fi;
     char sa[6][150];
     char s2[30];
+    struct info_file_data i_data;
     
 /* An error reading from the backup device is a fatal error */
     if ((!no_sel) && (restore_mode == RESTORE_NORMAL))
@@ -492,37 +497,27 @@ PRIVATE void do_restore(void)
     rtotal_processed = 0;
     rtotal_files = 0;
     mes_box = status_box(mes_box, "                                         ", 2, TRUE, 5);/* create window */
-    if (sort_dir) {
-	status_box(mes_box, "Sorting archive directory", 2, FALSE, 1);
-	do_sort_dir(0);
-    }
     nodelay(mes_box, TRUE);
     status_box(mes_box, "Restoring devices/directories", 2, FALSE, 1);
     if (restore_devs_dirs() == -1) return;
-    fe = find_first_file(&z, archive_files);	 /* get first file to restore */
-    if (fe == NULL) return;			 /* no files to restore */
+    find_first_file(&z, &i_data);		 /* get first file to restore */
+    if (z==-1) return;				 /* no files to restore */
     if (start_restore(mes_box) == -1) return;	 /* do initial things */
     
     at_vol = 1;
-    lvol = 0;
-    fcur_fe = archive_files; flpos = 0;
+    lpos = -1;
+    cur_d = 0;
+    status_box(mes_box, "", 5, FALSE, 1);
     while (1) {
-	fe = find_first_file(&flpos, fcur_fe);
-	fcur_fe = fe;
-	if (fe == NULL) goto fin;		 /* no files to restore */
-	if (fe->i.volume != lvol) cur_fe = NULL; /* reset search */
-	status_box(mes_box, "", 0, FALSE, 1);
-	status_box(mes_box, "", 2, FALSE, 1);
-	status_box(mes_box, "", 4, FALSE, 1);
-	status_box(mes_box, "", 5, FALSE, 1);
-	status_box(mes_box, "", 6, FALSE, 1);
-	status_box(mes_box, "", 7, FALSE, 1);
-	if (calc_file_pos(&lpos, &cur_fe, fe, &tape, &block, &offset))/* work out exactly where this file is */
-	  at_vol = fe->i.volume;
+	find_first_file(&cur_d, &i_data);
+	if (cur_d == -1) goto fin;		 /* no file to restore */
+	if (calc_file_pos(&lpos, &cur_d, &i_data, &tape, &block, &offset,
+			  at_vol != i_data.f.volume))/* work out exactly where this file is */
+	    at_vol = i_data.f.volume;
 	if (check_tape(mes_box, 3, tape) == -1) goto fin;
-	if (at_vol != fe->i.volume) 		 /* position on correct volume */
-	    if (goto_end_vol(mes_box, 3, fe->i.volume-1, at_vol, 0, FALSE) == -1) goto fin;
-	at_vol = fe->i.volume;
+	if (at_vol != i_data.f.volume) 		 /* position on correct volume */
+	    if (goto_end_vol(mes_box, 3, i_data.f.volume-1, at_vol, 0, O_RDONLY) == -1) goto fin;
+	at_vol = i_data.f.volume;
 	if (tape_goto_block(mes_box, 3, block) == -1) goto fin;
 	if ((read_offset != offset) && (offset)) {
 	    if (read_offset > offset) {
@@ -538,16 +533,16 @@ PRIVATE void do_restore(void)
 	    if (tape_read(write_buffer, offset - 
 			  ((read_offset == -1) ? 0 : read_offset)) == -1) goto fin;
 	}
-	restore_print_status(mes_box, 0, 0, at_vol, fe->i.size, name(fe),
+	restore_print_status(mes_box, 0, 0, at_vol, i_data.f.size, i_data.name,
 			     t_start, t_current);
 	if (tape_read_fi(&fi) == -1) goto fin;
 	if (tape_read(fn, fi.name_len) == -1) goto fin;
-	restore_action(&fi, fn, fe);
+	restore_action(&fi, fn, &i_data);
     }
     
     status_box(mes_box, "", 0, FALSE, 1);
-    status_box(mes_box, "Closing & Rewinding tape", 3, FALSE, 1);
     status_box(mes_box, "", 2, FALSE, 1);
+    status_box(mes_box, "Closing & Rewinding tape", 3, FALSE, 1);
     status_box(mes_box, "", 4, FALSE, 1);
     status_box(mes_box, "", 5, FALSE, 1);
     status_box(mes_box, "", 6, FALSE, 1);
@@ -559,25 +554,25 @@ PRIVATE void do_restore(void)
     strcpy(sa[1], "");
     sprintf(sa[2], "Restored: %d files,  %s", rtotal_files,
 	    print_kb(s2, rtotal_read));
-    if (log_level > 1) write_log(sa[2]);
+    if (log_level) write_log(sa[2]);
     t_current = time(NULL);
     sprintf(sa[3], "Time elapsed %s.", convtime(s2, t_start, t_current));
-    if (log_level > 1) write_log(sa[3]);
+    if (log_level) write_log(sa[3]);
     strcpy(sa[4], "");
 #ifdef TRIPLE_BUFFER
     sprintf(sa[5], "%d warnings, %d errors", log_warnings, log_errors);
 #else						 /* if not using triple buffering */
     sprintf(sa[5], "Can't give warning/error count");
 #endif    
-    if (log_level > 1) write_log(sa[5]);
-    multi_message_box(sa, 6, MB_OK);
+    if (log_level) write_log(sa[5]);
+    multi_message_box(sa, 6, MB_OK, TRUE);
 }
 
 
     
 PUBLIC _errstat read_vol_dir(_u32 archive_id) 	 /* also in utils.c */
 {
-    if (do_read_vol_dir(archive_id, tape, O_RDONLY, TRUE, TRUE) ==  TAPE_EXIST)
+    if (do_read_vol_dir(archive_id, tape, O_RDONLY, TRUE, TRUE, TRUE) ==  TAPE_EXIST)
       return 1;
     return -1;
 }
@@ -664,12 +659,16 @@ PRIVATE void print_diff(void)
 PRIVATE void print_voldir(void) 
 {
     _s32 c;
-    struct oa_file_entry *fe;
     char   s[MAX_FNAME], s1[20], s2[20], *xx;
     struct tm t;
     struct volume_header vh;
-    _s32   c_sz=0, u_sz=0;
+    _s32   c_sz, u_sz;
+    struct info_file_data i_data;
+    struct info_file_key i_key;
+    _s32 x;
 
+    printf("Sorted in position in archive\n");
+    printf("-----------------------------\n\n");
 
     printf("Number volumes = %d\n", ifd.number_volumes);
 
@@ -687,34 +686,78 @@ PRIVATE void print_voldir(void)
     }
     printf("\n\n%-25s %3s %5s %14s %12s %12s\n", "Pathname", "Vol", "Pos", "Backup", "File Size", "On tape");
     printf("----------------------------------------------------------------------------\n");
-    fe = archive_files;
-    for (c=0; c<ifd.no_in_archive; c++) {
-	strcpy(s, name(fe)); s[22] = 0;
-	t = *localtime(&fe->i.backup_time);
-	printf("%-25s %3d %5d %2d/%2d/%2d %2d:%02d %12s %12s\n", s, abs(fe->i.volume), fe->i.pos_in_archive,t.tm_mday,
-	       t.tm_mon+1, t.tm_year, t.tm_hour, t.tm_min, convert(s1, fe->i.size),
-	       convert(s2, fe->i.act_size));
-	c_sz += fe->i.act_size;
-	u_sz += fe->i.size;
-	advance_ofe(&fe);
+
+    c_sz = 0; u_sz = 0;
+    for (x=0; x<ifd.no_in_archive; x++) {
+	read_info_rec(x, &i_data);
+	if (i_data.f.pos_in_archive) {
+	    pr_filename(i_data.name, s, 25);
+	    t = *localtime(&i_data.f.backup_time);
+	    printf("%-25s %3d %5d %2d/%2d/%2d %2d:%02d %12s %12s\n", s, abs(i_data.f.volume), i_data.f.pos_in_archive,t.tm_mday,
+		   t.tm_mon+1, t.tm_year, t.tm_hour, t.tm_min, convert(s1, i_data.f.size),
+		   convert(s2, i_data.f.act_size));
+	    c_sz += i_data.f.act_size;
+	    u_sz += i_data.f.size;
+	}
+
+    }
+    printf("\n%-47s    %12s %12s\n", "TOTALS", convert(s1, u_sz), convert(s2, c_sz));
+
+
+    printf("\n\nSorted in alphabetical\n");
+    printf("-----------------------------\n\n");
+
+    printf("Number volumes = %d\n", ifd.number_volumes);
+
+    c_sz = 0; u_sz = 0;
+    for (c=0; c<ifd.number_volumes;c++) {
+	xx = (char *) get_vh(vol_headers, c+1);
+	memcpy((void *) &vh, xx, sizeof(struct volume_header));
+	if (*vh.volume_title)
+	  printf("\nVolume %d %s\n", c+1, vh.volume_title);
+	else
+	  printf("\nVolume %d <no title>\n", c+1);
+	printf("  Contains %d files\n", vh.no_in_volume);
+	t = *localtime(&vh.backup_time);
+	printf("  Backed up at %d/%d/%d %d:%d\n", t.tm_year, t.tm_mon+1, t.tm_mday,
+		t.tm_hour, t.tm_min);
+    }
+    printf("\n\n%-25s %3s %5s %14s %12s %12s\n", "Pathname", "Vol", "Pos", "Backup", "File Size", "On tape");
+    printf("----------------------------------------------------------------------------\n");
+    
+    x = ntraverse(0, &i_key, NULL, &c, TRAVERSE_TOP, INFO_NAME);
+    while (!x) {
+	read_info_rec(c, &i_data);
+	if (i_data.f.pos_in_archive) {
+	    strcpy(s, i_data.name); s[22] = 0;
+	    t = *localtime(&i_data.f.backup_time);
+	    printf("%-25s %3d %5d %2d/%2d/%2d %2d:%02d %12s %12s\n", s, abs(i_data.f.volume), i_data.f.pos_in_archive,t.tm_mday,
+		   t.tm_mon+1, t.tm_year, t.tm_hour, t.tm_min, convert(s1, i_data.f.size),
+		   convert(s2, i_data.f.act_size));
+	    c_sz += i_data.f.act_size;
+	    u_sz += i_data.f.size;
+	}
+	x = ntraverse(0, &i_key, NULL, &c, TRAVERSE_CONTINUE, INFO_NAME);
+
     }
     printf("\n%-47s    %12s %12s\n", "TOTALS", convert(s1, u_sz), convert(s2, c_sz));
     return;
 }
 
 	
-PRIVATE void restore_mfn(struct direntry *x, char *dir_name, char *prefix)
+PRIVATE char restore_mfn(struct direntry *x, char *dir_name, char *prefix)
 {
     char t[MAX_FNAME], pth[MAX_FNAME];
     int  fd;
     struct info_file_header ifd;
-    
+
     strcpy(t, "ID ");
     strcat(t, &x->entry.d_name[strlen(prefix)]);
     strcpy(pth, dir_name);
     if (pth[strlen(pth)-1] != '/')
       strcat(pth, "/");
     strcat(pth, x->entry.d_name);
+    if (pth[strlen(pth)-2] == '.') return FALSE;
     fd = open(pth, O_RDONLY);
     if (fd != -1) {
 	read(fd, &ifd, sizeof(ifd));
@@ -726,6 +769,7 @@ PRIVATE void restore_mfn(struct direntry *x, char *dir_name, char *prefix)
 	close(fd);
     }
     strcpy(x->entry.d_name, t);
+    return TRUE;
 }
 
 
@@ -738,7 +782,8 @@ PRIVATE void full_restore(void)
 
     backrest_kill_windows();
     print_my_name();
-    mes_box = status_box(mes_box, "Rewinding tape", 2, TRUE, 4);/* create window */
+    mes_box = status_box(mes_box, "Rewinding tape", 2, TRUE, 5);/* create window */
+    nodelay(mes_box, TRUE);
     if (start_restore(mes_box) == -1) return;
     ifd.archive_id = tdh.archive_id;		 /* so that title line prints correctly */
     ifd.number_tapes = -1;			 /* don't bother with updating etc.. */
@@ -750,8 +795,9 @@ PRIVATE void full_restore(void)
     sprintf(sam[0], "%s Finished",
 	    (restore_mode == RESTORE_VERIFY) ? "Verify": "Restore");
     strcpy(sam[1], "");
-    sprintf(sam[2], "%s : %d files,  %s", 
-	    (restore_mode == RESTORE_VERIFY) ? "Verified" : "Restored", file_no,
+    sprintf(sam[2], "%s : %d files, %d volumes - %s", 
+	    (restore_mode == RESTORE_VERIFY) ? "Verified" : "Restored",
+	    file_no, ifd.number_volumes,
 	    print_kb(s2, mktr));
     if (log_level > 1) write_log(sam[2]);
     strcpy(sam[3], "");
@@ -771,7 +817,7 @@ PRIVATE void full_restore(void)
     sprintf(sam[7], "Can't give warning/error count");
 #endif    
     if (log_level > 1) write_log(sam[7]);
-    multi_message_box(sam, 8, MB_OK);
+    multi_message_box(sam, 8, MB_OK, TRUE);
 }
 
 
@@ -799,7 +845,7 @@ PUBLIC _errstat select_archive(_u32 *archive_id, char noinfo) /* also used in ut
     if (gdh) {
 	if (!no_windows)
 	  mes = status_box(mes, "Rewinding tape", 1, TRUE,  1); /* see what tape is */
-	ret = get_tape_header(mes, 1, &th);	 /* in drive */
+	ret = get_tape_header(mes, 1, &th, O_RDONLY);	 /* in drive */
 	tape_close();
 	if (mes) close_statusbox(mes);
 	if (ret == -1) goto selfile;			 /* error getting tape */
@@ -856,6 +902,7 @@ PUBLIC void taper_restore(void)
     _s32  x;
     _u32 arch;
     _errstat sa=0;
+    WINDOW *mes_box=NULL;
 
     backrest_do_mallocs();
     if (check_device_names() == -1) goto end;	 /* check devices & other parms */
@@ -889,9 +936,10 @@ PUBLIC void taper_restore(void)
 	  tape_close();
       }
     else {
-	info = my_realloc(info, 1);		 /* free memory */
 	print_title_line();
 	if (ifd.no_in_archive) {
+	    if (!get_string(win_main, rel_path, sizeof(rel_path),
+			    "Location to restore files")) return;
 	    x = select_restore_files();
 	    switch(x) {
 	       case 0:				 /* user aborted */
@@ -915,13 +963,16 @@ PUBLIC void taper_restore(void)
 	backrest_kill_windows();
 	print_my_name();
 	chdir(original_cur_dir);		 /* change directory to ensure */
-	if (x) 
-	    do_restore();			 /* restore to proper place */
+	if (x) do_restore();			 /* restore to proper place */
     }
     
     end:;
     backrest_free_memory();
+    if (!no_windows) 
+	mes_box = status_box(mes_box, "Closing/Compressing info file", 2, TRUE, 3);
+    close_info_file();
     close_logfile(((restore_mode == RESTORE_VERIFY) ? "Verify archive" : "Restore"));
+    if (!no_windows) close_statusbox(mes_box);
     return;			                 /* succesful return */
 }
     
