@@ -1,19 +1,19 @@
 /*
-   Time-stamp: <96/07/19 20:18:22 yusuf>
+   Time-stamp: <97/08/11 18:49:59 yusuf>
 
-   $Id: select_box.c,v 1.19 1996/07/27 20:42:13 yusuf Exp $	
+   $Id: select_box.c,v 1.37 1997/08/11 12:32:05 yusuf Exp $	
 
 */
 
 #ifndef lint
-static char vcid[] = "$Id: select_box.c,v 1.19 1996/07/27 20:42:13 yusuf Exp $";
+static char vcid[] = "$Id: select_box.c,v 1.37 1997/08/11 12:32:05 yusuf Exp $";
 #endif /* lint */
 
 
 
+#include <signal.h>
 #include "select_box.h"
 
-
 extern WINDOW *win_main;
 WINDOW *dir_win;
 struct direntry      *sb_directory;		 /* for the select directory */
@@ -121,7 +121,7 @@ _s32 min(_s32 a, _s32 b) {
 int get_string(WINDOW *win, char *s, int maxlen, char *prompt)
 {
 /* Prints up a box and prompts for a string 
-   Returns 0 if pressed ESC, 1 otherwise
+   Returns 0 if pressed ESC or F10, 1 otherwise
  */   
     WINDOW *mes=NULL;
     FIELD *field[2];
@@ -169,7 +169,7 @@ int get_string(WINDOW *win, char *s, int maxlen, char *prompt)
 	 case 127:
 	    c = REQ_DEL_PREV; break;
 	}
-	if (c == '\r' || c == '\n' || c == KEY_F0+10) break;
+	if (c == '\r' || c == '\n' || c == KEY_F0+10 || c == 27) break;
 	form_driver(form, c);
 	wrefresh(form->w);
     }
@@ -180,7 +180,8 @@ int get_string(WINDOW *win, char *s, int maxlen, char *prompt)
     free_field(field[0]);
     my_delwin(mes);
     curs_set(0);
-    return (c != KEY_F0+10);
+    if (c == 27 || c==KEY_F0+10) return 0;
+    return 1;
 }
 
     
@@ -327,6 +328,13 @@ void mb_options(WINDOW *mes_box, int mes_box_lines, int type, int res, int bold)
 }
 
 
+void null_alarm(int sig)
+{
+    /* ignores alarm */
+
+    signal(SIGALRM, null_alarm);
+}
+
 int message_box_engine(WINDOW *mes_box, int type) {
 /* Prints a message box with the message 's'. The user has the option of
  * answering YES or NO. The box is printed in the middle of the screen
@@ -338,7 +346,13 @@ int message_box_engine(WINDOW *mes_box, int type) {
 */
     int     res=1, c;
     int     mes_box_lines=mes_box->_maxy;
-
+    void *old_signal;
+    
+    old_signal = signal(SIGALRM, null_alarm);	 /* suspend signal handler */
+    if (type == MB_NOYES) {			 /* no-yes means */
+	type = MB_YESNO;			 /* no is default */
+	res = 0;
+    }
     while (1) {
 	mb_options(mes_box, mes_box_lines, type, res, 0);
 	c = wgetch(mes_box);
@@ -359,6 +373,7 @@ int message_box_engine(WINDOW *mes_box, int type) {
     }
     mb_options(mes_box, mes_box_lines, type, res, 1);
     my_delwin(mes_box);
+    signal(SIGALRM, old_signal);	 /* restore signal handler */
     return res;
 }
 
@@ -377,7 +392,7 @@ int message_box(char *s, int type) {
     int screen_len=curscr->_maxy;
     int w;
 
-    w = min(max(strlen(s)+10, 15), screen_width-4);
+    w = min(max(strlen(s)+10, 15), screen_width-4); 
     mes_box = my_newwin(7, w, screen_len/2-4, (screen_width-w)/2);
     my_werase(mes_box, COLOR_DIALOG);
     wrefresh(mes_box);
@@ -387,13 +402,15 @@ int message_box(char *s, int type) {
 }
 
     
-int multi_message_box(char lns[][150], int count, int type) {
+int multi_message_box(char lns[][150], int count, int type, int do_centre) {
 /* Prints a message box with multiple lines. 
  * 
  * Each line is a string. Expect count strings. Each string is 150 long
  * 
  * Accepts       type          type of message box
+                 count         number of lines 
  *               lns           an array of strings
+                 centre        TRUE = centre text, FALSE = don't
  * Returns       1             if user answered yes
  *               0             if user answered no
 */
@@ -406,6 +423,7 @@ int multi_message_box(char lns[][150], int count, int type) {
 	max_len = max(max_len, strlen(lns[c]));   /* work out max string length */
 	c++;
     }
+    max_len = min(max_len, screen_width-15);
     mes_box = my_newwin(4+count, max_len+10,
 		     screen_len/2-(2+count/2),
 		     (screen_width-(max_len+10))/2);
@@ -414,13 +432,34 @@ int multi_message_box(char lns[][150], int count, int type) {
     keypad(mes_box, TRUE);
     c=0;
     while (c<count) {
-	centre(mes_box, 1+c, lns[c], COLOR_DIALOG);
+	if (do_centre)
+	    centre(mes_box, 1+c, lns[c], COLOR_DIALOG);
+	else 
+	    mvwaddstr(mes_box, 1+c, 1, lns[c]);
 	c++;
     }
     return message_box_engine(mes_box, type);
 }
 
     
+char *make_permstring(char *s, mode_t perms)
+{
+/* Makes a permission string rwxrwxrwx */
+
+    *s = 0;
+    strcat(s, (perms & S_IRUSR) ? "r" : "-");
+    strcat(s, (perms & S_IWUSR) ? "w" : "-");
+    strcat(s, (perms & S_IXUSR) ? "x" : "-");
+    strcat(s, (perms & S_IRGRP) ? "r" : "-");
+    strcat(s, (perms & S_IWGRP) ? "w" : "-");
+    strcat(s, (perms & S_IXGRP) ? "x" : "-");
+    strcat(s, (perms & S_IROTH) ? "r" : "-");
+    strcat(s, (perms & S_IWOTH) ? "w" : "-");
+    strcat(s, (perms & S_IXOTH) ? "x" : "-");
+    return s;
+}
+    
+
 void clear_sd(select_details *sd) {
     sd->top_entry = 0;
     sd->cur_entry = 0;
@@ -468,15 +507,67 @@ void print_scroll_bar(WINDOW *win, _s32 no_entries, select_details *sd,
     *p_scroll = 1;
 }
 
+
+void adjust_sd(WINDOW *win, select_details *sd, print_screen ps, char *p_scroll, _u32 *no_entries)
+{
+    /* Adjust sd */
+
+    int screen_len = win->_maxy;
+    int x=0;
+
+    while (sd->cursor_line < 1) {	       	 /* went off top of screen */
+	sd->top_entry--;
+	if (sd->top_entry < 0) {		 /* no entries off top */
+	    sd->top_entry = 0;
+	    sd->cur_entry++;
+	}
+	sd->cursor_line++;
+	x=1;
+    }
     
+    while (sd->cursor_line > screen_len-2) {	 /* went off bottom */
+	sd->top_entry++;
+	if (sd->top_entry + screen_len-2 > *no_entries) {
+	    sd->top_entry--;
+	    sd->cur_entry--;
+	}
+	sd->cursor_line--;
+	x=1;
+    }
+
+    if (sd->cur_entry > *no_entries-1) {	 /*  */
+	sd->cur_entry = *no_entries-1;
+	sd->cursor_line = sd->cur_entry - sd->top_entry + 1;
+	x=1;
+    }
+    if (x) ps(win, sd->top_entry, p_scroll);      
+}
+
+
+void adjust_cur_entry(WINDOW *win, select_details *sd, print_screen ps, char *p_scroll,
+		      _u32 *no_entries, _s32 cur_entry)
+{
+/* Adjusts the top_entry & cursor_line so that the cursor points
+   to the current entry */
+
+    if (sd->cur_entry == cur_entry) return;	 /* same - don't bother */
+    sd->cursor_line += (cur_entry - sd->cur_entry);
+    sd->cur_entry = cur_entry;
+    adjust_sd(win, sd, ps, p_scroll, no_entries);
+}
+
 int select_box(WINDOW *win, _s32 *no_entries,  select_details *sd,
-	       print_screen ps, print_line pl, tag_entry te, delete_entry de,
-	       refresh_screen rs,
+	       print_screen ps, print_line pl, tag_entry te, tag_entry tE,
+	       delete_entry de, refresh_screen rs, entry_details ed,
 	       WINDOW *sel_win, _s32 *sel_no_entries,
 	       select_details *sel_sd, print_screen sel_ps,
+	       WINDOW *excl_win, _s32 *excl_no_entries,
+	       select_details *excl_sd, print_screen excl_ps,
 	       print_screen asterix, select_details *filed,
 	       char *select_help, sr_backupset save_backupset,
-	       sr_restoreset rest_backupset) 
+	       sr_restoreset rest_backupset,
+	       int allow_search, int allow_goto,
+	       tag_entry ee, tag_entry eE, char *unselect_help)
 {
     _s32  c;
     int   screen_len, screen_width, teret;
@@ -499,22 +590,34 @@ int select_box(WINDOW *win, _s32 *no_entries,  select_details *sd,
 	wattroff(win, A_REVERSE);
 	pl(win, sd->cur_entry, sd->cursor_line, TRUE); 
 	switch(c) {
-	 case 'B':
-	 case 'b':
+	 case 'S':
+	 case 's':
 	    if (save_backupset) {
 		save_backupset(0); 
 		rs();
 		if (sel_ps) sel_ps(sel_win, sel_sd->top_entry, &p_scroll);
+		if (excl_ps) excl_ps(excl_win, excl_sd->top_entry, &p_scroll);
 	    }
 	    break;
-	 case 'L':
-	 case 'l':
+	 case 'R':
+	 case 'r':
 	    if (rest_backupset) {
 		rest_backupset();
 		rs();
 		if (sel_ps) sel_ps(sel_win, sel_sd->top_entry, &p_scroll);
+		if (excl_ps) excl_ps(excl_win, excl_sd->top_entry, &p_scroll);
 	    }
 	    break;
+	case 'L':
+	case 'l':
+	    if (allow_search)
+		return SELECT_SEARCH;
+	    break;
+	case 'J':
+	case 'j':
+	    if (allow_goto)
+		return SELECT_GOTO;
+	    break;
 	 case 'q':
 	 case 'Q':
 	 case 'a':
@@ -529,10 +632,13 @@ int select_box(WINDOW *win, _s32 *no_entries,  select_details *sd,
 	      return SELECT_FINISHED;
 	    rs();
 	    break;
+	case 'g':
+	case 'G':
+		
 	 case KEY_UP:
 	 case KEY_LEFT:
 	    sd->cursor_line--; sd->cur_entry--; break;
-	 case KEY_DOWN:				 /* plus fall through from above */
+	 case KEY_DOWN:				 
 	 case KEY_RIGHT:
 	    sd->cursor_line++; sd->cur_entry++; break;
 	 case '\t':
@@ -551,21 +657,43 @@ int select_box(WINDOW *win, _s32 *no_entries,  select_details *sd,
 		rs();
 	    }
 	    break;
-	 case 'S':				 /* only if tag function enabled */
- 	 case 's':
-	 case ' ':
-	    if (te) {
-		teret = te(sd->cur_entry);
-		pl(win, sd->cur_entry, sd->cursor_line, TRUE); 
-		if (teret == 1)			 /* need to position on last line */
-		  check_eop(sel_win, sel_sd, *sel_no_entries, sel_ps);
+	case 'd':				 /* entry details */
+	case 'D':
+	    if (ed) {
+		ed(sd);
 		rs();
-		if (sd->cur_entry + 1 < *no_entries) {
-		    sd->cursor_line++; 
-		    sd->cur_entry++; 
-		}
 	    }
-	    break;	 /* make it go down a line */
+	    break;
+	case 'i':				 /* only if tag function enabled */
+	case 'I':
+	case 'e':
+	case 'E':
+	    teret = 0;
+	    switch(c) {
+	    case 'i':
+		if (te) teret = te(sd->cur_entry);
+		break;
+	    case 'I':
+		if (tE) teret = tE(sd->cur_entry);
+		break;
+	    case 'e':
+		if (ee) teret = ee(sd->cur_entry);
+		break;
+	    case 'E':
+		if (eE) teret = eE(sd->cur_entry);
+		break;
+	    }
+	    pl(win, sd->cur_entry, sd->cursor_line, TRUE); 
+	    if (teret == 1)	{		 /* need to position on last line */
+		if (sel_win) check_eop(sel_win, sel_sd, *sel_no_entries, sel_ps);
+		if (excl_win) check_eop(excl_win, excl_sd, *excl_no_entries, excl_ps);
+	    }
+	    rs();
+	    if (sd->cur_entry + 1 < *no_entries) {
+		sd->cursor_line++; 
+		sd->cur_entry++; 
+	    }
+	    break;
 	 case KEY_PPAGE:
  	    if (sd->top_entry-(screen_len-4) >= 0) {
 		sd->top_entry -= screen_len-4;
@@ -597,32 +725,49 @@ int select_box(WINDOW *win, _s32 *no_entries,  select_details *sd,
 	    ps(win, sd->top_entry, &p_scroll);
 	    break;
 	 default:
-	    help = my_newwin(21, curscr->_maxx-10,
-			  (curscr->_maxy-20)/2,
+	    help = my_newwin(24, curscr->_maxx-10,
+			  (curscr->_maxy-24)/2,
 			  10/2);
 	    my_werase(help, COLOR_HELP);
 	    box(help, ACS_VLINE, ACS_HLINE);
-	    centre(help, 1, "Commands Available", COLOR_HELP);
-	    centre(help, 2, "------------------", COLOR_HELP);
-	    centre(help, 4, "Up arrow, Left arrow        move cursor up                     ", COLOR_HELP);
-	    centre(help, 5, "Down arrow, Right arrow     move cursor down                   ", COLOR_HELP);
-	    centre(help, 6, "Page up                     move cursor up a page              ", COLOR_HELP);
-	    centre(help, 7, "Page down                   move cursor down a page            ", COLOR_HELP);
-	    centre(help, 8, "End                         move cursor to end of screen       ", COLOR_HELP);
-	    centre(help, 9, "Home                        move cursor to beginning of screen ", COLOR_HELP);
-	    centre(help,10, "TAB                         move to next panel                 ", COLOR_HELP);
-	    centre(help,11, "H, h, ?                     print this screen                  ", COLOR_HELP);
-	    c=12;
+	    centre(help, 1, "Up arrow, Left arrow        move cursor up                     ", COLOR_HELP);
+	    centre(help, 2, "Down arrow, Right arrow     move cursor down                   ", COLOR_HELP);
+	    centre(help, 3, "Page up                     move cursor up a page              ", COLOR_HELP);
+	    centre(help, 4, "Page down                   move cursor down a page            ", COLOR_HELP);
+	    centre(help, 5, "End                         move cursor to end of screen       ", COLOR_HELP);
+	    centre(help, 6, "Home                        move cursor to beginning of screen ", COLOR_HELP);
+	    centre(help, 7, "TAB                         move to next panel                 ", COLOR_HELP);
+	    centre(help, 8, "H, h, ?                     print this screen                  ", COLOR_HELP);
+	    c=9;
 	    if (de)
 	     centre(help,c++,"U, u                        unselect                           ", COLOR_HELP);
 	    if (te) {
-		sprintf(s,  "S, s, space                 %-36s", select_help);
+		sprintf(s,  "i                           %-34s", select_help);
 		centre(help,c++,s,COLOR_HELP);
 	    }
-	    if (save_backupset) {
-	     centre(help,c++,"B, b                        Save file set                      ", COLOR_HELP);
-	     centre(help,c++,"L, l                        Load file set                      ", COLOR_HELP);
+	    if (tE) {
+		sprintf(s,  "I                           %-34s", select_help);
+		centre(help,c++,s,COLOR_HELP);
 	    }
+	    if (ee) {
+		sprintf(s,  "e                           %-34s", unselect_help);
+		centre(help,c++,s,COLOR_HELP);
+	    }
+	    if (eE) {
+		sprintf(s,  "E                           %-34s", unselect_help);
+		centre(help,c++,s,COLOR_HELP);
+	    }
+	    if (ed) 
+	     centre(help,c++,"D, d                        details about entry                ", COLOR_HELP);
+	    if (save_backupset) {
+	     centre(help,c++,"S, s                        Save file set                      ", COLOR_HELP);
+	     centre(help,c++,"R, r                        Restore file set                   ", COLOR_HELP);
+	    }
+	    if (allow_search)
+		centre(help,c++,"L, l                        Look for entry                     ", COLOR_HELP);
+	if (allow_goto)
+		centre(help,c++,"J, j                        Jump to a directory                ", COLOR_HELP);
+
 	    centre(help,c++,"Q, q, A, a                  abort                              ", COLOR_HELP);
 	    centre(help,c++,"F, f                        finish selecting and start         ", COLOR_HELP);
 	    centre(help,c+1, "Press any key...", COLOR_HELP);
@@ -632,29 +777,7 @@ int select_box(WINDOW *win, _s32 *no_entries,  select_details *sd,
 	    rs();
 	    break;
 	}
-	if (sd->cursor_line < 1) {	       	 /* went off top of screen */
-	    sd->top_entry--;
-	    if (sd->top_entry < 0) {		 /* no entries off top */
-		sd->top_entry = 0;
-		sd->cur_entry++;
-	    }
-	    ps(win, sd->top_entry, &p_scroll);      
-	    sd->cursor_line = 1;
-	}
-	
-	if (sd->cursor_line > screen_len-2) {	 /* went off bottom */
-	    sd->top_entry++;
-	    if (sd->top_entry + screen_len-2 > *no_entries) {
-		sd->top_entry--;
-		sd->cur_entry--;
-	    }
-	    sd->cursor_line = screen_len-2;
-	    ps(win, sd->top_entry, &p_scroll);
-	}
-	if (sd->cur_entry > *no_entries-1) {
-	    sd->cur_entry = *no_entries-1;
-	    sd->cursor_line = sd->cur_entry - sd->top_entry + 1;
-	}
+	adjust_sd(win, sd, ps, &p_scroll, no_entries);
     }
     return 0; /* shouldn't get here */
 }
@@ -818,7 +941,8 @@ int read_dir(char *dir_name, char *prefix, modify_filename fn,
 	      if ( (strcmp(x->d_name, "..")) &&	         /* don't let user modify up dir */
 		  !S_ISDIR(cur_ent->org_info.st_mode) )  /* or any directories */
 		if (fn)			                 /* let caller */
-		  fn(cur_ent, dir_name, prefix);         /* modify filename if it wants to */
+		  if (fn(cur_ent, dir_name, prefix) == FALSE) /* modify filename */
+		    continue;			 /* FALSE means don't want this */
 	      if (!(!strcmp(x->d_name, "..") && ((!*dir_name) || (!strcmp(dir_name, "/"))))) {
 		  cur_ent++;	       /* don't include .. directory if in root */
 		  in_count++;
@@ -854,7 +978,7 @@ void print_dir_line(WINDOW *win, struct direntry *entry, int line,
  * Returns the size of the directory entry in bytes */
     
 
-    char       s[NAME_MAX+1], s2[MAX_FNAME];
+    char       s[NAME_MAX+1], s2[MAX_FNAME], s3[MAX_FNAME];
 
     get_file_type(s, entry->info.st_mode);
     strcat(s, " "); 
@@ -870,8 +994,8 @@ void print_dir_line(WINDOW *win, struct direntry *entry, int line,
 	convert(s2, entry->info.st_size);
     else
       *s2 = 0;
-    sprintf(s, "%s%-*s", s, dir_left_width-18, entry->entry.d_name);
-    s[dir_left_width-18] = 0; sprintf(s, "%s %12s", s, s2);
+    pr_filename(entry->entry.d_name, s3, dir_left_width-21);
+    sprintf(s, "%s %s %12s", s, s3, s2);
     mvwaddstr(win, line+1, 2, s);
     wattroff(win, A_BOLD);
     if (ref)
@@ -1026,11 +1150,13 @@ int select_file(char *start_dir, char *prefix, char *deflt, modify_filename fn,
 
     while (1) {
 	ret = select_box(dir_win, &sb_directory_count, &sb_dir_sd,
-			 print_dir, sb_print_dir_line, NULL, 
-			 NULL, dir_paint,
+			 print_dir, sb_print_dir_line, NULL, NULL,
+			 NULL, dir_paint, NULL, 
 			 NULL, NULL, NULL,
 			 NULL, NULL, NULL,
-			 "", NULL, NULL);
+			 NULL, NULL,NULL, NULL,
+			 "", NULL, NULL, FALSE, FALSE,
+			 NULL, NULL, NULL);
 	switch(ret) {
 	 case SELECT_ABORT:
 	    delwin(dir_win);
@@ -1097,5 +1223,10 @@ int retryabort(char *s)
     strcpy(s1, strerror(errno));
     strcat(s1, " while ");
     strcat(s1, s);
+    if (win_main == NULL) return -1;		 /* must abort */
     return (message_box(s1, MB_RETRYABORT) == 1) ? 0 : -1;
 }
+
+
+
+    
