@@ -1,23 +1,21 @@
 /*
-   Time-stamp: <96/07/19 20:18:25 yusuf>
+   Time-stamp: <97/08/10 08:57:42 yusuf>
 
-   $Id: select_box.h,v 1.15 1996/07/27 20:42:14 yusuf Exp $
+   $Id: select_box.h,v 1.24 1997/08/10 04:25:23 yusuf Exp $
 */
 
 
 #ifndef _FROM_MAIN_
-  #define inline __inline__
-
-  #include <dirent.h>
-  #include <errno.h>
-  #include <unistd.h>
-  #include <stdio.h>
-  #include <stdlib.h>
-  #include <curses.h>
-  #include <form.h>
-  #include <fcntl.h>
-  #include <sys/stat.h>
-  #include "ttypes.h"
+#include "config.h"
+#include <dirent.h>
+#include <errno.h>
+#include <unistd.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <curses.h>
+#include <form.h>
+#include <fcntl.h>
+#include <sys/stat.h>
 #endif
 
 #include "memory.h"
@@ -31,11 +29,14 @@
 #define MB_OKCANCEL 3
 #define MB_APPENDOVERWRITE 4
 #define MB_RETRYABORT 5
+#define MB_NOYES 6
 
 #define SELECT_ABORT 0
 #define SELECT_FINISHED 1
 #define SELECT_TAB 2
 #define SELECT_ENTER 3
+#define SELECT_SEARCH 4
+#define SELECT_GOTO 5
 
 #define COLOR_MAIN 1
 #define COLOR_DIALOG 2				 /* color of dialog box */
@@ -51,6 +52,7 @@ typedef struct {
     _s32 top_entry, cur_entry;
     int cursor_line;
 } select_details;
+extern char *make_permstring(char *s, mode_t perms);
 extern void clear_sd(select_details *sd);
 
 
@@ -65,25 +67,32 @@ extern void centre(WINDOW *win, int line, char *s, int color);
 extern WINDOW *status_box(WINDOW *win, char *s, int line, int create, int no_lines);
 extern void close_statusbox(WINDOW *win);
 extern int message_box(char *s, int type);
-extern int multi_message_box(char lns[][150], int count, int type);
+extern int multi_message_box(char lns[][150], int count, int type, int centre);
 
 typedef void (*print_screen) (WINDOW *win, _s32 top, char *p_scroll);
 typedef void (*print_line) (WINDOW *win, _s32 cursor_entry, int cursor_line, char refresh);
 typedef int (*tag_entry) (_s32 cursor_entry);
 typedef void (*delete_entry) (select_details *sd, _s32 *no_sel);
+typedef void (*entry_details) (select_details *sd);
 typedef void (*refresh_screen) (void);
 typedef void (*help_screen) (void);
 typedef void (*sr_backupset) (int in_backup);
 typedef void (*sr_restoreset) (void);
 
+extern void adjust_cur_entry(WINDOW *win, select_details *sd, print_screen ps, char *p_scroll,
+		      _u32 *no_entries, _s32 cur_entry);
 extern int select_box(WINDOW *win, _s32 *no_entries,  select_details *sd,
-		      print_screen ps, print_line pl, tag_entry te, delete_entry de,
-		      refresh_screen rs,
+		      print_screen ps, print_line pl, tag_entry te, tag_entry tE,
+		      delete_entry de, refresh_screen rs, entry_details ed,
 		      WINDOW *sel_win, _s32 *sel_no_entries,
 		      select_details *sel_sd, print_screen sel_ps,
+		      WINDOW *excl_win, _s32 *excl_no_entries,
+		      select_details *excl_sd, print_screen excl_ps,
 		      print_screen asterix, select_details *fdetails,
 		      char *select_help, sr_backupset save_backupset,
-		      sr_restoreset rest_backupset);
+		      sr_restoreset rest_backupset,
+		      int allow_search, int allow_goto,
+		      tag_entry ee, tag_entry eE, char *unselect_help);
 extern void print_scroll_bar(WINDOW *win, _s32 no_entries, select_details *sd,
 		      int s_len, int s_width, char *p_scroll);
 extern int select_menu(WINDOW *win, char *menu_items[], int *in_op);
@@ -105,7 +114,7 @@ extern select_details dir_sd;
 extern int dir_left_width;
 extern int dir_screen_ylen;
 
-typedef void (*modify_filename) (struct direntry *x, char *dir_name, char *prefix);
+typedef char (*modify_filename) (struct direntry *x, char *dir_name, char *prefix);
 	      
 extern void my_strcpy(char *dest, char *src, int maxlen);
 extern void find_correct_sd(char *old_dir, select_details *sd, _s32 dir_count,
