@@ -1,7 +1,7 @@
 /*
-   Time-stamp: <96/07/19 20:16:51 yusuf>
+   Time-stamp: <97/08/10 09:55:47 yusuf>
 
-   $Id: structs.h,v 1.11 1996/07/27 20:42:15 yusuf Exp $
+   $Id: structs.h,v 1.24 1997/08/10 04:25:23 yusuf Exp $
 */
 
 
@@ -17,9 +17,11 @@ struct tape_header {
 /* Header of info file */
 struct info_file_header {
     _u32  magic;
+    _u32  free[INFO_INDICIES];			 /* for use with m-tree */
+    _u32  end[INFO_INDICIES];			 /* indicies */
+    _u32  root[INFO_INDICIES];
     _u32  archive_id;
     _s32  number_tapes;
-    _u32  info_file_size;			 /* note that this doesn't include the */
     _s32  number_volumes;			 /* volume/tape info at end of info file */
     _u32  size_volume_headers;			 /* size of volume headers */
     _s32  no_in_archive;
@@ -46,7 +48,7 @@ struct volume_header {
     _s32 no_in_volume;				 /* # files in volume */
     char volume_title[MAX_ARCHIVETITLE];	 /* name of volume */
     _time_t backup_time;			 /* time volume backed up */
-    _s32  no_sels;				 /* number of selections */
+    _s32  no_sels;				 /* number of select */
     _u32 size_header;				 /* size of volume header information */
 };
 
@@ -60,7 +62,7 @@ struct file_info {
     /* If the volume < 0, then this means that this file is a directory that
        was added because of a change in common path names. The volume is the
        negative of the original */
-    _s32     pos_in_archive;
+    _s32      pos_in_archive;
     _dev_t    dev;
     _uid_t    uid;				 
     _gid_t    gid;
@@ -79,6 +81,21 @@ struct file_info {
 };
 
 
+struct info_file_data {				 /* structure holding information */
+    _u32 recnum;				 /* record number */
+    struct file_info f;				 /* in data file */
+    char   name[MAX_FNAME];
+};
+
+struct info_file_key {				 /* common key for info file */
+    _u32 recnum;
+    char name[MAX_FNAME];
+    _u32  pos_in_archive;
+    _s32     volume;
+    _time_t  mtime;
+};
+    
+    
 /* Information about which tape each volume is on 
    Appened to end of info file */
 struct volume_tape_info {
@@ -98,40 +115,35 @@ struct tape_size_info {
 };
 
 
+struct memory_file {				 /* representation of a file in memory */
+    _u32 size;
+    _s8 selected;				 /* 0 not selected, 1 selected  */
+    _s8 unselected;				 /* 0 not unselected, 1 unselected */
+    _u32 most_recent;				 /* rec # of most recent */
+};
+
+    
 /* Representation in memory of a file on the archive */
-struct oa_file_entry {
-    struct file_info    i;
-    _s8                 selected;		 /* 1 if selected directly, 2 if selected indirectly,
-						    0 if not */
-    _s8                 incremental;		 /* in sel_restore: 1 = restore most recent
+#define ENTRY_INCLUDE 1
+#define ENTRY_EXCLUDE 2
+struct selected_entry {
+    struct info_file_data i, savedi;
+    _s8                   incremental;		 /* in sel_restore: 1 = restore most recent
 						                    0 = restore volume given
 						    in sel_backup:  1 = incremental backup
 						                    0 = full backup */
-    struct oa_file_entry   *in_archive_files;	 /* when used by sel_restore. For sel_backup, 1 = directly select */
-    _u32                dirsize;		 /* size if directory */
-    _u32                mm_size, ab_size;	 /* sizes used by sel_restore */
-    _s8                 end_entry;		 /* = END_ENTRY - signifies end of file entry */
-
-/* Notes on selected field:
-
-   0 = not selected
-   1 = selected directly
-   2 = selected indirectly (because directory it is in was selected)
-   3 = selected directly but then duplicated because directory it was
-       in has subsequently been selected
-*/		       
+    _s8                 selected;		 /* 0 if directly, 1 if indirectly */
+    _u32                size;			 /* size of selection */
+    _u32                mm_size, ab_size;
 };						 
 
-struct fixed_oa {
-    struct oa_file_entry f;
-    char fn[MAX_FNAME];
-};
 
 /* Representation in memory of the directory I create in sel_restore */
 struct dir_file_entry {
-    struct oa_file_entry o;
-    char                name[MAX_FNAME];	 
-    char                filter[MAX_FILTER];
+    struct info_file_data i;
+    char                name[MAX_FNAME];
+    _s8                 selected;
+    _u32                size;
 };						 
 
 struct shared_mems {				 /* things that are */
@@ -182,7 +194,7 @@ struct pref_info {
 };
 
 
-typedef _errstat (*file_passed_action) (struct file_info *fi, char * fn, struct oa_file_entry *fe);
+typedef _errstat (*file_passed_action) (struct file_info *fi, char * fn, struct info_file_data *i_data);
 typedef void (*print_status) (WINDOW *mes_box, _s32 cur_in_vol, 
 			      _s32 no_in_vol, _s32 vol,
 			      _u32 file_size,
@@ -190,7 +202,14 @@ typedef void (*print_status) (WINDOW *mes_box, _s32 cur_in_vol,
 typedef _errstat (*chksum_err) (struct file_info *fi, char *fn);
 
 
-#define PAST_STRING_FI(cp) \
-  cp += (((struct file_info *) (cp-sizeof(struct file_info)))->name_len)
-#define PAST_STRING_OAFE(cp) \
-  cp += (((struct oa_file_entry *) (cp-sizeof(struct oa_file_entry)))->i.name_len)
+
+/* For mtree.c */
+      
+typedef _u32 dskblk; 
+typedef struct info_file_key keytype;		 /* key */
+typedef struct {				 /* structure of a node */
+	int n;					
+	keytype keys[TREEORD+1];
+	dskblk a[TREEORD+1];	
+} info_file_datai;
+typedef info_file_datai node;
