@@ -1,12 +1,12 @@
 /*
-   Time-stamp: <96/08/05 19:49:31 yusuf>
+   Time-stamp: <97/07/12 20:05:28 yusuf>
 
-   $Id: tapeio.c,v 1.17 1996/08/05 19:02:02 yusuf Exp $	
+   $Id: tapeio.c,v 1.33 1997/08/09 00:52:36 yusuf Exp $	
 
 */
 
 #ifndef lint
-static char vcid[] = "$Id: tapeio.c,v 1.17 1996/08/05 19:02:02 yusuf Exp $";
+static char vcid[] = "$Id: tapeio.c,v 1.33 1997/08/09 00:52:36 yusuf Exp $";
 #endif /* lint */
 
 
@@ -18,30 +18,36 @@ FILE *fd_pipetoc=NULL;
 char pipefc[MAX_FNAME];
 FILE *fd_pipefc=NULL;
 
+
+ssize_t overflow_count;				 /* used for reading error handling*/
+char *overflow_buffer;
+
 void paint_main(void);		       /* in taper.c */
-size_t raw_read(int fd, char *data, size_t count)
+ssize_t raw_read(int fd, char *data, ssize_t count, int allowerr)
 {
 /* Does a read from the tape */
     
-    size_t x, x1, rc;
+    ssize_t x, x1, rc;
     int  serr;
     char *m;
     
     rc = 0;
     while (1) {
-	x = read(fd, data+rc, count-rc);
+	x = rmtread(fd, data+rc, count-rc);
 	serr = errno;
 	if (x != -1) x += rc;			 /* adjust for already got */
 	if (x == count) return x;		 /* got all data */
 	if (x == -1) {				 /* error reading */
 	    if (errno == ENOSPC) return 0;	 /* end of tape */
 	    if (errno == ENODATA) return 0;	 /* no data = end of tape */
+	    if (allowerr) return -1;		 /* error allowed */
 	    if (retryabort("reading from tape") == -1) {
 		errno = serr;
 		return -1;
 	    }
 	    continue;
 	}
+	if (x == 0) return 0;			 /* 0 return == end of tape */
 /*  We have a short read: two possibilites
      a. end of tape
      b. i/o error
@@ -55,26 +61,38 @@ size_t raw_read(int fd, char *data, size_t count)
         and get 10,000 bytes.
 
        the first 1,000 bytes of this 2nd read is put in the read buffer
-       the other 9,000 bytes are discarded.
+       the other 9,000 bytes are discarded is placed in an overflow buffer
+         and overflow_count set to 9000. When read from buffer, it checks
+	 this overflow buffer first before coming back for more data
+*/
+				
+						
+	if (illegal_eot)			 /* end of tape properly so can't */
+	    if (message_box("Problem reading from tape. End of tape?", MB_YESNO)) /* distinguish b/w two conditions */
+		return x;			 /* tape drive doesn't handle */
 
-      This is a very unusual situation and, basically, shouldn't occur
-*/						 
 	rc = x;					 /* rc = how much got */
 	m = alloca(block_size);
 	while (rc < count) {
-	  x1 = read(fd, m, block_size);
-	  if (x1 == 0) return rc;			 /* must assume end of tape */
-	  if (x1 == -1) {			 /* good - error condition */
-	    if ((errno == ENOSPC) || (errno == ENODATA)) /* correct - end of tape */
-	      return rc;
-	    if (retryabort("reading from tape") == -1)  { /* a real error */
-	      errno = serr;
-	      return -1;
+	    x1 = rmtread(fd, m, block_size);
+	    if (x1 == 0) return rc;			 /* must assume end of tape */
+	    if (x1 == -1) {			 /* good - error condition */
+		if ((errno == ENOSPC) || (errno == ENODATA)) /* correct - end of tape */
+		    return rc;
+		if (retryabort("reading from tape") == -1)  { /* a real error */
+		    errno = serr;
+		    return -1;
+		}
+		continue;
 	    }
-	  }
-	  /* we actually got some data -?why * - anyway, put in buffer */
-	  memcpy(data+rc, m, min(x1, count-rc));
-	  rc += x1; 
+	    /* we actually got some data -?why * - anyway, put in buffer */
+	    memcpy(data+rc, m, min(x1, count-rc));
+	    rc += x1; 
+	}
+	if (rc > count) {			 /* put excess data in overflow buffer  */
+	    overflow_buffer = my_malloc(rc-block_size);
+	    memcpy(overflow_buffer, m+block_size-(rc-block_size), rc-block_size);
+	    overflow_count = rc-block_size;
 	}
 	if (rc >= count) return count;		 /* got all data */
     }
@@ -82,7 +100,7 @@ size_t raw_read(int fd, char *data, size_t count)
 }
 
 
-size_t raw_write(int fd, char *data, size_t count)
+ssize_t raw_write(int fd, char *data, ssize_t count)
 {
 /* Does a write to the tape 
  
@@ -93,11 +111,13 @@ size_t raw_write(int fd, char *data, size_t count)
  * If couldn't write any bytes due to I/O error, returns -(count+1)
  */
     
-    size_t x, x1, rc;
+    ssize_t x, x1, rc;
+    char s[200];
+    int serr;
     
     rc = 0;
     while (1) {
-	x = write(fd, data+rc, count-rc);
+	x = rmtwrite(fd, data+rc, count-rc);
 	if (x != -1) x += rc;			 /* adjust for already got */
 	if (x == count) return x;		 /* got all data */
 	if (x == -1) {				 /* error writing */
@@ -115,15 +135,20 @@ size_t raw_write(int fd, char *data, size_t count)
 */						 
 	rc = x;
 	while (rc < count) {
-	  x1 = write(fd, data+rc, block_size);	 /* write a block */
-	  if (x1 == 0) return rc;		 /* must assume end of tape */
-	  if (x1 == -1) {			 /* good - error condition */
-	    if ((errno == ENOSPC) || (errno == ENODATA)) /* correct - end of tape */
-	      return rc;			 /* correct - end of tape */
-	    if (rc == 0) return -(count+1);	 /*  none written  */
-	    return -rc;				 /* real error - wrote rc bytes */
-	  }
-	  rc += x1; 
+	    x1 = rmtwrite(fd, data+rc, block_size);	 /* write a block */
+	    serr = errno;
+	    if (log_level > 3) {
+		sprintf(s, "W:Writing in error loop - rc=%d, count=%d, x1=%d", rc, count, x1);
+		write_log(s);
+	    }
+	    if (x1 == 0) return rc;		 /* must assume end of tape */
+	    if (x1 == -1) {			 /* good - error condition */
+		if ((serr == ENOSPC) || (serr == ENODATA)) /* correct - end of tape */
+		    return rc;			 /* correct - end of tape */
+		if (rc == 0) return -(count+1);	 /*  none written  */
+		return -rc;				 /* real error - wrote rc bytes */
+	    }
+	    rc += x1; 
 	}
 	if (rc >= count) return count;		 /* got all data */
     }
@@ -135,37 +160,13 @@ _errstat is_regfile(int d)
 {
 /* Returns 1 if dv is the file handle of a regular file */
     struct stat sb;
-    
-    fstat(d, &sb);
+
+    if (d == -1) return FALSE;			 /* not open ?why */
+    rmtfstat(d, &sb);
     return S_ISREG(sb.st_mode);
 }
 
 
-_errstat tape_eom(int allowerr)
-{
-/* If allowerr = 1, errors allowed */    
-    struct mtop m;
-    _errstat x;
-    
-    m.mt_op = MTEOM;
-    m.mt_count = 1;
-
-/* When doing eom, must flush buffers */    
-    if (write_offset != -1) {
-	flush_buffers();
-	wait_finish_writing();
-    }
-    read_offset = -1;				 /* discard rest of block */
-    if (log_level > 2) write_log("Going to eom");
-    blocks_passed = 0;
-    x = (is_regfile(dv)) ? lseek(dv, 0L, SEEK_END) : 
-                              ioctl(dv, MTIOCTOP, (char *) &m);
-    if ((x == -1) && (!allowerr)) 		 /* only do error */
-      do_exit(ERROR_TAPE_EOM);			 /* if errors not allowed */
-    return x;
-}
-
-
 _errstat tape_fsf(int count, int allowerr)
 {
 /* If allowerr = 1, errors allowed */    
@@ -192,18 +193,44 @@ _errstat tape_fsf(int count, int allowerr)
 	return do_exit(ERROR_NO_FSF);		 /* can't fsf */
     }
     blocks_passed = 0;
-    x = (is_regfile(dv)) ? lseek(dv, 0L, SEEK_END) : 
-                              ioctl(dv, MTIOCTOP, (char *) &m);
+    x = (is_regfile(dv)) ? rmtlseek(dv, 0L, SEEK_END) : 
+                              rmtioctl(dv, MTIOCTOP, (char *) &m);
     if ((x == -1) && (!allowerr)) 		 /* only do error */
       do_exit(ERROR_TAPE_FSF);			 /* if errors not allowed */
     return x;
 }
 
 
-_errstat tape_seek(int toblock)
+_errstat tape_eom()
 {
     struct mtop m;
-    struct mtget mg;
+    char   l[200];
+    _errstat x;
+    
+    m.mt_op = MTEOM;
+    m.mt_count = 1;
+    
+/* When doing eom, must flush buffers */    
+    if (write_offset != -1) {
+	flush_buffers();
+	wait_finish_writing();
+    }
+    read_offset = -1;				 /* discard rest of block */
+    sprintf(l, "Advancing to end of tape");
+    if (log_level > 2) write_log(l);
+    blocks_passed = 0;
+    x = (is_regfile(dv)) ? rmtlseek(dv, 0L, SEEK_END) : 
+                              rmtioctl(dv, MTIOCTOP, (char *) &m);
+    if (x == -1)				 
+      do_exit(ERROR_TAPE_EOM);			 
+    return x;
+}
+
+
+_errstat tape_seek(int toblock, char must_seek)
+{
+    /* If must_seek == TRUE, then must seek and not stream */    
+    struct mtop m;
     _s32 tb;
     char s[200];
     
@@ -211,25 +238,26 @@ _errstat tape_seek(int toblock)
     if (is_regfile(dv)) {
 	if (log_level > 2) write_log("File seek to correct block");
 	if (toblock < blocks_passed) return 0;
-	lseek(dv, block_size*(toblock-blocks_passed), SEEK_CUR);
+	rmtlseek(dv, block_size*(toblock-blocks_passed), SEEK_CUR);
 	read_offset = -1;
 	blocks_passed = toblock;
 	return 0;
     }
-    if (toblock-blocks_passed < min_seek) {
-      sprintf(s, "Only need to pass %d blocks - leaving to stream",
-	      toblock-blocks_passed);
-      if (log_level > 2) write_log(s);
-      return -1; /* continue streaming */
-    }
+    if (!must_seek)
+	if (toblock-blocks_passed < min_seek) {
+	    sprintf(s, "Only need to pass %d blocks - leaving to stream",
+		    toblock-blocks_passed);
+	    if (log_level > 2) write_log(s);
+	    return -1; /* continue streaming */
+	}
     
     read_offset = -1;
-    if (log_level > 2) write_log("ioctl get info");
-    memset(&mg, 0, sizeof(mg));
-    if (ioctl(dv, MTIOCGET, (char *) &mg) == -1)
-      tb = toblock;				 /* if tape drive doesn't */
-    else					 /* support this - assume that */
-      tb = mg.mt_blkno;				 /* first block is 0 */
+    if (tape_tell(&tb) == -1)
+	tb = toblock;
+    if (log_level > 2) {
+	sprintf(s, "tell returned %d", tb);
+	write_log(s);
+    }
     if (log_level > 2) write_log("ioctl seek");
     m.mt_op = MTSEEK;
     m.mt_count = toblock+(tb-blocks_passed);
@@ -237,7 +265,7 @@ _errstat tape_seek(int toblock)
       sprintf(s, "seek is adjusting %d", tb-blocks_passed);
       if (log_level > 2) write_log(s);
     }
-    if (ioctl(dv, MTIOCTOP, (char *) &m) != -1) {
+    if (rmtioctl(dv, MTIOCTOP, (char *) &m) != -1) {
 	blocks_passed = toblock;
 	return 0;
     }
@@ -254,19 +282,19 @@ _errstat tape_goto_block(WINDOW *mes, int line, _s32 block)
  * 
  * Returns 0 if OK, -1 if error
 */
-    _s32 c;
     char l[400];
+    _s32 c;
     
     if (block < blocks_passed) return 0;
     c = blocks_passed;
     sprintf(l, "Seeking block %d", block);
     if (log_level > 2) write_log(l);
     if (mes) status_box(mes, l, line,  FALSE, 1);
-    if (tape_seek(block) != -1) return 0;
+    if (tape_seek(block, FALSE) != -1) return 0;
     if (log_level > 2) write_log("Couln't ioctl seek - trying manually");
     read_offset = -1;
     while (c < block) {
-	if (raw_read(dv, read_buffer, block_size) != block_size)
+	if (raw_read(dv, read_buffer, block_size, FALSE) != block_size)
 	  return do_exit(ERROR_TAPE_GOTO);
 	blocks_passed++;
 	c++;
@@ -307,7 +335,7 @@ _errstat tape_read_s32(_s32 *x)
 }
 
 
-_errstat tape_readheader(struct tape_header *t, _s8 allowz)
+_errstat tape_readheader(struct tape_header *t, _s8 allowz, int mode)
 {
 /* Reads the tape header. It assumes that the tape is at the
  * beginning and no data has been read in as yet.
@@ -334,7 +362,7 @@ _errstat tape_readheader(struct tape_header *t, _s8 allowz)
     while (1) {
 	memset(t, 0, sizeof(*t));
 	if (log_level > 2) write_log("Reading in block for tape header");
-	read_buffer_count = raw_read(dv, read_buffer, block_size);
+	read_buffer_count = raw_read(dv, read_buffer, block_size, allowz);
 	blocks_passed=1;
 	if (read_buffer_count != block_size) {	 /* read in a block */
 	    if (tape_type == TAPE_TYPE_ZFTAPE) { /* zftape has a problem sometimes, so read again to double check */
@@ -342,7 +370,7 @@ _errstat tape_readheader(struct tape_header *t, _s8 allowz)
 		if ((read_buffer_count == -1) && (errno == EINVAL)) {   /* correct block size initially */
 		    if (tape_get_blk_size(dv) == -1) return (allowz) ? 0 : -1;
 		    if (log_level > 2) write_log("Rereading tape header block");
-		    read_buffer_count = raw_read(dv, read_buffer, block_size);
+		    read_buffer_count = raw_read(dv, read_buffer, block_size, allowz);
 		    if (read_buffer_count != block_size) {
 			if (allowz) return 0;	 /* errors allowed */
 			write_fatal_log("reading in tape header block");
@@ -392,22 +420,21 @@ _errstat tape_readheader(struct tape_header *t, _s8 allowz)
 	    if (!is_regfile(dv) && !done_it) {	 /* only if using tape drive */
 		mt.mt_op = MTREW;		 /* try reading more than one segment  */
 		mt.mt_count = 1;
-		ioctl(dv, MTIOCTOP, (char *) &mt);   /* rewind */
+		rmtioctl(dv, MTIOCTOP, (char *) &mt);   /* rewind */
 		m = malloc((28672/block_size+1)*block_size);
 		if (m == NULL)
 		  do_exit(ERROR_MEMORY);
 		sprintf(s, "Special zftape. Reading %d, Read %ld",((28672/block_size+1)*block_size),
-			(long) raw_read(dv, m, (28672/block_size+1)*block_size));   /* read more than 28K */
+			(long) raw_read(dv, m, (28672/block_size+1)*block_size, allowz));   /* read more than 28K */
 		if (log_level > 2) write_log(s);
 		free(m);
 		mt.mt_op = MTREW;
 		mt.mt_count = 1;
 		sprintf(s, "Special zftape. Rewinding %d",/* rewind */
-			ioctl (dv, MTIOCTOP, (char *) &mt));	
+			rmtioctl (dv, MTIOCTOP, (char *) &mt));	
 		if (log_level > 2) write_log(s);
-		fsync(dv);
-		close(dv);			 /* by close */
-		if (tape_open(O_RDWR)  == -1) /* and reopen */
+		rmtclose(dv);			 /* by close */
+		if (tape_open(mode)  == -1) /* and reopen */
 		  return -1;
 		done_it = 1;			 /* only do it once */
 	    }
@@ -467,11 +494,11 @@ _errstat check_tape(WINDOW *mes, int line, _s32 tape_required)
 		tape_required, ifd.archive_id);
 	if (log_level > 2) write_log(l);
 	if (tape_rewind() == -1) return -1;
-	if (tape_open(O_RDWR) == -1) 
+	if (tape_open(O_RDONLY) == -1) 
 	    continue;
 	vols_passed = 0;			 /* no volumes passed on this tape */
 	tdh1.magic = 0;
-	tape_readheader(&tdh1, 0);
+	tape_readheader(&tdh1, 0, O_RDONLY);
 	if ((tdh1.magic != TAPER_MAGIC_NUMBER) && (tdh1.magic != TAPER_64_MAGIC)) {
 	    if (!message_box("This tape doesn't contain taper data", MB_OKCANCEL))
 		return -1;
@@ -506,7 +533,20 @@ _errstat check_tape(WINDOW *mes, int line, _s32 tape_required)
 
 
 
-_errstat tape_rewind()
+void clear_buffer_pointers(void)
+{
+    left_tr = 0;
+    read_offset = -1; 
+    read_buffer_count = 0;
+    overflow_count = 0;
+    write_offset = 0;
+    w_cur_pos = 0;
+    w_current_buffer = w_buffer_1;
+    write_buf_no = NULL;
+}
+
+
+_errstat tape_rewind_engine(int opendevice)
 {
 /* Rewinds tape. Tape must be closed prior to calling of rewind.
  * Returns 0 if rewound, -1 if error
@@ -515,30 +555,43 @@ _errstat tape_rewind()
  * 
  * It doesn't use tape_open/tape_close to avoid buffering problems
 */
-    _errstat err=0, myerrno, x;
+    _errstat err=0, myerrno;
     struct mtop m;
+    int x;
     
     m.mt_count = 1;
     m.mt_op = MTREW;
     
     if (!have_rewind) return 0;
     if (log_level > 2) write_log("Rewinding tape");
-    if ((x=open(tape, O_RDWR)) == -1)		 /* try and open tape */
-      return do_exit(ERROR_OPENING_BACKUP);
+    if (opendevice) {
+	if ((x=rmtopen(tape, O_RDWR, S_IRWXU)) == -1) /* try and open tape */
+	    return do_exit(ERROR_OPENING_BACKUP);
+    }
+    else
+	x = dv;
     if (!is_regfile(x))
-      err = ioctl (x, MTIOCTOP, (char *) &m);	 /* do status if not regular file */
+	err = rmtioctl (x, MTIOCTOP, (char *) &m); /* do status if not regular file */
+    else
+	lseek(x, 0, SEEK_SET);
     myerrno = errno;				 /* preserve errno accross close */
-    close(x);					 /* close */
+    if (opendevice) rmtclose(x);		 /* close */
     errno = myerrno;
+    blocks_passed = 0;
+    clear_buffer_pointers();
     return (err == -1) ? do_exit(ERROR_REWIND) : 0;
 }
 
+_errstat tape_rewind()
+{
+    return tape_rewind_engine(TRUE);
+}
+
 
 void tape_set_blk_size()
 {
 /* Sets the block size, if required */    
     struct mtop m;
-    _errstat err;
     char l[100];
 
     block_size = org_block_size;
@@ -548,12 +601,27 @@ void tape_set_blk_size()
     if (set_blksize || get_blksize) {
 	sprintf(l, "Setting block size to %d", block_size);
 	if (log_level > 2) write_log(l);
-	err = (is_regfile(dv)) ? 0 : ioctl(dv, MTIOCTOP, (char *) &m);
-	/*if (err == -1) do_exit(ERROR_SETTING_BLKSIZE); */
+	(is_regfile(dv)) ? 0 : rmtioctl(dv, MTIOCTOP, (char *) &m);
 	if (init_buffers(1) == -1) do_exit(ERROR_SETTING_BLKSIZE);/* reallocate buffer sizes */
     }
 }
 
+_errstat tape_tell(_u32 *pos)
+{
+    _errstat err=0;
+    struct mtpos m;
+
+    if (log_level > 2) write_log("Doing a tell");
+    if (is_regfile(dv)) 
+        *pos = rmtlseek(dv, 0, SEEK_CUR) / block_size;
+    else {
+	err = rmtioctl(dv, MTIOCPOS, &m);
+	*pos = m.mt_blkno;
+    }
+    return err;
+}
+
+
 _errstat tape_get_blk_size(_s32 x)
 {
     struct mtblksz m;
@@ -565,14 +633,14 @@ _errstat tape_get_blk_size(_s32 x)
 	if (log_level > 2) write_log("Getting block size");
 	m.mt_blksz = block_size;
 	obs = block_size;
-	err = (is_regfile(x)) ? 0 : ioctl(x, MTIOC_ZFTAPE_GETBLKSZ, (char *) &m);
+	err = (is_regfile(x)) ? 0 : rmtioctl(x, MTIOC_ZFTAPE_GETBLKSZ, (char *) &m);
 	if (err == -1) 
-	  do_exit(ERROR_GETTING_BLKSIZE);
+	 do_exit(ERROR_GETTING_BLKSIZE);
 	else
 	  block_size = m.mt_blksz;
 	sprintf(s, "Got block size %d", block_size);
 	if (log_level > 2) write_log(s);
-	if (block_size == 1) {			 /* if got block size of 1, means that */
+	if (block_size < 1025) {		 /* if got block size <1025, means that */
 	    block_size = obs;			 /* block size hasn't been set for this */
 	    tape_set_blk_size();		 /* volume - so set it */
 	}
@@ -598,23 +666,11 @@ _errstat tape_erase()
     m.mt_count = 1;
     
     if (log_level > 2) write_log("Erasing tape");
-    err = (is_regfile(dv)) ? 0 : ioctl(dv, MTIOCTOP, (char *) &m);
+    err = (is_regfile(dv)) ? 0 : rmtioctl(dv, MTIOCTOP, (char *) &m);
     if (err == -1) return do_exit(ERROR_ERASING_TAPE);
     return 0;
 }
 
-void clear_buffer_pointers(void)
-{
-    left_tr = 0;
-    read_offset = -1; 
-    read_buffer_count = 0;
-    write_offset = 0;
-    w_cur_pos = 0;
-    w_current_buffer = w_buffer_1;
-    write_buf_no = NULL;
-}
-
-
 _errstat mount_device(void)
 {
 /* If a mounted type, tries to mount the device.
@@ -645,7 +701,7 @@ _errstat ntape_open(int mode)
     sprintf(l, "Opening non-rewinding %s", ntape);
     if (log_level > 2) write_log(l);
     clear_buffer_pointers();
-    dv = open(ntape, mode);
+    dv = rmtopen(ntape, mode, S_IRWXU);
     blocks_passed = 0;
     if (dv == -1) 
 	return do_exit(ERROR_OPENING_BACKUP);
@@ -658,16 +714,26 @@ _errstat tape_open_engine(int mode, int rezero)
 {
 /* Opens rewinding device */
     char l[200];
+    int  serr;
 
     if (mount_device() == -1) return -1;
     sprintf(l, "Opening rewinding %s", tape);
     if (log_level > 2) write_log(l);
     if (rezero) clear_buffer_pointers();	 /* only if user wants us to */
-    dv = open(tape, mode);
+    while (1) {
+	dv = rmtopen(tape, mode, S_IRWXU);
+	if (dv != -1) break;
+	serr = errno;
+	if (retryabort("opening tape") == -1) {
+	    errno = serr;
+	    return -1;
+	}
+    }
     blocks_passed = 0;
     if (dv == -1) 
 	return do_exit(ERROR_OPENING_BACKUP);
     if (tape_get_blk_size(dv) == -1) return -1;
+    tape_set_blk_size();
     return dv;
 }
 
@@ -684,7 +750,7 @@ _errstat tape_close_engine(_s8 flush)
     _errstat ret;
 
     /* closes tape device */
-    if (dv==-1) return 0;			 /* not open */
+    if ((dv==-1) || (dv == 0)) return 0;			 /* not open */
     if (flush) {
 	flush_buffers();
 	wait_finish_writing();
@@ -701,9 +767,8 @@ _errstat tape_close_engine(_s8 flush)
 	if (log_level > 3) write_log("Close - waitpid child done");
 	write_pid = 0;
     }
-    fsync(dv);
     if (log_level > 2) write_log("Closing tape");
-    ret = (dv) ? close(dv) : 0;
+    ret = (dv) ? rmtclose(dv) : 0;
     dv = -1;
     if (ummount_device() == -1) return -1;
     return ret;
@@ -716,20 +781,27 @@ _errstat tape_close()
 }
 
 
-int read_from_buf(char *buf, size_t count)
+int read_from_buf(char *buf, ssize_t count, int allowerr)
 {
-    size_t tr_bytes, fit_in;
+    /* Before asking for more data to be read in, it checks to
+       see if there is any data in the overflow buffer */
+    ssize_t tr_bytes, fit_in;
+    char s[200];
     
+    if (log_level > 2) {
+	sprintf(s, "Read request for %u bytes", count);
+	write_log(s);
+    }
     bytes_short = block_size;
     if (read_offset == -1) {			 /* nothing in buffer */
 	if (log_level > 2) write_log("Reading in block from tape");
 	if (tape_size) {			 /* ?end of tape */
-	    if (blocks_passed * block_size > tape_size * 1048576) {
+	    if (blocks_passed * block_size > (long unsigned) tape_size * (long unsigned) 1000000) {
 		if (log_level > 2) write_log("Forced end of tape");
 		return 0;
 	    }
 	}
-	read_buffer_count = raw_read(dv, read_buffer, block_size);   /* read in a block */
+	read_buffer_count = raw_read(dv, read_buffer, block_size, allowerr);   /* read in a block */
 	if (read_buffer_count == -1) {
 	    if (log_level) write_log("reading data from tape");
 	    return -1;
@@ -747,19 +819,28 @@ int read_from_buf(char *buf, size_t count)
 	    }
 	    if (log_level > 2) write_log("Reading in block from tape");
 	    if (tape_size) {				 /* ?end of tape */
-		if (blocks_passed * block_size > tape_size * 1048576) {
+		if (blocks_passed * block_size > (long unsigned) tape_size * (long unsigned) 1000000) {
 		    if (log_level > 2) write_log("Forced end of tape");
 		    return tr_bytes;
 		}
 	    }
-	    read_buffer_count = raw_read(dv, read_buffer, block_size);
-	    if (read_buffer_count == -1) {
-		if (log_level) write_log("reading data from tape");
-		return -1;
+	    if (overflow_count) {		 /* there's data in the overflow buffer */
+		memcpy(read_buffer+block_size-overflow_count, overflow_buffer, overflow_count);
+		my_free(overflow_buffer);	 /* put data at end of read buffer */
+		read_buffer_count = block_size;
+		read_offset = block_size-overflow_count;
+		overflow_count = 0;		 
+	    }
+	    else {
+		read_buffer_count = raw_read(dv, read_buffer, block_size, FALSE);
+		if (read_buffer_count == -1) {
+		    if (log_level) write_log("reading data from tape");
+		    return -1;
+		}
+		blocks_passed++;
+		if (read_buffer_count < 1) return tr_bytes;   /* if couldn't get any more, accept what we got */
+		read_offset = 0;
 	    }
-	    blocks_passed++;
-	    if (read_buffer_count < 1) return tr_bytes;   /* if couldn't get any more, accept what we got */
-	    read_offset = 0;
 	}
 	fit_in = min(count-tr_bytes, read_buffer_count-read_offset);
 	memcpy(buf+tr_bytes, read_buffer+read_offset, fit_in);
@@ -849,7 +930,7 @@ _errstat do_write_block(_s8 *buf_to_write, _s32 old_length, _s8 fc)
  * Returns -1 if error, 0 otherwise. 
  
  */
-    size_t c, c1;
+    ssize_t c, c1;
     _s32  tr;
     char  s[100];
     _errstat x;
@@ -888,7 +969,7 @@ _errstat do_write_block(_s8 *buf_to_write, _s32 old_length, _s8 fc)
 	    write_log(s);
 	}
 	if (tape_size) 				 /* see if user has specified tape size */
-	    if (blocks_passed*block_size > tape_size * 1048576) {
+	    if (blocks_passed*block_size > (long unsigned) tape_size * (long unsigned) 1000000) {
 		if (log_level > 2) write_log("Forced end of tape");
 		c = 0;
 		bytes_short = block_size;
@@ -897,7 +978,7 @@ _errstat do_write_block(_s8 *buf_to_write, _s32 old_length, _s8 fc)
 
 	c = raw_write(dv, write_buffer, block_size);/* write data */
 	if (log_level > 2) {
-	    sprintf(s, "Wrote block to tape [%ld bytes]", (long) c);
+	    sprintf(s, "Wrote block to tape [%d bytes]", c);
 	    write_log(s);
 	}
 
@@ -910,7 +991,7 @@ _errstat do_write_block(_s8 *buf_to_write, _s32 old_length, _s8 fc)
 	if (c < 0) {				 /* error */
 	    blocks_passed--;		
 	    x = -1;				 /* signify I/O error */
-	    c = (c == (-block_size+1)) ? 0 : -c; /* convert to actual # bytes written */
+	    c = (c == -(block_size+1)) ? 0 : -c; /* convert to actual # bytes written */
 	    sprintf(s, "W: Error %d doing writing. dv = %d. Wrote %d bytes", errno, dv, -c);
 	    if (log_level > 3) write_log(s);
 /* This is because blocks_passed will be stuffed up with partial block writes 
@@ -919,23 +1000,23 @@ _errstat do_write_block(_s8 *buf_to_write, _s32 old_length, _s8 fc)
 	    if ((log_level) & !c) 
 		write_warning_log("Write child encountered an error - you must recreate info file after this backup");
 	}
-	else {
-	    x = -2;					 /* end of space */
+	else {					 /* c == 0 */
+	    x = -2;				 /* end of space */
 	    if (log_level > 3) 
 	      write_log("W: End of tape encountered");
 	}
 
 /* Couldn't write all of block - either because end of tape (x==-2)
  * or I/O error (x==-1) */
-	bytes_short = block_size - c;
-	for (c1=0; c1<c; c1++)			 /* move data that wasn't written to  */
-	  write_buffer[c1] = write_buffer[c1+c];/* beginning of buffer */
-	write_offset = block_size - c;
 	left_tr = tr;				 /* tell parent how much is left */
 	if (log_level > 3) {
 	    sprintf(s, "W: Short write - left_tr = %d", left_tr);
 	    write_log(s);
 	}
+	bytes_short = block_size - c;
+	for (c1=0; c1<bytes_short; c1++)	 /* move data that wasn't written to  */
+	  write_buffer[c1] = write_buffer[c1+c]; /* beginning of buffer */
+	write_offset = block_size - c;
 	return x;				 /* tell parent couldn't write it all*/
     }
     left_tr = 0;
@@ -988,10 +1069,10 @@ _errstat do_next_tape(void)
 	}
 	if (tape_rewind() == -1) 
 	    continue;
-	if (tape_open_engine(O_RDWR, 0) == -1) 
-	    continue;
+	if (tape_open_engine(O_RDWR, 0) == -1)	 /* must be RDWR since called */
+	    continue;				 /* by wriet child */
 	tdh1.magic = 0;
-	tape_readheader(&tdh1, 1);
+	tape_readheader(&tdh1, 1, O_RDWR);
 	tape_close_engine(0);
 	if ((tdh1.magic == TAPER_MAGIC_NUMBER) || (tdh1.magic == TAPER_64_MAGIC)) {
 	    if (tdh1.archive_id == 0) 
@@ -1008,7 +1089,7 @@ _errstat do_next_tape(void)
 
     touchwin(mes);
     mes = status_box(mes, "Erasing tape", 1, FALSE, 1);   
-    if (tape_open_engine(O_RDWR, 0) == -1) {
+    if (tape_open_engine(O_RDWR, 0) == -1) {	 /* erasing a tape - must be RDWR */
 	write_error_log("Error opening new tape");
 	c=-1; 
 	goto end;
@@ -1050,7 +1131,7 @@ _errstat do_next_tape(void)
 
 void child_term(int sig)
 {
-    if (dv) close(dv);
+    if (dv) rmtclose(dv);
     free_buffers();
 #ifdef TRIPLE_BUFFER    
     fclose(fd_pipetoc); fclose(fd_pipefc);
@@ -1326,10 +1407,10 @@ _errstat flush_buffers()
 
 
 
-int tape_write(_vptr buf, size_t count)
+ssize_t tape_write(_vptr buf, ssize_t count)
 {
-    size_t tr_bytes;
-    size_t fit_in;
+    ssize_t tr_bytes;
+    ssize_t fit_in;
     
     tr_bytes = 0;				 /* either now or previously */
     while (tr_bytes < count) {
@@ -1344,8 +1425,7 @@ int tape_write(_vptr buf, size_t count)
     return tr_bytes;
 } 
 
-
-int tape_read(_vptr buf, size_t count)
+ssize_t tape_read_engine(_vptr buf, ssize_t count, int allowerr)
 {
 /* An end of tape is assumed if we get an return code=-1 and errno=ENOSPC   --OR--
  * the count returned is less than we asked for 
@@ -1353,9 +1433,9 @@ int tape_read(_vptr buf, size_t count)
  * note that if the return count is 0, then this is assumed to be an
  * end of volume and not and end of tape, so the zero is returned.
 */
-    size_t c;
+    ssize_t c;
     
-    c =  read_from_buf(buf, count);
+    c =  read_from_buf(buf, count, allowerr);
     if (c == count) return c;			 /* OK - written what we asked for */
     if (c == -1) {				 /* error */
       if (errno != ENOSPC)			 /* make sure not an end of tape error */
@@ -1377,7 +1457,13 @@ int tape_read(_vptr buf, size_t count)
     if (c == 0) return 0;			 /* end of tape */
     if (check_tape(NULL, 0, tdh.tape_number+1) == -1)/* get next tape */
       return -1;
-    return read_from_buf((char *) buf+c, count-c)+c;/* return data */
+    return read_from_buf((char *) buf+c, count-c, allowerr)+c;/* return data */
+}
+
+
+ssize_t tape_read(_vptr buf, ssize_t count)
+{
+    return tape_read_engine(buf, count, FALSE);
 }
 
 
@@ -1391,7 +1477,7 @@ _errstat tape_read_namelen(char *s)
     _s32 x;
     
     if (tape_read_s32(&x) == -1) return -1;
-    if (tape_read(s, x) != x) return -1;
+    if (tape_read(s, min(x, MAX_FNAME)) != min(x, MAX_FNAME)) return -1;
     return 0;
 }
 
@@ -1458,7 +1544,7 @@ _errstat tape_write_fi(struct file_info *fi)
 }
 
 
-_errstat get_tape_header(WINDOW *mess, int line, struct tape_header *tdh)
+_errstat get_tape_header(WINDOW *mess, int line, struct tape_header *tdh, int mode)
 {
 /* Tries to get the tape header of the tape currently in the drive.
  * 
@@ -1476,10 +1562,10 @@ _errstat get_tape_header(WINDOW *mess, int line, struct tape_header *tdh)
      
     if (mess)
       status_box(mess, "Identifying tape", line, FALSE, 1);
-    if (tape_open(O_RDWR) == -1) 		 /* open rewinding device */
+    if (tape_open(mode) == -1) 		 /* open rewinding device */
       return -1;
 
-    if (tape_readheader(tdh, 1) == -1)   	 /* could open but can't */
+    if (tape_readheader(tdh, 1, mode) == -1)   	 /* could open but can't */
 	return BAD_MAGIC;			 /* read - therefore is bad magic */
     
     if ((tdh->magic != TAPER_MAGIC_NUMBER) && (tdh->magic != TAPER_64_MAGIC)) {
@@ -1517,7 +1603,7 @@ _errstat tape_read_volheader(struct volume_header *vh, int allower)
     struct volume_header vh1;
     
     if (log_level > 2) write_log("Reading in volume header");
-    if (tape_read((char *) &vh1, sizeof(struct volume_header)) != sizeof(struct volume_header)) { /* volume magic number */
+    if (tape_read_engine((char *) &vh1, sizeof(struct volume_header), TRUE) != sizeof(struct volume_header)) { /* volume magic number */
 	if (allower) {
 	    if (log_level > 2) write_log("Allowable error reading in volume header");
 	    return 0;
@@ -1566,7 +1652,7 @@ _errstat read_volheader(struct volume_header *vh, _s8 read_vh, _s8 into_mem)
 }
 
 
-_errstat goto_end_vol(WINDOW *mes, int ln, _s32 vol, _s32 at_vol, _s8 get_th, _s8 to_end)
+_errstat goto_end_vol(WINDOW *mes, int ln, _s32 vol, _s32 at_vol, _s8 get_th, int mode)
 {
 /* Positions tape at beginning of volume 'vol'+1. Assumes at at_vol
  * Prints status on line 'ln' of box 'mes'
@@ -1606,7 +1692,7 @@ _errstat goto_end_vol(WINDOW *mes, int ln, _s32 vol, _s32 at_vol, _s8 get_th, _s
 
     if (get_th) 				 /* only get if user wants */
       while (1) {				 /* use to get tape header */
-	  c = get_tape_header(mes, ln, &tdh);
+	  c = get_tape_header(mes, ln, &tdh, mode);
 	  at_vol = 1;
 	  if (c==-1) return -1;
 	  if (c==BAD_MAGIC) {
@@ -1623,31 +1709,19 @@ _errstat goto_end_vol(WINDOW *mes, int ln, _s32 vol, _s32 at_vol, _s8 get_th, _s
     if (have_fsf) {
 	if (!fast_fsf) {
 	    tape_close();			 /* close rewinding device opened earlier */
-	    if ((dv = ntape_open(O_RDWR)) == -1) /* now open non-rewinding so we can position */
+	    if ((dv = ntape_open(mode)) == -1) /* now open non-rewinding so we can position */
 	      return -1;
-	    if (to_end) {
-		if (tape_eom(FALSE) == -1) 
-		  return -1;
-	    }
-	    else {
-		if (tape_fsf(pass_vols, FALSE) == -1)/* reposition */
-		  return do_exit(ERROR_TAPE_FSF);
-	    }
+	    if (tape_fsf(pass_vols, FALSE) == -1)/* reposition */
+		return do_exit(ERROR_TAPE_FSF);
 	    tape_close();			 /* close it to effect change */
-	    if ((dv = tape_open(O_RDWR)) == -1)	 /* reopen at new position (rewinding device)*/
+	    if ((dv = tape_open(mode)) == -1)	 /* reopen at new position (rewinding device)*/
 	      return -1;
 	    return pass_vols+1;
 	}
 	else {					 /* have a fast fsf */
 	    if (!(pass_vols-at_vol+1)) return 0;
-	    if (to_end) {
-		if (tape_eom(FALSE) == -1) 
-		  return -1;
-	    }
-	    else {
-		if (tape_fsf(pass_vols-at_vol+1, FALSE) == -1) 
-		  return -1;
-	    }
+	    if (tape_fsf(pass_vols-at_vol+1, FALSE) == -1) 
+		return -1;
 	}
 	clear_buffer_pointers();
 	tape_set_blk_size();		 
@@ -1656,7 +1730,7 @@ _errstat goto_end_vol(WINDOW *mes, int ln, _s32 vol, _s32 at_vol, _s8 get_th, _s
 
     if (blocks_passed < pass_blocks) {		 /* ?need to advance */
 	for (c=blocks_passed; c<pass_blocks; c++) {   /* yes */
-	    if (raw_read(dv, read_buffer, block_size) != block_size)
+	    if (raw_read(dv, read_buffer, block_size, FALSE) != block_size)
 	      return do_exit(ERROR_SKIPPING);
 	}
     }
