@@ -1,12 +1,12 @@
 /*
-   Time-stamp: <96/08/05 19:32:56 yusuf>
+   Time-stamp: <97/08/16 17:33:46 yusuf>
 
-   $Id: taper.c,v 1.27 1996/08/05 19:02:03 yusuf Exp $	
+   $Id: taper.c,v 1.48 1997/12/28 06:48:43 yusuf Exp $	
 
 */
 
 #ifndef lint
-static char vcid[] = "$Id: taper.c,v 1.27 1996/08/05 19:02:03 yusuf Exp $";
+static char vcid[] = "$Id: taper.c,v 1.48 1997/12/28 06:48:43 yusuf Exp $";
 #endif /* lint */
 
 
@@ -42,7 +42,7 @@ PRIVATE void usage(void)
     fprintf(stderr, "-F file (--exclude-files)         which files not to include in archive\n");
     fprintf(stderr, "-g name (--volume-title)          title of volume\n");
     fprintf(stderr, "+/-G    (--get-blksize-[on|off])  Get block size after open\n");
-    fprintf(stderr, "+/- h   (--hard-links-[on|off])   turns on/off storing of links or actual files\n");
+    fprintf(stderr, "+/- h   (--soft-links-[on|off])   turns on/off storing of soft links as links or actual files\n");
     fprintf(stderr, "-H min  (--comp-head-start]       number of minutes head start for compressor\n");
     fprintf(stderr, "-i path (--info-files)            where the archive information files are\n");
     fprintf(stderr, "+/- I   (--compress-info-[on|off])turns on/off compression of info files\n");
@@ -50,7 +50,6 @@ PRIVATE void usage(void)
 /*    fprintf(stderr, "+/- J   (--info-at-end-[on|off])  turns on/off info file at end of archive\n");    not yet */
     fprintf(stderr, "-l file (--log-file)              name of log file\n");
     fprintf(stderr, "-m num  (--log-level)             level of logging (0=none..4=all)\n");
-    fprintf(stderr, "+/-M    (--memory-tight-[on|off]) limit memory use\n");
     fprintf(stderr, "-o num  (--overwrite)             level of file overwriting in restore\n");
     fprintf(stderr, "+/-O    (--tape-overwrite-[on|off]) overwrite tape if data exists on it\n");
     fprintf(stderr, "-p file (--preference-file)       name of preferences file\n");
@@ -159,7 +158,7 @@ PRIVATE void read_environment(void)
       strcpy(tape, getenv(TAPE_ENVIRON));
 
     if (getenv(NTAPE_ENVIRON))
-      strcpy(tape, getenv(NTAPE_ENVIRON));
+      strcpy(ntape, getenv(NTAPE_ENVIRON));
     
     strcpy(log_file, getenv("HOME"));
     strcat(log_file, "/"); 
@@ -242,6 +241,30 @@ PRIVATE _errstat evcol_pair(char *vl, char *fore, char *back)
 }
 
 
+PRIVATE void strip_xs_spaces(char *s)
+{
+/* Converts all multiple spaces to one space only.
+   Spaces are preserved within ' or " only
+
+   No trailing spaces allowed.
+*/
+
+    int in_q = 0;
+    int c, c1=0;
+
+    while (s[strlen(s)-1] == ' ')		 /* remove trailing */
+	s[strlen(s)-1] = 0;
+    
+    while (c1 < strlen(s)) {			 /* remove spaces */
+	if (s[c1] == '\'' || s[c1] == '"')
+	  in_q = !in_q;
+	if ((s[c1] == ' ') && (s[c1+1] == ' ') && (!in_q))
+	  for (c=c1; c<strlen(s); c++) s[c] = s[c+1];
+	else
+	  c1++;
+    }
+}    
+    
 PRIVATE void strip_spaces(char *s)
 /* strips spaces from the string in 's'.
  * Spaces are preserved within ' or " only
@@ -264,7 +287,7 @@ PRIVATE void strip_spaces(char *s)
 PRIVATE void process_pref_line(char *s)
 {
     int c=0;					 /* given a preference line - s */
-    char *val;
+    char val[MAX_FNAME], *x;
     
     strip_spaces(s);				 /* get rid of spaces */
     if (*s)
@@ -274,9 +297,10 @@ PRIVATE void process_pref_line(char *s)
 	return;
     if (strchr(s, '=') == 0)			 /* no equals */
       do_exit(ERROR_PREF_FORMAT);
-    val = strchr(s, '=')+1;
-    if (*val == '"')				 /* preference is surrounded by quotes */
-      val++;
+    x = strchr(s, '=')+1;
+    if (*x == '"')				 /* preference is surrounded by quotes */
+      x++;
+    strcpy(val, x);
     if (val[strlen(val)-1] == '"')
       val[strlen(val)-1] = 0;
     *strchr(s, '=') = 0;
@@ -359,8 +383,6 @@ PRIVATE void read_prefs_file(int argc, char *argv[])
     char s[1000];
     int line=0;
 
-    strcpy(pref_file, getenv("HOME")); strcat(pref_file, "/");
-    strcat(pref_file, DEFAULT_PREFS);		 /* default preference filename*/
     *pref_file = 0;
     c=1;					 /* check if command line name given */
     while (c < argc) {
@@ -406,6 +428,13 @@ PRIVATE void read_prefs_file(int argc, char *argv[])
 	}
 	return;
     }
+    while (!feof(fd)) {				 /* find tape name preference */
+	if (fgets(s, sizeof(s), fd) == NULL) break;
+	if (!strncasecmp(s, "tape-type", strlen("tape-type")))
+	    process_pref_line(s);
+    }
+    
+    rewind(fd);
     while (line++, !feof(fd)) {
 	if (fgets(s, sizeof(s), fd) == NULL)
 	  break;
@@ -536,6 +565,7 @@ PUBLIC _errstat fill_in_defaults()
 	if (can_seek == -1) can_seek = 0;
 	if (set_blksize == -1) set_blksize = 0;
 	if (get_blksize == -1) get_blksize = 0;
+	if (illegal_eot == -1) illegal_eot = 0;
 	have_rewind = 1;
 	have_fsf = 1;
 	break;
@@ -548,6 +578,7 @@ PUBLIC _errstat fill_in_defaults()
 	if (set_blksize == -1) set_blksize = 1;
 	if (get_blksize == -1) get_blksize = 1;
 	if (can_seek == -1) can_seek = 1;
+	if (illegal_eot == -1) illegal_eot = 0;
 	have_rewind = 1;
 	have_fsf = 1;
 	break;
@@ -560,6 +591,7 @@ PUBLIC _errstat fill_in_defaults()
 	if (can_seek == -1) can_seek = 1;
 	if (set_blksize == -1) set_blksize = 0;
 	if (get_blksize == -1) get_blksize = 0;
+	if (illegal_eot == -1) illegal_eot = 0;
 	have_rewind = 1;
 	have_fsf = 1;
 	break;
@@ -574,6 +606,7 @@ PUBLIC _errstat fill_in_defaults()
 	get_blksize = 0;
 	have_rewind = 1;
 	have_fsf = 0;
+	illegal_eot = 0;
 	break;
      case TAPE_TYPE_REMOVABLE:
 	if (erase_tape == -1) erase_tape = 0;
@@ -582,6 +615,7 @@ PUBLIC _errstat fill_in_defaults()
 	if (*ntape == -1) strcpy(ntape, REMOVABLE_DEFAULT_NTAPE);
 	fast_fsf = 0;
 	if (can_seek == -1) can_seek = 0;
+	if (illegal_eot == -1) illegal_eot = 0;
 	set_blksize = 0;
 	get_blksize = 0;
 	have_rewind = 0;
@@ -596,6 +630,7 @@ PUBLIC _errstat fill_in_defaults()
 	if (can_seek == -1) can_seek = 0;
 	if (set_blksize == -1) set_blksize = 0;
 	if (get_blksize == -1) get_blksize = 0;
+	if (illegal_eot == -1) illegal_eot = 0;
 	have_rewind = 1;
 	have_fsf = 1;
 	break;
@@ -616,6 +651,7 @@ PRIVATE void setup_colors(void)
 	init_pair(COLOR_DIRECTORY, color_directory_foreground, color_directory_background);
 	init_pair(COLOR_ONVOL, color_onvol_foreground, color_onvol_background);
 	init_pair(COLOR_SELECTED, color_selected_foreground, color_selected_background);
+	init_pair(COLOR_EXCLUDED, color_excluded_foreground, color_excluded_background);
 	init_pair(COLOR_BOTTOM, color_bottom_foreground, color_bottom_background);
 	init_pair(COLOR_TITLE, color_title_foreground, color_title_background);
 	init_pair(COLOR_FORM, color_form_foreground, color_form_background);
@@ -660,6 +696,7 @@ PRIVATE void num_to_string(_s32 num, int rev, int top, int line, char *strings[]
 PRIVATE void print_pref_line(struct pref_info options[], int c, int top, int rev)
 {
     char *py[]={"NO", "YES", NULL};
+    char *pS[]={"LINK", "FILE", NULL};
     char *pl[]={"OFF", "ERRORS", "INFO", "VERBOSE", "ALL", NULL};
     char *pc[]={"NONE", "E GZIP", "I COMPRESS", "I GZIP", NULL};
     char *po[]={"OFF", "MORE RECENT", "ALWAYS", NULL};
@@ -679,6 +716,9 @@ PRIVATE void print_pref_line(struct pref_info options[], int c, int top, int rev
      case 'Y':				 /* yes/no */
 	num_to_string(*((char *) options[c].opt), rev, top, c, py);
 	break;
+    case 's':
+	num_to_string(*((char *) options[c].opt), rev, top, c, pS);
+	break;
      case 'L':				 /* log level */
 	num_to_string(*((char *) options[c].opt), rev, top, c, pl);
 	break;
@@ -772,6 +812,9 @@ PRIVATE void option_left(struct pref_info *options, _s32 op, int top, _s8 allab,
      case 'Y':
 	do_left(options[op].opt, 2);
 	break;
+     case 's':
+	do_left(options[op].opt, 2);
+	break;
      case 'I':
 	*((_s32 *) options[op].opt) = *((_s32 *) options[op].opt) -1;
 	if (*((_s32 *) options[op].opt) < 0)
@@ -810,6 +853,9 @@ PRIVATE void option_right(struct pref_info *options, _s32 op, int top, _s8 allab
      case 'Y':
 	do_right(options[op].opt, 2);
 	break;
+     case 's':
+	do_right(options[op].opt, 2);
+	break;
      case 'B':
 	block_size += 1024;
 	if (block_size < 0) 
@@ -930,11 +976,11 @@ PUBLIC _errstat taper_change_prefs(struct pref_info *options, _s8 allab)
 	 case KEY_DOWN:
 	 case '\n':
 	 case '\r':
-	    c++; 
-	    break;
+	     c++; 
+	     break;
 	 case KEY_UP:
-	    c--; 
-	    break;
+	     c--; 
+	     break;
 	 case ABORT_KEY:
 	    if (allab) return -1;
 	    break;
@@ -946,8 +992,7 @@ PUBLIC _errstat taper_change_prefs(struct pref_info *options, _s8 allab)
 		 case 'S':
 		    my_strcpy(options[c].opt, field_buffer(fields[k], 0), 
 			      options[c].length);   /* appropriate vars */
-		    if (!strcmp(options[c].opt, " "))/* if space only */
-		      *(char *) options[c].opt = 0;  /* make NULL */
+		    strip_xs_spaces(options[c].opt);
 		    k++;
 		    break;
 		 case 'N':
@@ -1027,17 +1072,15 @@ PRIVATE char *oo(_s32 o)
 
 PRIVATE void taper_save_preference_file(void)
 {
-    struct pref_info o[] = { {"Preference file", pref_file, 'S', MAX_FNAME},
-			     {NULL, NULL, 0}
-		       };
     FILE *pf;
     char s[MAX_FNAME];
     int c=0;
     char tp;
     
-    if (taper_change_prefs(o, 1) == -1)
-      return;
-    if (*pref_file != '/') {
+    if (!get_string(win_main, pref_file, sizeof(pref_file), "Enter preference filename"))
+	return;
+    if (!message_box("Confirm save", MB_YESNO)) return;
+    if ((*pref_file != '/') && (!strchr(pref_file, ':'))) {
 	getcwd(s, sizeof(s));
 	strcat(s, "/");
 	strcat(s, pref_file);
@@ -1049,7 +1092,7 @@ PRIVATE void taper_save_preference_file(void)
 	do_exit(ERROR_CREATING_PREFS);
 	return;
     }
-    tp = *make_tt(tape_type);
+    tp = make_short_tt(tape_type);
     
     if (ap[c].save_pref) {
 	while (ap[c].pref_name) {
@@ -1098,15 +1141,13 @@ PRIVATE void taper_save_preference_file(void)
 PRIVATE void taper_save_command_line(void)
 {
     char s[MAX_FNAME];
-    struct pref_info o[] = { {"Command line file", cf, 'S', MAX_FNAME},
-			     {NULL, NULL, 0}
-		       };
     FILE *pf;
     int  c=0;
 
     strcpy(cf, "start_taper");
-    if (taper_change_prefs(o, 1) == -1)
-      return;
+    if (!get_string(win_main, cf, sizeof(cf), "Enter command line filename"))
+	return;
+    if (!message_box("Confirm save", MB_YESNO)) return;
     if (*cf != '/') {
 	getcwd(s, sizeof(s));
 	strcat(s, "/");
@@ -1171,7 +1212,9 @@ PRIVATE void taper_init_windows(void)
     curs_set(0);
 }
 
+#ifndef GLIBC_2
 typedef void (*sighandler_t)(int);		 /* makes for clearer def */
+#endif
     
 PRIVATE void my_int_sig(int sig)
 /* Handles the INT signal. Basically doesn't allow the user to interrupt -
@@ -1225,10 +1268,11 @@ PRIVATE void preferences_menu(void)
 	{"Block size", &block_size, 'B', 0},
 	{"Need to erase tapes", &erase_tape, 'Y', 0},
 	{"Tape size in MB", &tape_size, 'N', 0},
+	{"Use eom", &use_eom, 'Y', 0},
+	{"Illegal end of tape", &illegal_eot, 'Y', 0},
 	{NULL, NULL, 0, 0}
     };
     struct pref_info global_options[] = { 	
-	{"Memory tight", &memory_tight, 'Y', 0},
 	{"Info file directory", taper_info_files, 'S', MAX_ARCHIVETITLE},
 	{"Temporary files directory", temp_dir, 'S', MAX_FNAME},  
 	{"Log file", log_file, 'S', MAX_FNAME},
@@ -1236,8 +1280,7 @@ PRIVATE void preferences_menu(void)
 	{"Max log size", &log_file_size, 'I', 0},
 	{"Strip path", &ostrip, 'I', 0},
 	{"Prompt directories", &dir_selection, 'Y', 0},
-	{"Alpha sort archives", &sort_dir, 'Y', 0},
-	{"/proc device", &proc_dev, 'I', 0},
+ 	{"/proc device", &proc_dev, 'I', 0},
 	{NULL, NULL, 0, 0}
     };
     struct pref_info backup_options_1[] = { {"Archive title", archive_title, 'S', MAX_ARCHIVETITLE},
@@ -1247,14 +1290,15 @@ PRIVATE void preferences_menu(void)
         {"Exclude dirs", &exclude_dirs, 'S', MAX_FNAME},
 	{"Compress", &compression, 'C', 0},
 	{"Compression head start", &comp_head_start, 'I', 0},
-	{"Hard links", &hard_links, 'Y', 0},
+	{"Soft links", &soft_links, 's', 0},
 	{NULL, NULL, 0, 0}
     };
     struct pref_info backup_options_2[] = { 
 	{"Compress info files", &compress_info, 'Y', 0},  
 	{"Limit to one filesystem", &ofs, 'Y', 0},  
 	{"Default incremental backup", &incremental, 'Y', 0},
-	{"Free space for compress", &min_free, 'N', 0},  
+	{"Free space for compress", &min_free, 'N', 0},
+	{"Size directories", &size_dirs, 'Y', 0},
 /*	{"Write info at end of tape", &info_end, 'Y', 0}, not yet */
 	{NULL, NULL, 0, 0}
     };
@@ -1266,6 +1310,7 @@ PRIVATE void preferences_menu(void)
 	{"Auto descend directories", &auto_descend, 'Y', 0},
 	{"Prompt if bad checksum", &bad_checksum, 'Y', 0},
 	{"Min blocks before seeking", &min_seek, 'I', 0},
+        {"Set ownership of files", &set_owner, 'Y', 0},
 	{"Using pre-5.6 archive", &old_archive, 'Y', 0},  
 	{NULL, NULL, 0, 0}
     };
@@ -1282,6 +1327,7 @@ PRIVATE void preferences_menu(void)
 	switch (select_menu(win_main, menu, &in_op)) {
 	 case 0:					 /* global options */
 	    taper_change_prefs(tapedrive_options, 0);
+	    if (init_buffers(1) == -1) do_exit(ERROR_SETTING_BLKSIZE);/* reallocate buffer sizes */
 	    break;
 	 case 1:					 /* global options */
 	    taper_change_prefs(global_options, 0);
@@ -1312,14 +1358,18 @@ void utils_whereproc(void);
 void utils_erasevols(void);
 void utils_test_mktape(void);
 void utils_test_can_seek(void);
+void utils_test_eot(void);
 void utils_testlinks(void);
+void utils_reindex(void);
+void utils_recover(void);
 PRIVATE void utilities(void)
 {
     char *menu[] = {
 	"Make tape",  "Where's proc", 
 	  "Test fast fsf", "Test make tape",
-	  "Test can seek",
+	  "Test can seek", "Test end of tape", "Reindex info file",
 	  "Erase volumes", "Look for recursive links",
+	"Recover module",
 	  "Back to main menu", NULL};
     int in_op=0;
     
@@ -1341,12 +1391,20 @@ PRIVATE void utilities(void)
 	 case 4:
 	    utils_test_can_seek();
 	    break;
-	 case 5:
+	case 5:
+	    utils_test_eot();
+	    break;
+	case 6:
+	    utils_reindex();
+	    break;
+	 case 7:
 	    utils_erasevols();
 	    break;
-	 case 6:
+	 case 8:
 	    utils_testlinks();
-	 case 7:
+	case 9:
+	    utils_recover();
+	 case 10:
 	    return;
 	}
 	my_delwin_all();				 /* delete any open windows */
