@@ -1,7 +1,7 @@
 /*
-   Time-stamp: <96/07/19 20:18:46 yusuf>
+   Time-stamp: <97/04/20 10:04:17 yusuf>
 
-   $Id: taper.h,v 1.9 1996/07/27 20:42:17 yusuf Exp $
+   $Id: taper.h,v 1.14 1997/06/29 01:31:26 yusuf Exp $
 */
 
 
@@ -9,10 +9,15 @@
 
 #define __FROM_MAIN__
 
-
+#include "config.h"
 #include "version.h"
-
 #include <stdlib.h>
+#ifdef ASM_STRING
+  #include <linux/string.h>
+#else
+  #include <string.h>
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -31,6 +36,7 @@
 #include <sys/sem.h>
 #include <sys/mount.h>
 #include <sys/wait.h>
+#include <sys/time.h>
 #include <curses.h>
 #include <form.h>
 
@@ -44,3 +50,12 @@
 #include "common.h"
 #include "tapeio.h"
 #include "endianize.h"
+#include "library.h"
+#include "fifo.h"
+#include "rmt.h"
+
+
+#include <stdio.h>
+#include <sys/types.h>
+#include <signal.h>
+
