@@ -1,12 +1,12 @@
 /*
-   Time-stamp: <96/07/19 20:13:36 yusuf>
+   Time-stamp: <97/06/29 12:32:26 yusuf>
 
-   $Id: utils.c,v 1.13 1996/07/27 20:42:21 yusuf Exp $	
+   $Id: utils.c,v 1.30 1997/06/29 09:09:24 yusuf Exp $	
 
 */
 
 #ifndef lint
-static char vcid[] = "$Id: utils.c,v 1.13 1996/07/27 20:42:21 yusuf Exp $";
+static char vcid[] = "$Id: utils.c,v 1.30 1997/06/29 09:09:24 yusuf Exp $";
 #endif /* lint */
 
 
@@ -38,7 +38,7 @@ PUBLIC _errstat utils_write_nullheader(WINDOW **mes)
     status_box(*mes, "Writing tape", 1, FALSE, 1);
     memcpy(write_buffer, &tdh, sizeof(tdh));
     if (log_level > 2) write_log("Writing null tape header");
-    if (write(dv, write_buffer, block_size) != block_size)
+    if (rmtwrite(dv, write_buffer, block_size) != block_size)
       return do_exit(ERROR_WRITING);
     status_box(*mes, "Closing tape", 1, FALSE, 1);
     return 0;
@@ -141,6 +141,9 @@ PUBLIC void utils_test_can_seek(void)
     WINDOW *mes=NULL;
     int   c;
     int  old_cseek;
+    _u32  tl;
+    char s[MAX_FNAME];
+    char cant=0;
     
     clear_main();
     if (open_logfile("Test can seek") == -1) return;
@@ -178,15 +181,29 @@ PUBLIC void utils_test_can_seek(void)
 	status_box(mes, "Testing seek on volume #1", 1, FALSE, 1);
 	if (tape_open(O_RDWR) == -1) goto fin;
 	if (log_level > 2) write_log("Reading block #3");
-	if (tape_seek(3) == -1) goto cant;
+	if (tape_seek(3, TRUE) == -1) cant = 1;
+	tape_tell(&tl);
+	sprintf(m, "Tell returned block %d", tl);
+	if (log_level > 2) write_log(m);
 	memset(m, 0, block_size);
 	if (tape_read(m, block_size) != block_size) goto fin;
-	if (strcmp(m, "Volume#1 - block #3")) goto cant;
+	if (log_level > 2) {
+	    sprintf(s, "Read %100s", m);
+	    write_log(s);
+	}
+	if (strcmp(m, "Volume#1 - block #3")) cant=1;
 	if (log_level > 2) write_log("Reading block #7");
-	if (tape_seek(7) == -1) goto cant;
+	if (tape_seek(7, TRUE) == -1) cant=1;
+	tape_tell(&tl);
+	sprintf(m, "Tell returned block %d", tl);
+	if (log_level > 2) write_log(m);
 	memset(m, 0, block_size);
 	if (tape_read(m, block_size) != block_size) goto fin;
-	if (strcmp(m, "Volume#1 - block #7")) goto cant;
+	if (log_level > 2) {
+	    sprintf(s, "Read %100s", m);
+	    write_log(s);
+	}
+	if (strcmp(m, "Volume#1 - block #7")) cant=1;
 	status_box(mes, "Advancing to volume #2", 1, FALSE, 1);
 	if (log_level > 2) write_log("Advancing to vol #2");
 	tape_close();
@@ -205,35 +222,109 @@ PUBLIC void utils_test_can_seek(void)
 	}
 	status_box(mes, "Testing seek on volume #2", 1, FALSE, 1);
 	if (log_level > 2) write_log("Reading block #5");
-	if (tape_seek(5) == -1) goto cant;
+	if (tape_seek(5, TRUE) == -1) cant=1;
+	tape_tell(&tl);
+	sprintf(m, "Tell returned block %d", tl);
+	if (log_level > 2) write_log(m);
 	memset(m, 0, block_size);
 	if (tape_read(m, block_size) != block_size) goto fin;
-	if (strcmp(m, "Volume#2 - block #5")) goto cant;
+	if (log_level > 2) {
+	    sprintf(s, "Read %100s", m);
+	    write_log(s);
+	}
+	if (strcmp(m, "Volume#2 - block #5")) cant=1;
 	if (log_level > 2) write_log("Reading block #8");
-	if (tape_seek(8) == -1) goto cant;
+	if (tape_seek(8, TRUE) == -1) cant=1;
+	tape_tell(&tl);
+	sprintf(m, "Tell returned block %d", tl);
+	if (log_level > 2) write_log(m);
 	memset(m, 0, block_size);
 	if (tape_read(m, block_size) != block_size) goto fin;
-	if (strcmp(m, "Volume#2 - block #8")) goto cant;
-	message_box("Your tape drive can seek", MB_OK);
-	if (log_level) write_log("Your tape drive can seek");
-	goto fin;
-	
-	cant:;
+	if (log_level > 2) {
+	    sprintf(s, "Read %100s", m);
+	    write_log(s);
+	}
+	if (strcmp(m, "Volume#2 - block #8")) cant=1;
+	if (!cant) {
+	    message_box("Your tape drive can seek", MB_OK);
+	    if (log_level) write_log("Your tape drive can seek");
+	}
+	else {
 	  message_box("Your tape drive cannot seek", MB_OK);
 	  if (log_level) write_log("Your tape drive cannot seek");
-	
+	}
     }
+
+ fin:;
+    if (m) my_free(m);
+    can_seek = old_cseek;
+    touchwin(mes); 	
+    status_box(mes, "Closing tape", 1, FALSE, 1);
+    tape_close();
+    final_message("Test can seek");
+    close_statusbox(mes);
+    close_logfile("Test can seek");
+    return;		
+}
+
+
+
+PUBLIC void utils_test_eot(void)
+{
+    char *m=NULL;
+    WINDOW *mes=NULL;
+    int   rc;
+    char s[MAX_FNAME];
     
-    fin:;
-      if (m) my_free(m);
-      can_seek = old_cseek;
-      touchwin(mes); 	
-      status_box(mes, "Closing tape", 1, FALSE, 1);
-      tape_close();
-      final_message("Test can seek");
-      close_statusbox(mes);
-      close_logfile("Test can seek");
-      return;		
+    clear_main();
+    if (open_logfile("Test end of tape") == -1) return;
+    if (check_device_names() == -1) return;	 /* check devices & other parms */
+    if (!message_box("This will erase ALL data. Proceed", MB_YESNO)) {
+	close_logfile("Test end of tape");
+	return;
+    }
+    m = my_malloc(block_size);
+    if (tape_open(O_RDWR) == -1) goto fin;
+    status_box(mes, "Writing data", 1, FALSE, 1);
+    if (log_level) write_log("Writing data");
+    while (1)				 /* fill tape with data */
+	if (rmtwrite(dv, m, block_size) <block_size) break;
+    tape_close();
+
+
+    rc = 0;					 /* now read in all data */
+    if (log_level) write_log("Reading data");
+    while (1) {
+	sprintf(s, "Up to %d bytes", rc);
+	status_box(mes, s, 1, FALSE, 1);
+	if (rmtread(dv, m, block_size) < block_size) break;
+	rc+= block_size;
+    }
+
+    status_box(mes, "Short read - doing further read", 1, FALSE, 1);
+    if (log_level) write_log("Short read - doing further read");
+    rc = rmtread(dv, m, block_size);
+    tape_close();
+    close_statusbox(mes);
+    if (rc == -1) {
+	if (!((errno == ENOSPC) || (errno == ENODATA)))
+	    strcpy(s,"Your tape drive returns illegal error at end of tape");
+	else
+	    strcpy(s, "Your tape drive returns legal error at end of tape");
+    }    
+    else
+	if (rc == 0)
+	    strcpy(s, "Your tape drive returns 0 at end of tape");
+	else
+	    strcpy(s, "Didn't reach end of tape");
+    if (log_level) write_log(s);
+    message_box(s, MB_OK);
+
+ fin:;
+    if (m) my_free(m);
+    final_message("Test end of tape");
+    close_logfile("Test end of tape");
+    return;		
 }
 
 
@@ -264,17 +355,11 @@ PUBLIC void utils_erasevols(void)
     static _s32 vols;				 /* needs to be */
     char s[100];				 /* static so can use initialiser */
     _u32 arch;
-    _s32  x, onia, pos;
+    _s32  x, onia, x1;
     struct volume_header vh;
-    char *new_info=NULL, *new_cp;
-    _s32 new_cur_size;
-    struct file_info *fi;
+    struct info_file_data i_data;
+    struct info_file_key i_key;
     
-    struct pref_info get_vols[] = {
-	{"No vols to delete", &vols, 'I', 0},
-	{NULL, NULL, 0, 0}
-    };
-
     vols = 0;
     init_common_vars();
     backrest_do_mallocs();
@@ -289,9 +374,11 @@ PUBLIC void utils_erasevols(void)
       goto end;
     if (read_vol_dir(arch) == -1)
       goto end;					 /*  Read volume dir */
-    if (taper_change_prefs(get_vols, 1) == -1)
-      goto end;
 
+    *s=0;
+    if (!get_string(win_main, s, sizeof(s), "Enter # volumes to delete"))
+	goto end;
+    vols = atoi(s);
     if (vols == 0)				 /* no volumes selected */
       goto end;
     
@@ -317,7 +404,7 @@ PUBLIC void utils_erasevols(void)
     mes_box = status_box(mes_box, "Opening tape", 3, FALSE, 1);/* create window */
 
     ifd.number_volumes -= vols;
-    if (goto_end_vol(mes_box, 3, ifd.number_volumes, 1, TRUE, FALSE) == -1)
+    if (goto_end_vol(mes_box, 3, ifd.number_volumes, 1, TRUE, O_RDWR) == -1)
       goto end;
 
     write_offset = 0;				 /* make sure nothing's ahead of us */
@@ -330,48 +417,34 @@ PUBLIC void utils_erasevols(void)
     tape_close();
 
     status_box(mes_box, "Updating info files", 3, FALSE, 1);
-    ifd.info_file_size = sizeof(struct info_file_header);
-    onia = ifd.no_in_archive;
-    ifd.no_in_archive=0;
-    new_info = my_malloc(MEM_BLOCK_SIZE);
-    new_cur_size = MEM_BLOCK_SIZE;
-    cp = info;					 /* now remove entries from */
-    cp += sizeof(struct info_file_header);	 /* info file that are not needed */
-    new_cp = new_info;
-    new_cp += sizeof(struct info_file_header);
-    for (x=0; x<onia; x++) {
-	fi = (struct file_info *) cp;
-	if (abs(fi->volume) <= ifd.number_volumes) {/* want this entry */
-	    ifd.info_file_size += sizeof(struct file_info) + fi->name_len;
-	    if (ifd.info_file_size > new_cur_size) {	 /* will exceed memory block */
-		pos = new_cp - new_info;				 /* keep cp position */
-		new_info = my_realloc(new_info, new_cur_size + MEM_BLOCK_SIZE);
-		if (new_info == NULL) {
-		    new_cur_size = 0;
-		    do_exit(ERROR_MEMORY);
-		    goto end;
-		}
-		new_cp = new_info + pos;
-		new_cur_size += MEM_BLOCK_SIZE;
-	    }
-	    ifd.no_in_archive++;		 /* increment # in archive */
-	    memcpy(new_cp, cp, sizeof(struct file_info));   /* copy from old info to new info */
-	    strcpy(new_cp+sizeof(struct file_info),
-		   cp+sizeof(struct file_info));
-	    new_cp += sizeof(struct file_info);
-	    PAST_STRING_FI(new_cp);
-	}
-	cp += sizeof(struct file_info);		 /* advance in old info */
-	PAST_STRING_FI(cp);
+    onia = ifd.no_in_archive;			 /* save old # files */
+    ifd.no_in_archive=0;			 /* reset tree pointers */
+    for (x=0; x<INFO_INDICIES; x++) {		 /* quicker & more efficient */
+	ifd.free[x] = 0;			 /* to rebuild tree than try */
+	ifd.end[x] = 0;				 /* to delete each entry */
+	ifd.root[x] = 0;
     }
+    ifd.archive_id = tdh.archive_id;
+    for (x=0; x<onia; x++) {
+	if (read_info_rec(x, &i_data) == -1) {
+	    do_exit(ERROR_READING_INFO);
+	    return;
+	}
+	if (abs(i_data.f.volume) > ifd.number_volumes) break; /* don't need any more */
+	fill_info_key(i_data, &i_key);
+	for (x1=0; x1<INFO_INDICIES; x1++) 		 /* add to indexes */
+	    insertb(0, &i_key, x1);
+    }
+    ifd.no_in_archive = x-1;
+    if (process_info(mes_box, 3, FALSE)) goto end;
+
     ifd.size_volume_headers = get_vh(vol_headers, ifd.number_volumes+1)
                                  - (char *) vol_headers;/* calculate new size */
     
-    memcpy(new_info, &ifd, sizeof(ifd));	 /* copy header file */
-    write_out_info_file(new_info, ifd.archive_id);
+    write_info_file();
+    close_info_file();
     
     end:;
-    if (new_info) my_free(new_info);
     backrest_kill_windows();
     print_my_name();
     final_message("Erase vols");
@@ -381,6 +454,79 @@ PUBLIC void utils_erasevols(void)
 }
 
 
+PUBLIC void utils_reindex(void)
+{
+    WINDOW *mes_box=NULL;
+    static _s32 vols;				 /* needs to be */
+    _u32 arch;
+    _s32  x, x1;
+    struct info_file_data i_data;
+    struct info_file_key i_key;
+    char s[100];
+    char info_file[MAX_FNAME];
+    
+
+    vols = 0;
+    init_common_vars();
+    backrest_do_mallocs();
+    if (check_device_names() == -1) return;	 /* check devices & other parms */
+    if (open_logfile("Reindex info file") == -1)
+      return;
+    if (no_windows)
+      prompt_archive = 0;
+    backrest_init_windows();
+    backrest_clear_screen();
+    if (select_archive(&arch, FALSE) ==-1)	 /* find out which archive user wants */
+      goto end;
+    if (do_read_vol_dir(arch, tape, O_RDONLY, TRUE, TRUE, FALSE) == -1)
+      goto end;					 /*  Read volume dir */
+
+    mes_box = status_box(mes_box, "                                         ", 2, TRUE, 5);/* create window */
+    nodelay(mes_box, TRUE);
+    status_box(mes_box, "Reindexing info file", 3, FALSE, 1);/* create window */
+
+    for (x=0; x<INFO_INDICIES; x++) {		 /* quicker & more efficient */
+	ifd.free[x] = 0;			 /* to rebuild tree than try */
+	ifd.end[x] = 0;				 /* to delete each entry */
+	ifd.root[x] = 0;
+    }
+    make_info_filename(info_file, ifd.archive_id);
+    for (x=1;x<=INFO_INDICIES; x++) 
+	if (open_one_index(info_file, FALSE, x) == -1)
+	    return;
+	
+    for (x=0; x<ifd.no_in_archive; x++) {
+	sprintf(s, "File %d of %d", x, ifd.no_in_archive);
+	status_box(mes_box, s, 4, FALSE, 1);
+	if (read_info_rec(x, &i_data) == -1) {
+	    do_exit(ERROR_READING_INFO);
+	    return;
+	}
+	fill_info_key(i_data, &i_key);
+	for (x1=0; x1<INFO_INDICIES; x1++) 		 /* add to indexes */
+	    insertb(0, &i_key, x1);
+    }
+    status_box(mes_box, "", 3, FALSE, 1);
+    status_box(mes_box, "", 4, FALSE, 1);
+    if (process_info(mes_box, 3, FALSE)) goto end;
+    status_box(mes_box, "", 1, FALSE, 1);
+    
+    status_box(mes_box, "Writing info headers", 3, FALSE, 1);
+    write_info_file();
+    status_box(mes_box, "Closing/Compressing info file", 3, FALSE, 1);
+    close_info_file();
+    close_statusbox(mes_box);
+    
+    end:;
+    backrest_kill_windows();
+    print_my_name();
+    final_message("Reindex info file");
+    backrest_free_memory();
+    close_logfile("Reindex info file");
+    return;			                 /* succesful return */
+}
+
+
 PUBLIC void utils_test_mktape(void)
 {
     _s32 x;
@@ -398,10 +544,10 @@ PUBLIC void utils_test_mktape(void)
     mes = status_box(mes, "Opening tape device", 1, TRUE, 1);
     if (tape_open(O_RDWR) == -1) goto exit;
     status_box(mes, "Trying to read data", 1, FALSE, 1);
-    x = read(dv, write_buffer, block_size);
+    x = rmtread(dv, write_buffer, block_size);
     if (x==block_size) {
 	message_box("You do not need to run mktape on new tapes", MB_OK);
-        if (log_level > 1) write_log("You do not need to run mktape on new tapes");
+        if (log_level) write_log("You do not need to run mktape on new tapes");
 	goto exit;
     }
 /* See if the situation resolves by writing data */    
@@ -419,7 +565,7 @@ PUBLIC void utils_test_mktape(void)
       goto exit;
     status_box(mes, "Re-reading data", 1, FALSE, 1);
     memset(write_buffer, 0, block_size);
-    x = read(dv, write_buffer, block_size);
+    x = rmtread(dv, write_buffer, block_size);
     if (x==-1) {
 	do_exit(ERROR_READING);
 	goto exit;
@@ -427,7 +573,7 @@ PUBLIC void utils_test_mktape(void)
     tdh = (struct tape_header *) write_buffer;
     if (tdh->magic == TAPER_MAGIC_NUMBER) {
 	message_box("You need to run mktape on new tapes", MB_OK);
-        if (log_level > 1) write_log("You need to run mktape on new tapes");
+        if (log_level) write_log("You need to run mktape on new tapes");
 	goto exit;
     }
     message_box("ERROR: Didn't read what was written", MB_OK);
@@ -488,10 +634,10 @@ PUBLIC void utils_testlinks(void)
     is_rec = FALSE;
     strcpy(dir_list, s);
     mes = status_box(mes, "Checking directories", 1, TRUE, 1);
-    oldhl = hard_links;				 /* save old hard_links */
-    hard_links = TRUE;				 /* say yes to trace links */
+    oldhl = soft_links;				 /* save old hard_links */
+    soft_links = TRUE;				 /* say yes to trace links */
     process_dir(mes, 1, s, 0, utils_testlinks_dpd, TRUE);
-    hard_links = oldhl;
+    soft_links = oldhl;
     close_statusbox(mes); touchwin(win_main); wrefresh(win_main);
     if (is_rec == TRUE) 
 	sprintf(s, "Recursive link in %s", dir_list+(len-1)*MAX_FNAME);
@@ -501,3 +647,81 @@ PUBLIC void utils_testlinks(void)
     my_free(dir_list);
 }
     
+
+extern PUBLIC _s32 file_no;					 /* for cumulative  */
+extern PUBLIC _u32 mktr, mktrc;
+PUBLIC _errstat mkinfo_loop(file_passed_action action, print_status ps);
+PUBLIC void mkinfo_print_status(WINDOW *mes_box, _s32 cur_in_vol, 
+			 _s32 no_in_vol, _s32 vol,
+			 _u32 file_size, char *fn,
+			 time_t t_start, time_t t_current);
+PUBLIC _errstat restore_action(struct file_info *fi, char *fn, struct info_file_data *i_data);
+PUBLIC _errstat restore_start_child(WINDOW *mes_box);
+PUBLIC void end_restore(void);
+PUBLIC void utils_recover()
+{
+    _u32 prev_off=0;
+    _s32 x, x1, c;
+    char aprev_off[200], s[200];
+    char *m;
+    struct info_file_data ifd;
+    char lns[14][150];
+    time_t t_start;
+    WINDOW *mes_box=NULL;
+        
+    clear_main();
+    if (open_logfile("Attempt Recovery") == -1) return;
+    if (check_device_names() == -1) return;	 /* check devices & other parms */
+
+    if (tape_open(O_RDWR) == -1) return;
+    strcpy(aprev_off, "0");
+    m = my_malloc(block_size);
+    if (m == NULL) {do_exit(ERROR_MEMORY); return;}
+    while (1) {
+	if (tape_rewind_engine(FALSE) == -1) goto fin;
+	sprintf(s, "Previous offset = %u. Enter new offset", prev_off);
+	sprintf(aprev_off, "%u", prev_off+1);
+	if (!get_string(win_main, aprev_off, sizeof(aprev_off), s))
+	    goto fin;
+	prev_off = atoi(aprev_off);
+	x = prev_off;
+	while (x) {				 /* move to correct offset */
+	    x1 = tape_read(m, min(x, block_size));
+	    if (x1 < min(x, block_size)) goto fin;
+	    x -= x1;
+	}
+	if (tape_read_fi(&ifd.f) == -1) goto fin;
+	if (tape_read(ifd.name, min(MAX_FNAME, ifd.f.name_len)) == -1) goto fin;
+	restore_ed_engine(&ifd, lns);
+	sprintf(lns[11], "Current offset: %u", prev_off);
+	strcpy(lns[12], "");
+	if (multi_message_box(lns, 13, MB_NOYES, FALSE))
+	    break;
+    }
+    if (!message_box("Start restore from here", MB_YESNO))
+	goto fin;
+    /* Advance to correct position */    
+    mes_box = status_box(mes_box, "Moving to place", 2, TRUE, 5);/* create window */
+    nodelay(mes_box, TRUE);
+    if (tape_rewind_engine(FALSE) == -1) goto fin;
+	x = prev_off;
+	while (x) {				 /* move to correct offset */
+	    x1 = tape_read(m, min(x, block_size));
+	    if (x1 < min(x, block_size)) goto fin;
+	    x -= x1;
+	}
+    status_box(mes_box, "Starting restore", 2, FALSE, 4);
+    restore_mode = RESTORE_FULL;
+    if (restore_start_child(mes_box) == -1) return;
+    t_start = time(NULL);
+    traverse_volume(restore_action, -1, t_start, mes_box, TRUE,
+		    mkinfo_print_status, &c, FALSE, NULL);
+    end_restore();
+    close_statusbox(mes_box);
+    return;
+	    
+ fin:;
+    my_free(m);					
+    close_logfile("Attempt Recover");
+    tape_close();
+}
