@@ -1,7 +1,7 @@
 /*
-   Time-stamp: <96/08/05 19:31:24 yusuf>
+   Time-stamp: <98/03/03 12:29:40 yusuf>
 
-   $Id: vars.h,v 1.13 1996/08/05 19:02:05 yusuf Exp $
+   $Id: vars.h,v 1.29 1998/03/03 04:35:11 yusuf Exp $
 */
 
 
@@ -50,20 +50,20 @@ extern UBYTE *comp_buffer1;			 /* compression buffer */
 extern UBYTE *comp_buffer2;			 /* compression buffer */
 
 /* Common window variables */
-extern WINDOW    *win_main, *title, *files, *selection, *bottom, *on_vol;
+extern WINDOW    *win_main, *title, *files, *selection, *bottom, *on_vol, *exclude;
 extern int       screen_ylen, screen_xlen;		 /* physical dims of screen */
 extern int       screen_ylen_files, screen_ylen_selection;/* length of screens */
-extern int       left_width, right_width;
+extern int       top_left_width, top_right_width;
+extern int       btm_left_width, btm_right_width;
 
 
 /* Options */
 extern int  append;
 extern int  auto_descend;
-extern int  hard_links;
+extern int  soft_links;
 extern int   lstrip, ostrip;
 extern int  tape_overwrite;
 extern int  compression;				/* Default is internal compression */
-extern int  sort_dir;
 extern char  archive_title[MAX_ARCHIVETITLE];
 extern char  volume_title[MAX_ARCHIVETITLE];
 extern char  exclude_compress[MAX_FNAME];	 /* files to exclude in compression */
@@ -75,8 +75,10 @@ extern char  rel_path[MAX_FNAME];
 extern int  incremental;
 extern int  most_recent;
 extern int  prompt_archive;				 
+extern int  size_dirs;				 /* whether to size dirs in backup */
 extern _s32 pr_dir;
 extern _u32 diff_id;
+extern int  set_owner;				 /* ?set owner of file after restore */
 extern int  info_end;
 extern int  no_windows;
 extern char taper_info_files[MAX_FNAME];
@@ -91,13 +93,15 @@ extern dev_t proc_dev;				 /* device on which /proc is mounted */
 extern int   only_vol;				 /* limit restore to volume */
 extern int   comp_head_start;			 /* how much head start to give compression */
 extern char  pref_file[MAX_FNAME];		 /* file of preference file */
-extern int   memory_tight;			 /* memory tight  */
 extern int   bad_checksum;			 /* ?prompt if encounter bad checksum */
 extern char  temp_dir[MAX_FNAME];		 /* temporary directory */
 extern int   ofs;				 /* stay on one filesystem */
 extern int   old_archive;			 /* reconstructing old archive */
 extern _s32  min_free;
 extern int min_seek;				 /* min blocks to move before seeking */
+extern _s32  unattended_id;			 /* when unattended backing up, ensure this id is in drive */
+extern int use_eom;				 /* use eom rather than fsf */
+extern _s8 other_mes;				 /* tells backup refresh that another box is on top */
 
 /*  More options - these are all things that depend on type of tape drive used */
 extern signed int tape_type;
@@ -111,13 +115,16 @@ extern signed int  block_size;			 /* is used for reading/writing data to/from ta
 extern signed int   have_rewind;			 
 extern signed int   have_fsf;
 extern signed int   can_seek;
+extern signed int illegal_eot;			 /* end of tape handling */
 
 /* File counts and archive size counts */
 extern  _u32 total_compressed;			 /* # file bytes in volume - compressed */
 extern  _u32 total_uncompressed;		 /* # file bytes in volume - uncompressed */
 extern _s32 no_in_archive;			 /* # files in archive */
 extern _s32 no_sel;				 /* # files selected */
+extern _s32 no_exclude;				 /* # files excluded */
 extern _u32 total_selected;			 /* # bytes selected */
+extern _u32 total_excluded;			 /* # byttes unselected */
 extern _s32 no_vol_details;			 /* number of lines in vol_details */
 extern _s32 in_dir;
 extern int vols_passed;				 /* number of volumes passed on this tape */
@@ -132,15 +139,15 @@ extern char mailfn[MAX_FNAME];			 /* name of mail temp file */
 extern int lf;					 /* log file */
 extern int mf;					 /* mail file */
 extern int dv;
+extern int info_fd;				 /* info file */
+extern int info_index[INFO_INDICIES+1];
 
 /* Stipped & Common pathnames */
 extern char cf[MAX_FNAME];			 /* common pathname prefix */
-extern char stripped_cf[MAX_FNAME];		 /* portion of common pathname prefix that is not stripped */
+extern _s32 strip_cf;				 /* path user wants stripped */
 
 
 /* Info files */
-extern char *info;				 /* pointer to archive info */
-extern char *cp;				 /* offset used into above */
 extern struct tape_header tdh;			 /* headers for tape & */
 extern struct info_file_header ifd;		 /* info file */
 extern struct volume_header *vol_headers;	 /* volume headers */
@@ -149,13 +156,11 @@ extern struct tape_size_info *tsi;		 /* info about tape size */
 
 
 /* Memory allocation stuff for archive files, sel_files etc.. */
-extern struct oa_file_entry *archive_files;	 /* files on archive */
-extern _u32 len_archive_files;		 /* length of files on archive */
-extern struct oa_file_entry *sel_files;		 /* files selected by user */
-extern _u32 len_sel_files;
+extern struct selected_entry *sel_files;	 /* files selected by user */
+extern struct selected_entry *excluded_files;	 /* files unselected by user */
 extern struct volume_tape_info *vt_info;	 /* which tape each volume is on */
 extern struct dir_file_entry *dirs;		 /* restore directories */
-extern _s32 cur_af_size, cur_info_size, cur_sf_size;/* size of memory blocks */
+extern struct memory_file *m_files;		 /* representation of all files */
 
 /* Children */
 extern pid_t backup_child;
@@ -195,6 +200,8 @@ extern char color_status_foreground;
 extern char color_status_background;
 extern char color_selected_foreground;
 extern char color_selected_background;
+extern char color_excluded_foreground;
+extern char color_excluded_background;
 extern char color_directory_foreground;
 extern char color_directory_background;
 extern char color_onvol_foreground;
@@ -216,3 +223,5 @@ extern struct pref_template ap[];
 #define RESTORE_FULL 1
 #define RESTORE_VERIFY 2
 extern _s8 restore_mode;
+
+extern int f_force_local;			 /* for remote devices */
